@@ -454,6 +454,131 @@ rasqal_new_assignment_algebra_node(rasqal_query* query,
 }
 
 
+/*
+ * rasqal_new_groupby_algebra_node:
+ * @query: #rasqal_query query object
+ * @node1: inner algebra node
+ * @seq: sequence of order condition #rasqal_expression
+ *
+ * INTERNAL - Create a new GROUP algebra node for a sequence of GROUP BY conditions
+ * 
+ * #node and #seq become owned by the new node
+ *
+ * Return value: a new #rasqal_algebra_node object or NULL on failure
+ **/
+rasqal_algebra_node*
+rasqal_new_groupby_algebra_node(rasqal_query* query,
+                                rasqal_algebra_node* node1,
+                                raptor_sequence* seq)
+{
+  rasqal_algebra_node* node;
+
+  if(!query || !node1 || !seq || !raptor_sequence_size(seq))
+    goto fail;
+
+  node = rasqal_new_algebra_node(query, RASQAL_ALGEBRA_OPERATOR_GROUP);
+  if(node) {
+    node->node1 = node1;
+    node->seq = seq;
+    
+    return node;
+  }
+
+  fail:
+  if(node1)
+    rasqal_free_algebra_node(node1);
+  if(seq)
+    raptor_free_sequence(seq);
+
+  return NULL;
+}
+
+
+/*
+ * rasqal_new_aggregation_algebra_node:
+ * @query: #rasqal_query query object
+ * @node1: inner algebra node
+ * @exprs_seq: sequence of #rasqal_expression
+ * @vars_seq: sequence of #rasqal_sequence
+ *
+ * INTERNAL - Create a new AGGREGATION algebra node for a query over a sequence of expressions to variables
+ * 
+ * On construction @node1, @exprs_seq and @vars_seq become owned by
+ * the new node.
+ *
+ * Return value: a new #rasqal_algebra_node object or NULL on failure
+ **/
+rasqal_algebra_node*
+rasqal_new_aggregation_algebra_node(rasqal_query* query,
+                                    rasqal_algebra_node* node1,
+                                    raptor_sequence* exprs_seq,
+                                    raptor_sequence* vars_seq)
+{
+  rasqal_algebra_node* node;
+
+  if(!query || !node1 || !exprs_seq || !vars_seq)
+    goto fail;
+
+  node = rasqal_new_algebra_node(query, RASQAL_ALGEBRA_OPERATOR_AGGREGATION);
+  if(node) {
+    node->node1 = node1;
+    node->seq = exprs_seq;
+    node->vars_seq = vars_seq;
+    return node;
+  }
+
+  fail:
+  if(node1)
+    rasqal_free_algebra_node(node1);
+  if(exprs_seq)
+    raptor_free_sequence(exprs_seq);
+  if(vars_seq)
+    raptor_free_sequence(vars_seq);
+
+  return NULL;
+}
+
+
+/*
+ * rasqal_new_having_algebra_node:
+ * @query: #rasqal_query query object
+ * @node1: inner algebra node
+ * @exprs_seq: sequence of variables
+ *
+ * INTERNAL - Create a new HAVING algebra node for a sequence of expressions over an inner node
+ * 
+ * The inputs @node and @exprs_seq become owned by the new node
+ *
+ * Return value: a new #rasqal_algebra_node object or NULL on failure
+ **/
+rasqal_algebra_node*
+rasqal_new_having_algebra_node(rasqal_query* query,
+                               rasqal_algebra_node* node1,
+                               raptor_sequence* exprs_seq)
+{
+  rasqal_algebra_node* node;
+
+  if(!query || !node1 || !exprs_seq)
+    goto fail;
+
+  node = rasqal_new_algebra_node(query, RASQAL_ALGEBRA_OPERATOR_HAVING);
+  if(node) {
+    node->node1 = node1;
+    node->seq = exprs_seq;
+    
+    return node;
+  }
+
+  fail:
+  if(node1)
+    rasqal_free_algebra_node(node1);
+  if(exprs_seq)
+    raptor_free_sequence(exprs_seq);
+
+  return NULL;
+}
+
+
 /*
  * rasqal_free_algebra_node:
  * @gp: #rasqal_algebra_node object
@@ -511,7 +636,8 @@ rasqal_algebra_node_get_operator(rasqal_algebra_node* node)
 }
 
 
-static const char* const rasqal_algebra_node_operator_labels[RASQAL_ALGEBRA_OPERATOR_LAST+1] = {
+static const char* const
+rasqal_algebra_node_operator_labels[RASQAL_ALGEBRA_OPERATOR_LAST + 1] = {
   "UNKNOWN",
   "BGP",
   "Filter",
@@ -526,7 +652,10 @@ static const char* const rasqal_algebra_node_operator_labels[RASQAL_ALGEBRA_OPER
   "Reduced",
   "Slice",
   "Graph",
-  "Assignment"
+  "Assignment",
+  "Group",
+  "Aggregate",
+  "Having"
 };
 
 
@@ -558,7 +687,7 @@ rasqal_algebra_write_indent(raptor_iostream *iostr, int indent)
 {
   while(indent > 0) {
     int sp = (indent > SPACES_LENGTH) ? SPACES_LENGTH : indent;
-    raptor_iostream_write_bytes(iostr, spaces, sizeof(char), sp);
+    raptor_iostream_write_bytes(spaces, sizeof(char), sp, iostr);
     indent -= sp;
   }
 }
@@ -572,14 +701,14 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
   int indent_delta;
   
   if(node->op == RASQAL_ALGEBRA_OPERATOR_BGP && !node->triples) {
-    raptor_iostream_write_byte(iostr, 'Z');
+    raptor_iostream_write_byte('Z', iostr);
     return 0;
   }
   
   indent_delta = strlen(op_string);
 
-  raptor_iostream_write_counted_string(iostr, op_string, indent_delta);
-  raptor_iostream_write_counted_string(iostr, "(\n", 2);
+  raptor_iostream_counted_string_write(op_string, indent_delta, iostr);
+  raptor_iostream_counted_string_write("(\n", 2, iostr);
   indent_delta++;
   
   indent += indent_delta;
@@ -592,7 +721,7 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
       rasqal_triple *t;
       t = (rasqal_triple*)raptor_sequence_get_at(node->triples, i);
       if(arg_count) {
-        raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+        raptor_iostream_counted_string_write(" ,\n", 3, iostr);
         rasqal_algebra_write_indent(iostr, indent);
       }
       rasqal_triple_write(t, iostr);
@@ -601,14 +730,14 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
   }
   if(node->node1) {
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
     rasqal_algebra_algebra_node_write_internal(node->node1, iostr, indent);
     arg_count++;
     if(node->node2) {
       if(arg_count) {
-        raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+        raptor_iostream_counted_string_write(" ,\n", 3, iostr);
         rasqal_algebra_write_indent(iostr, indent);
       }
       rasqal_algebra_algebra_node_write_internal(node->node2, iostr, indent);
@@ -619,7 +748,7 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
   /* look for assignment var */
   if(node->var) {
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
     rasqal_variable_write(node->var, iostr);
@@ -629,7 +758,7 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
   /* look for FILTER expression */
   if(node->expr) {
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
     rasqal_expression_write(node->expr, iostr);
@@ -642,19 +771,19 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
       int i;
       
       if(arg_count) {
-        raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+        raptor_iostream_counted_string_write(" ,\n", 3, iostr);
         rasqal_algebra_write_indent(iostr, indent);
       }
-      raptor_iostream_write_counted_string(iostr, "Conditions([ ", 13);
+      raptor_iostream_counted_string_write("Conditions([ ", 13, iostr);
       for(i = 0; i < order_size; i++) {
         rasqal_expression* e;
         e = (rasqal_expression*)raptor_sequence_get_at(node->seq, i);
         if(i > 0)
-          raptor_iostream_write_counted_string(iostr, ", ", 2);
+          raptor_iostream_counted_string_write(", ", 2, iostr);
         rasqal_expression_write(e, iostr);
         arg_count++;
       }
-      raptor_iostream_write_counted_string(iostr, " ])", 3);
+      raptor_iostream_counted_string_write(" ])", 3, iostr);
     }
   }
 
@@ -663,50 +792,50 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
     int i;
       
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
-    raptor_iostream_write_counted_string(iostr, "Variables([ ", 12);
+    raptor_iostream_counted_string_write("Variables([ ", 12, iostr);
     for(i = 0; i < vars_size; i++) {
       rasqal_variable* v;
       v = (rasqal_variable*)raptor_sequence_get_at(node->vars_seq, i);
       if(i > 0)
-        raptor_iostream_write_counted_string(iostr, ", ", 2);
+        raptor_iostream_counted_string_write(", ", 2, iostr);
       rasqal_variable_write(v, iostr);
       arg_count++;
     }
-    raptor_iostream_write_counted_string(iostr, " ])", 3);
+    raptor_iostream_counted_string_write(" ])", 3, iostr);
   }
 
   if(node->op == RASQAL_ALGEBRA_OPERATOR_SLICE) {
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
-    raptor_iostream_write_string(iostr, "slice start ");
-    raptor_iostream_write_decimal(iostr, node->start);
-    raptor_iostream_write_string(iostr, " length ");
-    raptor_iostream_write_decimal(iostr, node->length);
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_string_write("slice start ", iostr);
+    raptor_iostream_decimal_write(node->start, iostr);
+    raptor_iostream_string_write(" length ", iostr);
+    raptor_iostream_decimal_write(node->length, iostr);
+    raptor_iostream_write_byte('\n', iostr);
     arg_count++;
   }
 
   if(node->op == RASQAL_ALGEBRA_OPERATOR_GRAPH) {
     if(arg_count) {
-      raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+      raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
-    raptor_iostream_write_string(iostr, "origin ");
+    raptor_iostream_string_write("origin ", iostr);
     rasqal_literal_write(node->graph, iostr);
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_write_byte('\n', iostr);
     arg_count++;
   }
 
-  raptor_iostream_write_byte(iostr, '\n');
+  raptor_iostream_write_byte('\n', iostr);
   indent-= indent_delta;
 
   rasqal_algebra_write_indent(iostr, indent);
-  raptor_iostream_write_byte(iostr, ')');
+  raptor_iostream_write_byte(')', iostr);
 
   return 0;
 }
@@ -736,11 +865,7 @@ rasqal_algebra_node_print(rasqal_algebra_node* node, FILE* fh)
 {
   raptor_iostream* iostr;
 
-#ifdef RAPTOR_V2_AVAILABLE
   iostr = raptor_new_iostream_to_file_handle(node->query->world->raptor_world_ptr, fh);
-#else
-  iostr = raptor_new_iostream_to_file_handle(fh);
-#endif
   rasqal_algebra_algebra_node_write(node, iostr);
   raptor_free_iostream(iostr);
 
@@ -1088,6 +1213,17 @@ rasqal_algebra_let_graph_pattern_to_algebra(rasqal_query* query,
 }
 
 
+static rasqal_algebra_node*
+rasqal_algebra_select_graph_pattern_to_algebra(rasqal_query* query,
+                                               rasqal_graph_pattern* gp)
+{
+  rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
+                          NULL, "Sub SELECT execution is not implemented");
+
+  return NULL;
+}
+
+
 static rasqal_algebra_node*
 rasqal_algebra_graph_pattern_to_algebra(rasqal_query* query,
                                         rasqal_graph_pattern* gp)
@@ -1116,7 +1252,13 @@ rasqal_algebra_graph_pattern_to_algebra(rasqal_query* query,
       node = rasqal_algebra_let_graph_pattern_to_algebra(query, gp);
       break;
 
+    case RASQAL_GRAPH_PATTERN_OPERATOR_SELECT:
+      node = rasqal_algebra_select_graph_pattern_to_algebra(query, gp);
+      break;
+
     case RASQAL_GRAPH_PATTERN_OPERATOR_FILTER:
+    case RASQAL_GRAPH_PATTERN_OPERATOR_SERVICE:
+    case RASQAL_GRAPH_PATTERN_OPERATOR_MINUS:
 
     case RASQAL_GRAPH_PATTERN_OPERATOR_UNKNOWN:
     default:
@@ -1247,6 +1389,262 @@ rasqal_algebra_remove_znodes(rasqal_query* query, rasqal_algebra_node* node,
 }
 
 
+static raptor_sequence*
+rasqal_algebra_get_variables_mentioned_in(rasqal_query* query,
+                                          int row_index)
+{
+  raptor_sequence* seq; /* sequence of rasqal_variable* */
+  int width;
+  short *row;
+  int i;
+  
+#ifdef HAVE_RAPTOR2_API
+  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                            (raptor_data_print_handler)rasqal_variable_print);
+#else
+  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
+                            (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
+  if(!seq)
+    return NULL;
+
+  width = rasqal_variables_table_get_total_variables_count(query->vars_table);
+  row = &query->variables_use_map[row_index * width];
+
+  for(i = 0; i < width; i++) {
+    rasqal_variable* v;
+
+    if(!(row[i] & RASQAL_VAR_USE_MENTIONED_HERE))
+      continue;
+
+    v = rasqal_variables_table_get(query->vars_table, i);
+    raptor_sequence_push(seq, rasqal_new_variable_from_variable(v));
+  }
+
+  return seq;
+}
+
+
+/**
+ * rasqal_agg_expr_var_compare:
+ * @user_data: comparison user data pointer
+ * @a: pointer to address of first #rasqal_expression
+ * @b: pointer to address of second #rasqal_expression
+ *
+ * INTERNAL - compare two void pointers to #rasqal_expression objects with signature suitable for for #rasqal_map comparison.
+ *
+ * Return value: <0, 0 or >1 comparison
+ */
+static int
+rasqal_agg_expr_var_compare(void* user_data, const void *a, const void *b)
+{
+  rasqal_algebra_aggregate* ae = (rasqal_algebra_aggregate*)user_data;
+  rasqal_expression* expr_a  = (rasqal_expression*)a;
+  rasqal_expression* expr_b  = (rasqal_expression*)b;
+  int result = 0;
+
+  result = rasqal_expression_compare(expr_a, expr_b, ae->flags, &ae->error);
+
+  return result;
+}
+
+
+/*
+ * SPEC:
+ *   at each node:
+ *     if expression contains an aggregate function
+ *       is expression is in map?
+ *       Yes:
+ *         get value => use existing internal variable for it
+ *       No: 
+ *         create a new internal variable for it $$internal{id}
+ *         inc id
+ *         add it to the map
+ *       rewrite expression to use internal variable
+ *
+ */
+static int
+rasqal_algebra_extract_aggregate_expression_visit(void *user_data,
+                                                  rasqal_expression *e)
+{
+  rasqal_algebra_aggregate* ae = (rasqal_algebra_aggregate*)user_data;
+  rasqal_variable* v;
+
+  ae->error = 0;
+
+  /* If not an aggregate expression, ignore it */
+  if(!rasqal_expression_is_aggregate(e))
+    return 0;
+
+
+  /* is expression is in map? */
+  v = rasqal_map_search(ae->agg_vars, e);
+  if(v) {
+    /* Yes: get value => use existing internal variable for it */
+    RASQAL_DEBUG2("Found variable %s for existing expression\n", v->name);
+
+    /* convert expression in-situ to use existing internal variable */
+    if(rasqal_expression_convert_aggregate_to_variable(e, v, NULL)) {
+      ae->error = 1;
+      return 1;
+    }
+
+
+  } else {
+    /* No */
+    char* var_name;
+    rasqal_expression* new_e = NULL;
+        
+    /* Check if a new variable is allowed to be added */
+    if(ae->adding_new_vars_is_error) {
+      rasqal_log_error_simple(ae->query->world, RAPTOR_LOG_LEVEL_ERROR,
+                              NULL, "Found new aggregate expression in %s",
+                              ae->error_part);
+      ae->error = 1;
+      return 1;
+    }
+    
+    /* If not an error, create a new internal variable name for it
+     * $$agg{id}$$ and add it to the map.
+     */
+    var_name = (char*)RASQAL_MALLOC(cstring, 20);
+    if(!var_name) {
+      ae->error = 1;
+      return 1;
+    }
+    
+    sprintf(var_name, "$$agg$$%d", ae->counter++);
+
+    v = rasqal_variables_table_add(ae->query->vars_table, 
+                                   RASQAL_VARIABLE_TYPE_ANONYMOUS, 
+                                   (const unsigned char*)var_name, NULL);
+    if(!v) {
+      ae->error = 1;
+      return 1;
+    }
+
+    /* convert expression in-situ to use new internal variable
+     * and create a new expression in new_e from the old fields
+     */
+    if(rasqal_expression_convert_aggregate_to_variable(e, v, &new_e)) {
+      ae->error = 1;
+      return 1;
+    }
+
+    /* new_e is a new reference and after this new_e is owned by map 
+     * (all variables are shared and owned by query object)
+     */
+    if(rasqal_map_add_kv(ae->agg_vars, new_e, v)) {
+      ae->error = 1;
+      return 1;
+    }
+
+#ifdef RASQAL_DEBUG
+    RASQAL_DEBUG1("after adding new variable, resulting aggregate vars ");
+    rasqal_map_print(ae->agg_vars, stderr);
+    fputc('\n', stderr);
+#endif
+
+    new_e = rasqal_new_expression_from_expression(new_e);
+    if(raptor_sequence_push(ae->agg_exprs, new_e)) {
+      ae->error = 1;
+      return 1;
+    }
+
+    /* add to variables sequence too */
+    v = rasqal_new_variable_from_variable(v);
+    if(raptor_sequence_push(ae->agg_vars_seq, v)) {
+      ae->error = 1;
+      return 1;
+    }
+
+  }
+
+
+  return 0;
+}
+
+
+static int
+rasqal_algebra_extract_aggregate_expressions(rasqal_query* query,
+                                             rasqal_algebra_node* node,
+                                             rasqal_algebra_aggregate* ae)
+{
+  raptor_sequence* seq;
+  int i;
+  int rc = 0;
+  rasqal_variable* v;
+  
+  if(query->verb != RASQAL_QUERY_VERB_SELECT)
+    return 0;
+
+  ae->query = query;
+
+  /* Initialisation of map (key: rasqal_expression, value: rasqal_variable ) */
+  ae->agg_vars = rasqal_new_map(/* compare key function */ rasqal_agg_expr_var_compare,
+                                /* compare data */ ae,
+                                /* free compare data */ NULL,
+                                (raptor_data_free_handler)rasqal_free_expression,
+                                (raptor_data_free_handler)rasqal_free_variable,
+                                (raptor_data_print_handler)rasqal_expression_print,
+                                (raptor_data_print_handler)rasqal_variable_print,
+                                /* flags */ 0);
+
+  /* Sequence to hold list of aggregate expressions */
+#ifdef HAVE_RAPTOR2_API
+  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_expression,
+                            (raptor_data_print_handler)rasqal_expression_print);
+#else
+  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_expression,
+                            (raptor_sequence_print_handler*)rasqal_expression_print);
+#endif
+  ae->agg_exprs = seq;
+
+#ifdef HAVE_RAPTOR2_API
+  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                            (raptor_data_print_handler)rasqal_variable_print);
+#else
+  seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
+                            (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
+  ae->agg_vars_seq = seq;
+
+  /* init internal variable counter */
+  ae->counter = 0;
+
+  ae->flags = 0;
+  
+  ae->error = 0;
+
+  /*
+   * walk each select/project expression recursively and pull out aggregate
+   * expressions into the ae->agg_vars map, replacing them with
+   * internal variable names.
+   */
+  for(seq = query->selects, i = 0;
+      (v = (rasqal_variable*)raptor_sequence_get_at(seq, i));
+      i++) {
+    rasqal_expression* expr = v->expression;
+
+    if(!expr)
+      continue;
+
+    if(rasqal_expression_visit(expr,
+                               rasqal_algebra_extract_aggregate_expression_visit,
+                               ae)) {
+      rc = 1;
+      goto tidy;
+    }
+  }
+
+  tidy:
+  if(ae->error)
+    rc = 1;
+  
+  return rc;
+}
+
+
 /**
  * rasqal_algebra_query_to_algebra:
  * @query: #rasqal_query to operate on
@@ -1261,7 +1659,7 @@ rasqal_algebra_query_to_algebra(rasqal_query* query)
   rasqal_graph_pattern* query_gp;
   rasqal_algebra_node* node;
   int modified = 0;
-
+  
   query_gp = rasqal_query_get_query_graph_pattern(query);
   if(!query_gp)
     return NULL;
@@ -1271,7 +1669,6 @@ rasqal_algebra_query_to_algebra(rasqal_query* query)
   if(!node)
     return NULL;
 
-
   rasqal_algebra_node_visit(query, node, 
                             rasqal_algebra_remove_znodes,
                             &modified);
@@ -1282,96 +1679,430 @@ rasqal_algebra_query_to_algebra(rasqal_query* query)
   fputs("\n", stderr);
 #endif
 
-  if(query->order_conditions_sequence) {
-    int order_size = raptor_sequence_size(query->order_conditions_sequence);
-    if(order_size) {
-      int i;
-      raptor_sequence* seq;
 
-      /* Make a deep copy of the query order conditions sequence for
-       * the ORDERBY algebra node
-       */
-#ifdef RAPTOR_V2_AVAILABLE
-      seq = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_expression, (raptor_data_print_handler*)rasqal_expression_print);
-#else
-      seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_expression, (raptor_sequence_print_handler*)rasqal_expression_print);
-#endif
-      if(!seq) {
-        rasqal_free_algebra_node(node);
-        return NULL;
-      }
-      for(i = 0; i < order_size; i++) {
-        rasqal_expression* e;
-        e = (rasqal_expression*)raptor_sequence_get_at(query->order_conditions_sequence, i);
-        raptor_sequence_push(seq, rasqal_new_expression_from_expression(e));
-      }
+  return node;
+}
 
-      node = rasqal_new_orderby_algebra_node(query, node, seq);
-      modified = 1;
+
+void
+rasqal_free_algebra_aggregate(rasqal_algebra_aggregate* ae)
+{
+  if(!ae)
+    return;
+  
+  if(ae->agg_exprs)
+    raptor_free_sequence(ae->agg_exprs);
+  
+  if(ae->agg_vars)
+    rasqal_free_map(ae->agg_vars);
+
+  if(ae->agg_vars_seq)
+    raptor_free_sequence(ae->agg_vars_seq);
+  
+  RASQAL_FREE(rasqal_algebra_aggregate, ae);
+}
+
+  
+static int
+rasqal_algebra_replace_aggregate_expressions(rasqal_query* query,
+                                             raptor_sequence* exprs_seq,
+                                             rasqal_algebra_aggregate* ae)
+{
+  int i;
+  rasqal_expression* expr;
+
+  /* It is now a mistake to find a new aggregate expressions not
+   * previously found in SELECT
+   */
+  ae->adding_new_vars_is_error = 1;
+  ae->error_part = "HAVING";
+  
+  for(i = 0;
+      (expr = (rasqal_expression*)raptor_sequence_get_at(exprs_seq, i));
+      i++) {
+    
+    if(rasqal_expression_visit(expr,
+                               rasqal_algebra_extract_aggregate_expression_visit,
+                               ae)) {
+      return 1;
     }
+    
+  }
 
+  return 0;
+}
+
+
+/**
+ * rasqal_algebra_query_prepare_aggregates:
+ * @query: #rasqal_query to read from
+ * @node: algebra node to prepare
+ *
+ * INTERNAL - prepare query aggregates
+ *
+ * Return value: aggregate expression data or NULL on failure
+ */
+rasqal_algebra_aggregate*
+rasqal_algebra_query_prepare_aggregates(rasqal_query* query,
+                                        rasqal_algebra_node* node)
+{
+  int modified = 0;
+  rasqal_algebra_aggregate* ae;
+  
+  ae = (rasqal_algebra_aggregate*)RASQAL_CALLOC(rasqal_algebra_aggregate,
+                                                sizeof(*ae), 1);
+  if(!ae)
+    return NULL;
+
+  if(rasqal_algebra_extract_aggregate_expressions(query, node, ae)) {
+    RASQAL_DEBUG1("rasqal_algebra_extract_aggregate_expressions() failed");
+    rasqal_free_algebra_aggregate(ae);
+    rasqal_free_algebra_node(node);
+    return NULL;
+  }
+
+  /* Update variable use structures since agg variables were created */
+  if(ae->counter)
+    rasqal_query_build_variables_use(query);
+  
+#if RASQAL_DEBUG
+  if(ae->counter) {
+    if(query->selects) {
+      RASQAL_DEBUG1("after aggregate expressions extracted:\n");
+      raptor_sequence_print(query->selects, stderr);
+      fputs("\n", stderr);
+
+      RASQAL_DEBUG1("aggregate expressions:\n");
+      raptor_sequence_print(ae->agg_exprs, stderr);
+      fputs("\n", stderr);
+    }
+  } else {
+    RASQAL_DEBUG1("found no aggregate expressions in select\n");
+  }
+#endif
+
+
+  /* if agg expressions were found, need to walk HAVING list and do a
+   * similar replacement substituion in the expressions
+   */
+  if(ae->counter && query->modifier && query->modifier->having_conditions) {
+    if(rasqal_algebra_replace_aggregate_expressions(query, 
+                                                    query->modifier->having_conditions,
+                                                    ae)) {
+      RASQAL_DEBUG1("rasqal_algebra_replace_aggregate_expressions() failed");
+      rasqal_free_algebra_aggregate(ae);
+      rasqal_free_algebra_node(node);
+      return NULL;
+    }
+  }
+  
+  modified = 0;
+  
+  return ae;
+}
+
+
+/**
+ * rasqal_algebra_query_add_group_by:
+ * @query: #rasqal_query to read from
+ * @node: node to apply modifiers to
+ *
+ * Apply any needed GROUP BY to query algebra structure
+ *
+ * Return value: non-0 on failure
+ */
+rasqal_algebra_node*
+rasqal_algebra_query_add_group_by(rasqal_query* query,
+                                  rasqal_algebra_node* node)
+{
+  raptor_sequence* modifier_seq;
+  int modified;
+  
+  if(!query->modifier)
+    return node;
+  
+  /* GROUP BY */
+  modifier_seq = query->modifier->group_conditions;
+
+  if(modifier_seq) {
+    raptor_sequence* seq;
+    
+    /* Make a deep copy of the query group conditions sequence for
+     * the GROUP algebra node
+     */
+    seq = rasqal_expression_copy_expression_sequence(modifier_seq);
+    if(!seq) {
+      rasqal_free_algebra_node(node);
+      return NULL;
+    }
+    
+    node = rasqal_new_groupby_algebra_node(query, node, seq);
+    modified = 1;
+    
+    
 #if RASQAL_DEBUG > 1
-  RASQAL_DEBUG2("modified=%d after adding orderby node, algebra node now:\n  ", modified);
+    RASQAL_DEBUG2("modified=%d after adding group node, algebra node now:\n  ", modified);
+    rasqal_algebra_node_print(node, stderr);
+    fputs("\n", stderr);
+#endif
+  }
+  
+  return node;
+}
+  
+
+/**
+ * rasqal_algebra_query_add_modifiers:
+ * @query: #rasqal_query to read from
+ * @node: node to apply modifiers to
+ *
+ * Apply any needed modifiers to query algebra structure
+ *
+ * Return value: non-0 on failure
+ */
+rasqal_algebra_node*
+rasqal_algebra_query_add_modifiers(rasqal_query* query,
+                                   rasqal_algebra_node* node)
+{
+  raptor_sequence* modifier_seq;
+  int modified;
+  
+  if(!query->modifier)
+    return node;
+  
+  /* ORDER BY */
+  modifier_seq = query->modifier->order_conditions;
+  if(modifier_seq) {
+    raptor_sequence* seq;
+    
+    /* Make a deep copy of the query order conditions sequence for
+     * the ORDERBY algebra node
+     */
+    seq = rasqal_expression_copy_expression_sequence(modifier_seq);
+    if(!seq) {
+      rasqal_free_algebra_node(node);
+      return NULL;
+    }
+    
+    node = rasqal_new_orderby_algebra_node(query, node, seq);
+    modified = 1;
+    
+#if RASQAL_DEBUG > 1
+    RASQAL_DEBUG2("modified=%d after adding orderby node, algebra node now:\n  ", modified);
+    rasqal_algebra_node_print(node, stderr);
+    fputs("\n", stderr);
+#endif
+  }
+
+  return node;
+}
+
+
+/**
+ * rasqal_algebra_query_add_aggregation:
+ * @query: #rasqal_query to read from
+ * @ae: aggregation structure
+ * @node: node to apply aggregation to
+ *
+ * Apply any aggregation step needed to query algebra structure
+ *
+ * Becomes the owner of @ae
+ *
+ * Return value: non-0 on failure
+ */
+rasqal_algebra_node*
+rasqal_algebra_query_add_aggregation(rasqal_query* query,
+                                     rasqal_algebra_aggregate* ae,
+                                     rasqal_algebra_node* node)
+{
+  int modified = 0;
+  raptor_sequence* exprs_seq;
+  raptor_sequence* vars_seq;
+  
+  if(!query || !ae || !node)
+    goto tidy;
+  
+  if(!ae->counter)
+    return node;
+
+  /* Move ownership of sequences inside ae to here */
+  exprs_seq = ae->agg_exprs; ae->agg_exprs = NULL;
+  vars_seq = ae->agg_vars_seq; ae->agg_vars_seq = NULL;
+
+  rasqal_free_algebra_aggregate(ae);
+  
+  node = rasqal_new_aggregation_algebra_node(query, node, exprs_seq, vars_seq);
+  exprs_seq = NULL; vars_seq = NULL;
+  if(!node) {
+    RASQAL_DEBUG1("rasqal_new_aggregation_algebra_node() failed");
+    goto tidy;
+  }
+  
+  modified = 1;
+  
+#if RASQAL_DEBUG > 1
+  RASQAL_DEBUG2("modified=%d after adding aggregation node, algebra node now:\n  ", modified);
   rasqal_algebra_node_print(node, stderr);
   fputs("\n", stderr);
 #endif
 
-  }
+  return node;
 
 
+  tidy:
+  if(ae)
+    rasqal_free_algebra_aggregate(ae);
+  if(node)
+    rasqal_free_algebra_node(node);
+
+  return NULL;
+}
+
+
+/**
+ * rasqal_algebra_query_add_projection:
+ * @query: #rasqal_query to read from
+ * @node: node to apply projection to
+ *
+ * Apply query projection to query algebra structure
+ *
+ * Return value: non-0 on failure
+ */
+rasqal_algebra_node*
+rasqal_algebra_query_add_projection(rasqal_query* query,
+                                   rasqal_algebra_node* node)
+{
+  int modified = 0;
+  int vars_size = 0;
+  raptor_sequence* seq = NULL;  /* sequence of rasqal_variable* */
+  raptor_sequence* vars_seq;
+  int i;
+  
   /* FIXME - do not always need a PROJECT node when the variables at
    * the top level node are the same as the projection list.
    */
-  if(1) {
-    int vars_size = 0;
-    raptor_sequence* seq = NULL;
-    raptor_sequence* vars_seq;
-    int i;
-
-    if(query->verb == RASQAL_QUERY_VERB_SELECT ||
-       query->verb == RASQAL_QUERY_VERB_CONSTRUCT)
-      /* sequence of rasqal_variable* */
-      seq = query->selects;
-
-    if(seq)
-      vars_size = raptor_sequence_size(seq);
-    
-#ifdef RAPTOR_V2_AVAILABLE
-    vars_seq = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_variable,
-                                   (raptor_data_print_handler*)rasqal_variable_print);
-#else
-    vars_seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
-                                   (raptor_sequence_print_handler*)rasqal_variable_print);
-#endif
-    if(!vars_seq) {
+  if(query->verb == RASQAL_QUERY_VERB_SELECT)
+    /* project all selected variables */
+    seq = query->selects;
+  else if(query->verb == RASQAL_QUERY_VERB_CONSTRUCT) {
+    /* project all variables mentioned in CONSTRUCT  */
+    seq = rasqal_algebra_get_variables_mentioned_in(query, 
+                                                    RASQAL_VAR_USE_MAP_OFFSET_VERBS);
+    if(!seq) {
       rasqal_free_algebra_node(node);
       return NULL;
     }
-    for(i = 0; i < vars_size; i++) {
-      rasqal_variable* v;
-      v = (rasqal_variable*)raptor_sequence_get_at(seq, i);
-      raptor_sequence_push(vars_seq, rasqal_new_variable_from_variable(v));
-    }
-
-    node = rasqal_new_project_algebra_node(query, node, vars_seq);
-    modified = 1;
+  }
+  
+  if(seq)
+    vars_size = raptor_sequence_size(seq);
+  
+#ifdef HAVE_RAPTOR2_API
+  vars_seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                 (raptor_data_print_handler)rasqal_variable_print);
+#else
+  vars_seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
+                                 (raptor_sequence_print_handler*)rasqal_variable_print);
+#endif
+  if(!vars_seq) {
+    rasqal_free_algebra_node(node);
+    return NULL;
+  }
+  
+  for(i = 0; i < vars_size; i++) {
+    rasqal_variable* v;
 
+    v = (rasqal_variable*)raptor_sequence_get_at(seq, i);
+    v = rasqal_new_variable_from_variable(v);
+    raptor_sequence_push(vars_seq, v);
+  }
+  
+  node = rasqal_new_project_algebra_node(query, node, vars_seq);
+  modified = 1;
+  
 #if RASQAL_DEBUG > 1
   RASQAL_DEBUG2("modified=%d after adding project node, algebra node now:\n  ", modified);
   rasqal_algebra_node_print(node, stderr);
   fputs("\n", stderr);
 #endif
-  }
+  
+  if(query->verb == RASQAL_QUERY_VERB_CONSTRUCT && seq) 
+    raptor_free_sequence(seq);
+
+  return node;
+}
 
 
+/**
+ * rasqal_algebra_query_add_distinct:
+ * @query: #rasqal_query to read from
+ * @node: node to apply distinct to
+ *
+ * Apply distinctness to query algebra structure
+ *
+ * Return value: non-0 on failure
+ */
+rasqal_algebra_node*
+rasqal_algebra_query_add_distinct(rasqal_query* query,
+                                  rasqal_algebra_node* node)
+{
+  int modified;
+  
   if(query->distinct) {
     node = rasqal_new_distinct_algebra_node(query, node);
     modified = 1;
 
 #if RASQAL_DEBUG > 1
-  RASQAL_DEBUG2("modified=%d after adding distinct node, algebra node now:\n  ", modified);
-  rasqal_algebra_node_print(node, stderr);
-  fputs("\n", stderr);
+    RASQAL_DEBUG2("modified=%d after adding distinct node, algebra node now:\n  ", modified);
+    rasqal_algebra_node_print(node, stderr);
+    fputs("\n", stderr);
+#endif
+  }
+
+  return node;
+}
+
+
+/**
+ * rasqal_algebra_query_add_having:
+ * @query: #rasqal_query to read from
+ * @node: node to apply having to
+ *
+ * Apply any needed HAVING expressions to query algebra structure
+ *
+ * Return value: non-0 on failure
+ */
+rasqal_algebra_node*
+rasqal_algebra_query_add_having(rasqal_query* query,
+                                rasqal_algebra_node* node)
+{
+  raptor_sequence* modifier_seq;
+  int modified;
+  
+  if(!query->modifier)
+    return node;
+  
+  /* HAVING */
+  modifier_seq = query->modifier->having_conditions;
+  if(modifier_seq) {
+    raptor_sequence* exprs_seq;
+    
+    /* Make a deep copy of the query order conditions sequence for
+     * the ORDERBY algebra node
+     */
+    exprs_seq = rasqal_expression_copy_expression_sequence(modifier_seq);
+    if(!exprs_seq) {
+      rasqal_free_algebra_node(node);
+      return NULL;
+    }
+    
+    node = rasqal_new_having_algebra_node(query, node, exprs_seq);
+    modified = 1;
+    
+#if RASQAL_DEBUG > 1
+    RASQAL_DEBUG2("modified=%d after adding having node, algebra node now:\n  ", modified);
+    rasqal_algebra_node_print(node, stderr);
+    fputs("\n", stderr);
 #endif
   }
 
@@ -1437,11 +2168,7 @@ main(int argc, char *argv[]) {
   uri_string = raptor_uri_filename_to_uri_string("");
   if(!uri_string)
     FAIL;
-#ifdef RAPTOR_V2_AVAILABLE
   base_uri = raptor_new_uri(world->raptor_world_ptr, uri_string);
-#else
-  base_uri = raptor_new_uri(uri_string);
-#endif
   if(!base_uri)
     FAIL;
   raptor_free_memory(uri_string);
@@ -1612,9 +2339,9 @@ main(int argc, char *argv[]) {
   /* This is an artificial order conditions sequence equivalent to
    * ORDER BY 1, 2 which would probably never appear in a query.
    */
-#ifdef RAPTOR_V2_AVAILABLE
-  conditions = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_expression,
-                                   (raptor_data_print_handler*)rasqal_expression_print);
+#ifdef HAVE_RAPTOR2_API
+  conditions = raptor_new_sequence((raptor_data_free_handler)rasqal_free_expression,
+                                   (raptor_data_print_handler)rasqal_expression_print);
 #else
   conditions = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_expression,
                                    (raptor_sequence_print_handler*)rasqal_expression_print);
