@@ -624,8 +624,8 @@ rasqal_engine_remove_filter_graph_patterns(rasqal_query* query,
   }
   
   
-#ifdef RAPTOR_V2_AVAILABLE
-  seq = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_graph_pattern, (raptor_data_print_handler*)rasqal_graph_pattern_print);
+#ifdef HAVE_RAPTOR2_API
+  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_graph_pattern, (raptor_data_print_handler)rasqal_graph_pattern_print);
 #else
   seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
 #endif
@@ -1428,21 +1428,24 @@ rasqal_engine_make_rowsource(rasqal_query* query,
   con->need_store_results = need_store_results;
 
   if(con->need_store_results) {
+    raptor_sequence *order_seq;
+    order_seq = rasqal_query_get_order_conditions_sequence(query);
+    
     /* make a row:NULL map in order to sort or do distinct */
     con->map = rasqal_engine_new_rowsort_map(query->distinct,
                                              query->compare_flags,
-                                             query->order_conditions_sequence);
+                                             order_seq);
     if(!con->map) {
       rasqal_rowsource_engine_finish(NULL, con);
       return NULL;
     }
 
-    if(query->order_conditions_sequence)
-      con->order_size = raptor_sequence_size(query->order_conditions_sequence);
+    if(order_seq)
+      con->order_size = raptor_sequence_size(order_seq);
   }
   
-#ifdef RAPTOR_V2_AVAILABLE
-  con->seq = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_row, (raptor_data_print_handler*)rasqal_row_print);
+#ifdef HAVE_RAPTOR2_API
+  con->seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row, (raptor_data_print_handler)rasqal_row_print);
 #else
   con->seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row, (raptor_sequence_print_handler*)rasqal_row_print);
 #endif
@@ -1806,8 +1809,8 @@ rasqal_query_engine_1_execute_init(void* ex_data,
     }
   }
 
-#ifdef RAPTOR_V2_AVAILABLE
-  execution_data->seq = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_gp_data, NULL);
+#ifdef HAVE_RAPTOR2_API
+  execution_data->seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_gp_data, NULL);
 #else
   execution_data->seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_gp_data, NULL);
 #endif
