@@ -81,24 +81,11 @@ rasqal_algebra_basic_algebra_node_to_rowsource(rasqal_engine_algebra_data* execu
                                                rasqal_engine_error *error_p)
 {
   rasqal_query *query = execution_data->query;
-  int size;
-  int *bound_in;
-  int i;
   
-  size = rasqal_variables_table_get_total_variables_count(query->vars_table);
-  bound_in = (int*)RASQAL_CALLOC(intarray, size+1, sizeof(int));
-  if(!bound_in)
-    return NULL;
-
-  /* bound_in becomes owned by the new rowsource */
-  for(i = 0; i < size; i++)
-    bound_in[i] = query->variables_bound_in[i];
-
   return rasqal_new_triples_rowsource(query->world, query,
                                       execution_data->triples_source,
                                       node->triples,
-                                      node->start_column, node->end_column,
-                                      bound_in, size);
+                                      node->start_column, node->end_column);
 }
 
 
@@ -337,7 +324,8 @@ eval(D(G), Graph(IRI,P)) = the empty multiset
     } else {
       /* case #2 - IRI is not a graph name in D - return empty rowsource */
       rasqal_free_algebra_node(node->node1);
-
+      node->node1 = NULL;
+      
       rs = rasqal_new_empty_rowsource(query->world, query);
     }
 
@@ -372,6 +360,56 @@ rasqal_algebra_distinct_algebra_node_to_rowsource(rasqal_engine_algebra_data* ex
 }
 
 
+static rasqal_rowsource*
+rasqal_algebra_group_algebra_node_to_rowsource(rasqal_engine_algebra_data* execution_data,
+                                               rasqal_algebra_node* node,
+                                               rasqal_engine_error *error_p)
+{
+  rasqal_query *query = execution_data->query;
+  rasqal_rowsource *rs;
+
+  rs = rasqal_algebra_node_to_rowsource(execution_data, node->node1, error_p);
+  if(!rs || *error_p)
+    return NULL;
+
+  return rasqal_new_groupby_rowsource(query->world, query, rs, node->seq);
+}
+
+
+static rasqal_rowsource*
+rasqal_algebra_aggregation_algebra_node_to_rowsource(rasqal_engine_algebra_data* execution_data,
+                                                     rasqal_algebra_node* node,
+                                                     rasqal_engine_error *error_p)
+{
+  rasqal_query *query = execution_data->query;
+  rasqal_rowsource *rs;
+
+  rs = rasqal_algebra_node_to_rowsource(execution_data, node->node1, error_p);
+  if(!rs || *error_p)
+    return NULL;
+
+  return rasqal_new_aggregation_rowsource(query->world, query, rs,
+                                          node->seq,
+                                          node->vars_seq);
+}
+
+
+static rasqal_rowsource*
+rasqal_algebra_having_algebra_node_to_rowsource(rasqal_engine_algebra_data* execution_data,
+                                                rasqal_algebra_node* node,
+                                                rasqal_engine_error *error_p)
+{
+  rasqal_query *query = execution_data->query;
+  rasqal_rowsource *rs;
+
+  rs = rasqal_algebra_node_to_rowsource(execution_data, node->node1, error_p);
+  if(!rs || *error_p)
+    return NULL;
+
+  return rasqal_new_having_rowsource(query->world, query, rs, node->seq);
+}
+
+
 static rasqal_rowsource*
 rasqal_algebra_node_to_rowsource(rasqal_engine_algebra_data* execution_data,
                                  rasqal_algebra_node* node,
@@ -430,6 +468,22 @@ rasqal_algebra_node_to_rowsource(rasqal_engine_algebra_data* execution_data,
                                                                node, error_p);
       break;
 
+    case RASQAL_ALGEBRA_OPERATOR_GROUP:
+      rs = rasqal_algebra_group_algebra_node_to_rowsource(execution_data,
+                                                          node, error_p);
+      break;
+
+    case RASQAL_ALGEBRA_OPERATOR_AGGREGATION:
+      rs = rasqal_algebra_aggregation_algebra_node_to_rowsource(execution_data,
+                                                                node, error_p);
+      break;
+
+    case RASQAL_ALGEBRA_OPERATOR_HAVING:
+      rs = rasqal_algebra_having_algebra_node_to_rowsource(execution_data,
+                                                           node, error_p);
+      break;
+
+
     case RASQAL_ALGEBRA_OPERATOR_UNKNOWN:
     case RASQAL_ALGEBRA_OPERATOR_DIFF:
     case RASQAL_ALGEBRA_OPERATOR_TOLIST:
@@ -459,6 +513,8 @@ rasqal_query_engine_algebra_execute_init(void* ex_data,
   rasqal_engine_algebra_data* execution_data;
   rasqal_engine_error error;
   int rc = 0;
+  rasqal_algebra_node* node;
+  rasqal_algebra_aggregate* ae;
   
   execution_data = (rasqal_engine_algebra_data*)ex_data;
 
@@ -474,25 +530,62 @@ rasqal_query_engine_algebra_execute_init(void* ex_data,
     }
   }
 
-  execution_data->algebra_node = rasqal_algebra_query_to_algebra(query);
-  if(!execution_data->algebra_node)
+  node = rasqal_algebra_query_to_algebra(query);
+  if(!node)
     return 1;
 
+  node = rasqal_algebra_query_add_group_by(query, node);
+  if(!node)
+    return 1;
+
+  ae = rasqal_algebra_query_prepare_aggregates(query, node);
+  if(!ae)
+    return 1;
+
+  if(ae) {
+    node = rasqal_algebra_query_add_aggregation(query, ae, node);
+    ae = NULL;
+    if(!node)
+      return 1;
+  }
+
+  node = rasqal_algebra_query_add_having(query, node);
+  if(!node)
+    return 1;
+
+  node = rasqal_algebra_query_add_projection(query, node);
+  if(!node)
+    return 1;
+
+  node = rasqal_algebra_query_add_modifiers(query, node);
+  if(!node)
+    return 1;
+
+  node = rasqal_algebra_query_add_distinct(query,node);
+  if(!node)
+    return 1;
+
+
+  execution_data->algebra_node = node;
+
+  /* count final number of nodes */
   execution_data->nodes_count = 0; 
-  rasqal_algebra_node_visit(query, execution_data->algebra_node,
+  rasqal_algebra_node_visit(query,
+                            execution_data->algebra_node,
                             rasqal_engine_algebra_count_nodes,
                             &execution_data->nodes_count);
   
+  
 #ifdef RASQAL_DEBUG
   RASQAL_DEBUG1("algebra result: \n");
-  rasqal_algebra_node_print(execution_data->algebra_node, DEBUG_FH);
+  rasqal_algebra_node_print(node, DEBUG_FH);
   fputc('\n', DEBUG_FH);
 #endif
   RASQAL_DEBUG2("algebra nodes: %d\n", execution_data->nodes_count);
 
   error = RASQAL_ENGINE_OK;
   execution_data->rowsource = rasqal_algebra_node_to_rowsource(execution_data,
-                                                               execution_data->algebra_node,
+                                                               node,
                                                                &error);
 #ifdef RASQAL_DEBUG
   RASQAL_DEBUG1("rowsource (query plan) result: \n");
