@@ -84,8 +84,8 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
                                       raptor_uri *base_uri)
 {
   int rc=1;
-  rasqal_query* query = rasqal_query_results_get_query(results);
-#ifndef RAPTOR_V2_AVAILABLE
+  rasqal_world* world = rasqal_query_results_get_world(results);
+#ifndef HAVE_RAPTOR2_API
   const raptor_uri_handler *uri_handler;
   void *uri_context;
 #endif
@@ -103,33 +103,33 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
 
   if(!rasqal_query_results_is_bindings(results) &&
      !rasqal_query_results_is_boolean(results)) {
-    rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
-                            &query->locator,
+    rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR,
+                            NULL,
                             "Can only write XML format v3 for variable binding and boolean results");
     return 1;
   }
   
 
-#ifdef RAPTOR_V2_AVAILABLE  
-  nstack = raptor_new_namespaces(query->world->raptor_world_ptr, 1);
+#ifdef HAVE_RAPTOR2_API  
+  nstack = raptor_new_namespaces(world->raptor_world_ptr, 1);
 #else
   raptor_uri_get_handler(&uri_handler, &uri_context);
   nstack = raptor_new_namespaces(uri_handler, uri_context,
-                                 (raptor_simple_message_handler)rasqal_query_simple_error, query,
+                                 (raptor_simple_message_handler)rasqal_world_simple_error, world,
                                  1);
 #endif
   if(!nstack)
     return 1;
 
-#ifdef RAPTOR_V2_AVAILABLE
-  xml_writer = raptor_new_xml_writer(query->world->raptor_world_ptr,
+#ifdef HAVE_RAPTOR2_API
+  xml_writer = raptor_new_xml_writer(world->raptor_world_ptr,
                                      nstack,
                                      iostr);
 #else
   xml_writer = raptor_new_xml_writer(nstack,
                                      uri_handler, uri_context,
                                      iostr,
-                                     (raptor_simple_message_handler)rasqal_query_simple_error, query,
+                                     (raptor_simple_message_handler)rasqal_world_simple_error, world,
                                      1);
 #endif
   if(!xml_writer)
@@ -179,8 +179,8 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       attrs=(raptor_qname **)raptor_alloc_memory(sizeof(raptor_qname*));
       if(!attrs)
         goto tidy;
-#ifdef RAPTOR_V2_AVAILABLE
-      attrs[0] = raptor_new_qname_from_namespace_local_name(query->world->raptor_world_ptr,
+#ifdef HAVE_RAPTOR2_API
+      attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                             res_ns,
                                                             (const unsigned char*)"name",
                                                             (const unsigned char*)name); /* attribute value */
@@ -279,8 +279,8 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
       attrs=(raptor_qname **)raptor_alloc_memory(sizeof(raptor_qname*));
       if(!attrs)
         goto tidy;
-#ifdef RAPTOR_V2_AVAILABLE
-      attrs[0] = raptor_new_qname_from_namespace_local_name(query->world->raptor_world_ptr,
+#ifdef HAVE_RAPTOR2_API
+      attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                             res_ns,
                                                             (const unsigned char*)"name",
                                                             name);
@@ -351,13 +351,13 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
               attrs[0]=raptor_new_qname(nstack,
                                         (const unsigned char*)"xml:lang",
                                         (const unsigned char*)l->language
-#ifndef RAPTOR_V2_AVAILABLE
-                                        , (raptor_simple_message_handler)rasqal_query_simple_error, query
+#ifndef HAVE_RAPTOR2_API
+                                        , (raptor_simple_message_handler)rasqal_world_simple_error, world
 #endif
                                         );
             else
-#ifdef RAPTOR_V2_AVAILABLE
-              attrs[0] = raptor_new_qname_from_namespace_local_name(query->world->raptor_world_ptr,
+#ifdef HAVE_RAPTOR2_API
+              attrs[0] = raptor_new_qname_from_namespace_local_name(world->raptor_world_ptr,
                                                                     res_ns,
                                                                     (const unsigned char*)"datatype",
                                                                     (const unsigned char*)raptor_uri_as_string(l->datatype));
@@ -396,11 +396,12 @@ rasqal_query_results_write_sparql_xml(raptor_iostream *iostr,
         case RASQAL_LITERAL_DECIMAL:
         case RASQAL_LITERAL_DATETIME:
         case RASQAL_LITERAL_UDT:
+        case RASQAL_LITERAL_INTEGER_SUBTYPE:
 
         case RASQAL_LITERAL_UNKNOWN:
         default:
-          rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
-                                  &query->locator,
+          rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR,
+                                  NULL,
                                   "Cannot turn literal type %d into XML", 
                                   l->type);
           goto tidy;
@@ -514,7 +515,9 @@ typedef struct
   raptor_sax2* sax2;
   raptor_locator locator;
   int depth; /* element depth */
+#ifndef HAVE_RAPTOR2_API
   raptor_error_handlers error_handlers; /* SAX2 error handler */
+#endif
 
   /* SPARQL XML Results parsing */
   rasqal_sparql_xml_read_state state; /* state */
@@ -626,7 +629,7 @@ rasqal_sparql_xml_sax2_start_element_handler(void *user_data,
         rasqal_variable *v;
         
         var_name = (unsigned char*)RASQAL_MALLOC(cstring, con->name_length+1);
-        strncpy((char*)var_name, con->name, con->name_length+1);
+        memcpy(var_name, con->name, con->name_length + 1);
 
         v = rasqal_variables_table_add(con->vars_table,
                                        RASQAL_VARIABLE_TYPE_NORMAL,
@@ -741,13 +744,9 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
         char* language_str=NULL;
 
         lvalue=(unsigned char*)RASQAL_MALLOC(cstring, con->value_len+1);
-        strncpy((char*)lvalue, con->value, con->value_len+1);
+        memcpy(lvalue, con->value, con->value_len + 1);
         if(con->datatype)
-#ifdef RAPTOR_V2_AVAILABLE
           datatype_uri = raptor_new_uri(con->world->raptor_world_ptr, (const unsigned char*)con->datatype);
-#else
-          datatype_uri = raptor_new_uri((const unsigned char*)con->datatype);
-#endif
         if(con->language) {
           language_str=(char*)RASQAL_MALLOC(cstring, strlen(con->language)+1);
           strcpy(language_str, con->language);
@@ -766,7 +765,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
         rasqal_literal* l;
         unsigned char* lvalue;
         lvalue=(unsigned char*)RASQAL_MALLOC(cstring, con->value_len+1);
-        strncpy((char*)lvalue, con->value, con->value_len+1);
+        memcpy(lvalue, con->value, con->value_len + 1);
         l = rasqal_new_simple_literal(con->world, RASQAL_LITERAL_BLANK, lvalue);
         rasqal_row_set_value_at(con->row, con->result_offset, l);
         rasqal_free_literal(l);
@@ -779,11 +778,7 @@ rasqal_sparql_xml_sax2_end_element_handler(void *user_data,
       if(1) {
         raptor_uri* uri;
         rasqal_literal* l;
-#ifdef RAPTOR_V2_AVAILABLE
         uri = raptor_new_uri(con->world->raptor_world_ptr, (const unsigned char*)con->value);
-#else
-        uri = raptor_new_uri((const unsigned char*)con->value);
-#endif
         l = rasqal_new_uri_literal(con->world, uri);
         rasqal_row_set_value_at(con->row, con->result_offset, l);
         rasqal_free_literal(l);
@@ -873,8 +868,9 @@ rasqal_rowsource_sparql_xml_process(rasqal_rowsource_sparql_xml_context* con)
   while(!raptor_iostream_read_eof(con->iostr)) {
     size_t read_len;
     
-    read_len=raptor_iostream_read_bytes(con->iostr, (char*)con->buffer,
-                                        1, FILE_READ_BUF_SIZE);
+    read_len = raptor_iostream_read_bytes((char*)con->buffer, 1,
+                                          FILE_READ_BUF_SIZE,
+                                          con->iostr);
     if(read_len > 0) {
       RASQAL_DEBUG2("processing %d bytes\n", (int)read_len);
       raptor_sax2_parse_chunk(con->sax2, con->buffer, read_len, 0);
@@ -975,12 +971,10 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
 
   con->locator.uri=base_uri;
 
-  con->error_handlers.locator=&con->locator;
-#ifdef RAPTOR_V2_AVAILABLE
-  raptor_error_handlers_init(world->raptor_world_ptr, &con->error_handlers);
-
+#ifdef HAVE_RAPTOR2_API
   con->sax2 = raptor_new_sax2(world->raptor_world_ptr, &con->locator, con);
 #else
+  con->error_handlers.locator = &con->locator;
   raptor_error_handlers_init(&con->error_handlers);
 
   con->sax2 = raptor_new_sax2(con, &con->error_handlers);
@@ -996,8 +990,8 @@ rasqal_query_results_get_rowsource_sparql_xml(rasqal_world *world,
   raptor_sax2_set_end_element_handler(con->sax2,
                                       rasqal_sparql_xml_sax2_end_element_handler);
 
-#ifdef RAPTOR_V2_AVAILABLE
-  con->results_sequence = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_row, (raptor_data_print_handler*)rasqal_row_print);
+#ifdef HAVE_RAPTOR2_API
+  con->results_sequence = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row, (raptor_data_print_handler)rasqal_row_print);
 #else
   con->results_sequence = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row, (raptor_sequence_print_handler*)rasqal_row_print);
 #endif
