@@ -43,6 +43,7 @@
 
 /* prototypes for helper functions */
 static void rasqal_delete_query_language_factories(rasqal_world*);
+static void rasqal_free_query_language_factory(rasqal_query_language_factory *factory);
 
 
 /* statics */
@@ -140,7 +141,7 @@ rasqal_world_open(rasqal_world *world)
   if(world->opened++)
     return 0; /* not an error */
 
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   /* Create and init a raptor_world unless one is provided externally with rasqal_world_set_raptor() */
   if(!world->raptor_world_ptr) {
     world->raptor_world_ptr = raptor_new_world();
@@ -163,16 +164,26 @@ rasqal_world_open(rasqal_world *world)
   if(rc)
     return rc;
 
+#ifndef HAVE_RAPTOR2_API
 /* FIXME */
 #ifndef RAPTOR_ERROR_HANDLER_MAGIC
 #define RAPTOR_ERROR_HANDLER_MAGIC 0xD00DB1FF
 #endif
-  world->error_handlers.magic=RAPTOR_ERROR_HANDLER_MAGIC;
+  world->error_handlers.magic = RAPTOR_ERROR_HANDLER_MAGIC;
+#endif
 
-  /* last one declared is the default - RDQL */
+  world->query_languages = raptor_new_sequence((raptor_data_free_handler)rasqal_free_query_language_factory, NULL);
+  if(!world->query_languages)
+    return 1;
+  
+  /* first query language declared is the default */
 
-#ifdef RASQAL_QUERY_RDQL
-  rc = rasqal_init_query_language_rdql(world);
+#ifdef RASQAL_QUERY_SPARQL  
+  rc = rasqal_init_query_language_sparql(world);
+  if(rc)
+    return rc;
+
+  rc = rasqal_init_query_language_sparql11(world);
   if(rc)
     return rc;
 #endif
@@ -183,12 +194,13 @@ rasqal_world_open(rasqal_world *world)
     return rc;
 #endif
 
-#ifdef RASQAL_QUERY_SPARQL  
-  rc = rasqal_init_query_language_sparql(world);
+#ifdef RASQAL_QUERY_RDQL
+  rc = rasqal_init_query_language_rdql(world);
   if(rc)
     return rc;
 #endif
 
+
 #ifdef RAPTOR_TRIPLES_SOURCE_RAPTOR
   rc = rasqal_raptor_init(world);
   if(rc)
@@ -224,7 +236,8 @@ rasqal_world_open(rasqal_world *world)
 void
 rasqal_free_world(rasqal_world* world) 
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(world, rasqal_world);
+  if(!world)
+    return;
   
   rasqal_finish_result_formats(world);
   rasqal_finish_query_results();
@@ -239,7 +252,7 @@ rasqal_free_world(rasqal_world* world)
 
   rasqal_uri_finish(world);
 
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   if(world->raptor_world_ptr && world->raptor_world_allocated_here)
     raptor_free_world(world->raptor_world_ptr);
 #else
@@ -289,6 +302,27 @@ rasqal_world_get_raptor(rasqal_world* world)
 }
 
 
+#ifdef HAVE_RAPTOR2_API
+/**
+ * rasqal_world_set_log_handler:
+ * @world: rasqal_world object
+ * @user_data: user data for log handler function
+ * @handler: log handler function
+ *
+ * Set the log handler for this rasqal_world.
+ **/
+void
+rasqal_world_set_log_handler(rasqal_world* world, void *user_data,
+                             raptor_log_handler handler)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(world, rasqal_world);
+
+  world->log_handler = handler;
+  world->log_handler_user_data = user_data;
+}
+#endif
+
+
 /* helper functions */
 
 /*
@@ -299,21 +333,10 @@ rasqal_free_query_language_factory(rasqal_query_language_factory *factory)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(factory, rasqal_query_language_factory);
   
-  if(factory) {
-    if(factory->finish_factory)
-      factory->finish_factory(factory);
+  if(factory->finish_factory)
+    factory->finish_factory(factory);
 
-    if(factory->name)
-      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->name);
-    if(factory->label)
-      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->label);
-    if(factory->alias)
-      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->alias);
-    if(factory->uri_string)
-      RASQAL_FREE(rasqal_query_language_factory, (void*)factory->uri_string);
-
-    RASQAL_FREE(rasqal_query_language_factory, factory);
-  }
+  RASQAL_FREE(rasqal_query_language_factory, factory);
 }
 
 
@@ -323,105 +346,91 @@ rasqal_free_query_language_factory(rasqal_query_language_factory *factory)
 static void
 rasqal_delete_query_language_factories(rasqal_world *world)
 {
-  rasqal_query_language_factory *factory, *next;
-  
-  for(factory=world->query_languages; factory; factory=next) {
-    next=factory->next;
-    rasqal_free_query_language_factory(factory);
+  if(world->query_languages) {
+    raptor_free_sequence(world->query_languages);
+    world->query_languages = NULL;
   }
-  world->query_languages=NULL;
 }
 
 
 /* class methods */
 
 /*
- * rasqal_query_language_register_factory - Register a syntax handled by a query factory
- * @name: the short syntax name
- * @label: readable label for syntax
- * @uri_string: URI string of the syntax (or NULL)
+ * rasqal_query_language_register_factory:
  * @factory: pointer to function to call to register the factory
  * 
- * INTERNAL
+ * INTERNAL - Register a query language syntax handled by a query factory
  *
- * Return value: non-0 on failure
+ * Return value: new factory or NULL on failure
  **/
 RASQAL_EXTERN_C
-int
+rasqal_query_language_factory*
 rasqal_query_language_register_factory(rasqal_world *world,
-                                     const char *name, const char *label,
-                                     const char *alias,
-                                     const unsigned char *uri_string,
-                                     void (*factory) (rasqal_query_language_factory*)) 
+                                       int (*factory) (rasqal_query_language_factory*))
 {
-  rasqal_query_language_factory *query, *h;
-  char *name_copy, *label_copy, *alias_copy;
-  unsigned char *uri_string_copy;
+  rasqal_query_language_factory *query = NULL;
   
-#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 1
-  RASQAL_DEBUG4("Received registration for syntax %s '%s' with alias '%s'\n", 
-                name, label, (alias ? alias : "none"));
-  RASQAL_DEBUG2("URI %s\n", (uri_string ? (const char*)uri_string : (const char*)"none"));
-#endif
-  
-  query=(rasqal_query_language_factory*)RASQAL_CALLOC(rasqal_query_language_factory, 1,
-                                                    sizeof(rasqal_query_language_factory));
+  query = (rasqal_query_language_factory*)RASQAL_CALLOC(rasqal_query_language_factory, 1,
+                                                        sizeof(*query));
   if(!query)
-    goto tidy_noquery;
+    goto tidy;
 
-  for(h = world->query_languages; h; h = h->next ) {
-    if(!strcmp(h->name, name) ||
-       (alias && !strcmp(h->name, alias))) {
-      RASQAL_FATAL2("query %s already registered\n", h->name);
+  query->world = world;
+  
+  query->desc.mime_types = NULL;
+
+  if(raptor_sequence_push(world->query_languages, query))
+    return NULL; /* on error, query is already freed by the sequence */
+  
+  /* Call the query registration function on the new object */
+  if(factory(query))
+    return NULL; /* query is owned and freed by the query_languages sequence */
+  
+  if(!query->desc.names || !query->desc.names[0] || !query->desc.label) {
+    rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR, NULL,
+                            "Query language failed to register required names and label fields\n");
+    goto tidy;
+  }
+
+#ifdef RASQAL_DEBUG
+  /* Maintainer only check of static data */
+  if(query->desc.mime_types) {
+    unsigned int i;
+    const raptor_type_q* type_q = NULL;
+
+    for(i = 0; 
+        (type_q = &query->desc.mime_types[i]) && type_q->mime_type;
+        i++) {
+      size_t len = strlen(type_q->mime_type);
+      if(len != type_q->mime_type_len) {
+        fprintf(stderr,
+                "Query language %s  mime type %s  actual len %d  static len %d\n",
+                query->desc.names[0], type_q->mime_type,
+                (int)len, (int)type_q->mime_type_len);
+      }
+    }
+
+    if(i != query->desc.mime_types_count) {
+        fprintf(stderr,
+                "Query language %s  saw %d mime types  static count %d\n",
+                query->desc.names[0], i, query->desc.mime_types_count);
     }
   }
-  
-  name_copy=(char*)RASQAL_CALLOC(cstring, strlen(name)+1, 1);
-  if(!name_copy)
-    goto tidy;
-  strcpy(name_copy, name);
-  query->name=name_copy;
-        
-  label_copy=(char*)RASQAL_CALLOC(cstring, strlen(label)+1, 1);
-  if(!label_copy)
-    goto tidy;
-  strcpy(label_copy, label);
-  query->label=label_copy;
-
-  if(uri_string) {
-    uri_string_copy=(unsigned char*)RASQAL_CALLOC(cstring, strlen((const char*)uri_string)+1, 1);
-    if(!uri_string_copy)
-      goto tidy;
-    strcpy((char*)uri_string_copy, (const char*)uri_string);
-    query->uri_string=uri_string_copy;
-  }
-        
-  if(alias) {
-    alias_copy=(char*)RASQAL_CALLOC(cstring, strlen(alias)+1, 1);
-    if(!alias_copy)
-      goto tidy;
-    strcpy(alias_copy, alias);
-    query->alias=alias_copy;
-  }
-
-  /* Call the query registration function on the new object */
-  (*factory)(query);
-  
-#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 1
-  RASQAL_DEBUG3("%s has context size %d\n", name, (int)query->context_length);
 #endif
-  
-  query->next = world->query_languages;
-  world->query_languages = query;
 
-  return 0;
+#if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 1
+  RASQAL_DEBUG3("Registered query language %s with context size %d\n",
+                query->names[0], query->context_length);
+#endif
 
+  return query;
+
+  /* Clean up on failure */
   tidy:
-  rasqal_free_query_language_factory(query);
-  tidy_noquery:
-  rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_FATAL, NULL,
-                          "Out of memory in rasqal_query_language_register_factory()");
-  return 1;
+  if(query)
+    rasqal_free_query_language_factory(query);
+
+  return NULL;
 }
 
 
@@ -438,32 +447,30 @@ rasqal_query_language_factory*
 rasqal_get_query_language_factory(rasqal_world *world, const char *name,
                                   const unsigned char *uri)
 {
-  rasqal_query_language_factory *factory;
+  rasqal_query_language_factory *factory = NULL;
 
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
-
-  /* for compatibility with old API that does not call this - FIXME Remove V2 */
-  rasqal_world_open(world);
-  
-  /* return 1st query if no particular one wanted - why? */
-  if(!name && !uri) {
-    factory=world->query_languages;
+  /* return 1st query language if no particular one wanted - why? */
+  if(!name) {
+    factory = (rasqal_query_language_factory*)raptor_sequence_get_at(world->query_languages, 0);
     if(!factory) {
-      RASQAL_DEBUG1("No (default) query_languages registered\n");
+      RASQAL_DEBUG1("No (default) query languages registered\n");
       return NULL;
     }
   } else {
-    for(factory=world->query_languages; factory; factory=factory->next) {
-      if((name && !strcmp(factory->name, name)) ||
-         (factory->alias && !strcmp(factory->alias, name)))
+    int i;
+    
+    for(i = 0;
+        (factory = (rasqal_query_language_factory*)raptor_sequence_get_at(world->query_languages, i));
+        i++) {
+      int namei;
+      const char* fname;
+      
+      for(namei = 0; (fname = factory->desc.names[namei]); namei++) {
+        if(!strcmp(fname, name))
+          break;
+      }
+      if(fname)
         break;
-      if(uri && !strcmp((const char*)factory->uri_string, (const char*)uri))
-        break;
-    }
-    /* else FACTORY name not found */
-    if(!factory) {
-      RASQAL_DEBUG2("No query language with name %s found\n", name);
-      return NULL;
     }
   }
         
@@ -489,7 +496,6 @@ rasqal_languages_enumerate(rasqal_world *world,
                            const char **name, const char **label,
                            const unsigned char **uri_string)
 {
-  unsigned int i;
   rasqal_query_language_factory *factory;
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
@@ -499,23 +505,20 @@ rasqal_languages_enumerate(rasqal_world *world,
   /* for compatibility with old API that does not call this - FIXME Remove V2 */
   rasqal_world_open(world);
   
-  factory = world->query_languages;
+  factory = raptor_sequence_get_at(world->query_languages, counter);
   if(!factory)
     return 1;
 
-  for(i=0; factory && i<=counter ; i++, factory=factory->next) {
-    if(i == counter) {
-      if(name)
-        *name=factory->name;
-      if(label)
-        *label=factory->label;
-      if(uri_string)
-        *uri_string=factory->uri_string;
-      return 0;
-    }
-  }
-        
-  return 1;
+  if(name)
+    *name = factory->desc.names[0];
+
+  if(label)
+    *label = factory->desc.label;
+
+  if(uri_string)
+    *uri_string = (const unsigned char*)factory->desc.uri_string;
+
+  return 0;
 }
 
 
@@ -538,7 +541,7 @@ rasqal_language_name_check(rasqal_world* world, const char *name)
 
 
 static const char* const rasqal_log_level_labels[RAPTOR_LOG_LEVEL_LAST+1]={
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   /* raptor2 levels */
  "none",
  "trace",
@@ -581,8 +584,14 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
 {
   char *buffer;
   size_t length;
-  raptor_message_handler handler=world->error_handlers.handlers[level].handler;
-  void* handler_data=world->error_handlers.handlers[level].user_data;
+#ifdef HAVE_RAPTOR2_API
+  raptor_log_message logmsg;
+  raptor_log_handler handler = world->log_handler;
+  void* handler_data = world->log_handler_user_data;
+#else
+  raptor_message_handler handler = world->error_handlers.handlers[level].handler;
+  void* handler_data = world->error_handlers.handlers[level].user_data;
+#endif
   
   if(level == RAPTOR_LOG_LEVEL_NONE)
     return;
@@ -590,7 +599,7 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
   buffer=raptor_vsnprintf(message, arguments);
   if(!buffer) {
     if(locator) {
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
       raptor_locator_print(locator, stderr);
 #else
       raptor_print_locator(stderr, locator);
@@ -609,14 +618,24 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
   if(buffer[length-1]=='\n')
     buffer[length-1]='\0';
   
-  if(handler)
+  if(handler) {
     /* This is the single place in rasqal that the user error handler
      * functions are called.
      */
+#ifdef HAVE_RAPTOR2_API
+    /* raptor2 raptor_log_handler */
+    logmsg.code = -1; /* no information to put as code */
+    logmsg.level = level;
+    logmsg.locator = locator;
+    logmsg.text = buffer;
+    handler(handler_data, &logmsg);
+#else
+    /* raptor1 raptor_message_handler */
     handler(handler_data, locator, buffer);
-  else {
+#endif
+  } else {
     if(locator) {
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
       raptor_locator_print(locator, stderr);
 #else
       raptor_print_locator(stderr, locator);
@@ -658,6 +677,30 @@ rasqal_query_simple_error(void* user_data, const char *message, ...)
 }
 
 
+/*
+ * rasqal_world_simple_error:
+ *
+ * INTERNAL - Handle a simple error
+ *
+ * Matches the raptor_simple_message_handler API but with a world object
+ */
+void
+rasqal_world_simple_error(void* user_data, const char *message, ...)
+{
+  rasqal_world* world = (rasqal_world*)user_data;
+
+  va_list arguments;
+
+  va_start(arguments, message);
+
+  rasqal_log_error_varargs(world,
+                           RAPTOR_LOG_LEVEL_ERROR, NULL,
+                           message, arguments);
+  
+  va_end(arguments);
+}
+
+
 /* wrapper */
 const char*
 rasqal_basename(const char *name)
@@ -698,13 +741,15 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
   unsigned long unichar=0;
   unsigned char *result;
   unsigned char *dest;
+  unsigned char *endp;
   int n;
   
   result=(unsigned char*)RASQAL_MALLOC(cstring, len+1);
   if(!result)
     return NULL;
 
-  dest=result;
+  dest = result;
+  endp = result + len;
 
   /* find end of string, fixing backslashed characters on the way */
   while(len > 0) {
@@ -712,7 +757,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
 
     if(c > 0x7f) {
       /* just copy the UTF-8 bytes through */
-      size_t unichar_len=raptor_utf8_to_unicode_char(NULL, (const unsigned char*)p, len+1);
+      size_t unichar_len = raptor_unicode_utf8_string_get_char((const unsigned char*)p, len+1, NULL);
       if(unichar_len > len) {
         if(error_handler)
           error_handler(error_data, "UTF-8 encoding error at character %d (0x%02X) found.", c, c);
@@ -774,7 +819,7 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
           break;
         }
           
-        dest+=raptor_unicode_char_to_utf8(unichar, dest);
+        dest += raptor_unicode_utf8_string_put_char(unichar, dest, endp - dest);
         break;
 
       default:
@@ -800,24 +845,13 @@ rasqal_escaped_name_to_utf8_string(const unsigned char *src, size_t len,
 int
 rasqal_uri_init(rasqal_world* world) 
 {
-#ifdef RAPTOR_V2_AVAILABLE
   world->rdf_namespace_uri = raptor_new_uri(world->raptor_world_ptr, raptor_rdf_namespace_uri);
-#else
-  world->rdf_namespace_uri = raptor_new_uri(raptor_rdf_namespace_uri);
-#endif
   if(!world->rdf_namespace_uri)
     goto oom;
 
-#ifdef RAPTOR_V2_AVAILABLE
   world->rdf_first_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"first");
   world->rdf_rest_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"rest");
   world->rdf_nil_uri = raptor_new_uri_from_uri_local_name(world->raptor_world_ptr, world->rdf_namespace_uri, (const unsigned char*)"nil");
-#else
-  world->rdf_first_uri = raptor_new_uri_from_uri_local_name(world->rdf_namespace_uri, (const unsigned char*)"first");
-  world->rdf_rest_uri = raptor_new_uri_from_uri_local_name(world->rdf_namespace_uri, (const unsigned char*)"rest");
-  world->rdf_nil_uri = raptor_new_uri_from_uri_local_name(world->rdf_namespace_uri, (const unsigned char*)"nil");
-#endif
-
   if(!world->rdf_first_uri || !world->rdf_rest_uri || !world->rdf_nil_uri)
     goto oom;
 
@@ -854,8 +888,8 @@ rasqal_uri_finish(rasqal_world* world)
 
 
 /**
- * rasqal_query_set_default_generate_bnodeid_parameters:
- * @rdf_query: #rasqal_query object
+ * rasqal_world_set_default_generate_bnodeid_parameters:
+ * @world: #rasqal_world object
  * @prefix: prefix string
  * @base: integer base identifier
  *
@@ -868,20 +902,21 @@ rasqal_uri_finish(rasqal_world* world)
  * but will use both parts.
  *
  * For finer control of the generated identifiers, use
- * rasqal_set_default_generate_bnodeid_handler()
+ * rasqal_world_set_generate_bnodeid_handler()
  *
  * If prefix is NULL, the default prefix is used (currently "bnodeid")
  * If base is less than 1, it is initialised to 1.
- * 
+ *
+ * Return value: non-0 on failure
  **/
-void
-rasqal_query_set_default_generate_bnodeid_parameters(rasqal_query* rdf_query, 
+int
+rasqal_world_set_default_generate_bnodeid_parameters(rasqal_world* world, 
                                                      char *prefix, int base)
 {
-  char *prefix_copy=NULL;
-  size_t length=0;
+  char *prefix_copy = NULL;
+  size_t length = 0;
 
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(rdf_query, rasqal_query);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
 
   if(--base < 0)
     base = 0;
@@ -891,54 +926,62 @@ rasqal_query_set_default_generate_bnodeid_parameters(rasqal_query* rdf_query,
     
     prefix_copy = (char*)RASQAL_MALLOC(cstring, length+1);
     if(!prefix_copy)
-      return;
+      return 1;
     strcpy(prefix_copy, prefix);
   }
   
-  if(rdf_query->default_generate_bnodeid_handler_prefix)
-    RASQAL_FREE(cstring, rdf_query->default_generate_bnodeid_handler_prefix);
+  if(world->default_generate_bnodeid_handler_prefix)
+    RASQAL_FREE(cstring, world->default_generate_bnodeid_handler_prefix);
 
-  rdf_query->default_generate_bnodeid_handler_prefix = prefix_copy;
-  rdf_query->default_generate_bnodeid_handler_prefix_length = length;
-  rdf_query->default_generate_bnodeid_handler_base = base;
+  world->default_generate_bnodeid_handler_prefix = prefix_copy;
+  world->default_generate_bnodeid_handler_prefix_length = length;
+  world->default_generate_bnodeid_handler_base = base;
+
+  return 0;
 }
 
 
 /**
- * rasqal_query_set_generate_bnodeid_handler:
- * @query: #rasqal_query query object
+ * rasqal_world_set_generate_bnodeid_handler:
+ * @world: #rasqal_world object
  * @user_data: user data pointer for callback
  * @handler: generate blank ID callback function
  *
- * Set the generate blank node ID handler function for the query.
+ * Set the generate blank node ID handler function
  *
- * Sets the function to generate blank node IDs for the query.
- * The handler is called with a pointer to the rasqal_query, the
+ * Sets the function to generate blank node IDs.
+ * The handler is called with a pointer to the rasqal_world, the
  * @user_data pointer and a user_bnodeid which is the value of
  * a user-provided blank node identifier (may be NULL).
  * It can either be returned directly as the generated value when present or
  * modified.  The passed in value must be free()d if it is not used.
  *
  * If handler is NULL, the default method is used
- * 
+ *
+ * If set, this overrides any bnodeid handler set via
+ * rasqal_query_set_generate_bnodeid_handler().
+ *  
+ * Return value: non-0 on failure
  **/
-void
-rasqal_query_set_generate_bnodeid_handler(rasqal_query* query,
+int
+rasqal_world_set_generate_bnodeid_handler(rasqal_world* world,
                                           void *user_data,
-                                          rasqal_generate_bnodeid_handler handler)
+                                          rasqal_generate_bnodeid_handler2 handler)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, 1);
 
-  query->generate_bnodeid_handler_user_data=user_data;
-  query->generate_bnodeid_handler=handler;
+  world->generate_bnodeid_handler_user_data = user_data;
+  world->generate_bnodeid_handler = handler;
+
+  return 0;  
 }
 
 
-static unsigned char*
-rasqal_default_generate_bnodeid_handler(void *user_data,
-                                        unsigned char *user_bnodeid) 
+unsigned char*
+rasqal_world_default_generate_bnodeid_handler(void *user_data,
+                                              unsigned char *user_bnodeid) 
 {
-  rasqal_query *rdf_query=(rasqal_query *)user_data;
+  rasqal_world *world = (rasqal_world*)user_data;
   int id;
   unsigned char *buffer;
   int length;
@@ -947,25 +990,25 @@ rasqal_default_generate_bnodeid_handler(void *user_data,
   if(user_bnodeid)
     return user_bnodeid;
 
-  id=++rdf_query->default_generate_bnodeid_handler_base;
+  id = ++world->default_generate_bnodeid_handler_base;
 
-  tmpid=id;
-  length=2; /* min length 1 + \0 */
-  while(tmpid/=10)
+  tmpid = id;
+  length = 2; /* min length 1 + \0 */
+  while(tmpid /= 10)
     length++;
 
-  if(rdf_query->default_generate_bnodeid_handler_prefix)
-    length += rdf_query->default_generate_bnodeid_handler_prefix_length;
+  if(world->default_generate_bnodeid_handler_prefix)
+    length += world->default_generate_bnodeid_handler_prefix_length;
   else
     length += 7; /* bnodeid */
   
-  buffer=(unsigned char*)RASQAL_MALLOC(cstring, length);
+  buffer = (unsigned char*)RASQAL_MALLOC(cstring, length);
   if(!buffer)
     return NULL;
-  if(rdf_query->default_generate_bnodeid_handler_prefix) {
-    strncpy((char*)buffer, rdf_query->default_generate_bnodeid_handler_prefix,
-            rdf_query->default_generate_bnodeid_handler_prefix_length);
-    sprintf((char*)buffer + rdf_query->default_generate_bnodeid_handler_prefix_length,
+  if(world->default_generate_bnodeid_handler_prefix) {
+    memcpy(buffer, world->default_generate_bnodeid_handler_prefix,
+           world->default_generate_bnodeid_handler_prefix_length);
+    sprintf((char*)buffer + world->default_generate_bnodeid_handler_prefix_length,
             "%d", id);
   } else 
     sprintf((char*)buffer, "bnodeid%d", id);
@@ -975,22 +1018,22 @@ rasqal_default_generate_bnodeid_handler(void *user_data,
 
 
 /*
- * rasqal_query_generate_bnodeid - Default generate id - internal
+ * rasqal_generate_bnodeid:
+ *
+ * Internal - Default generate ID
  */
 unsigned char*
-rasqal_query_generate_bnodeid(rasqal_query* rdf_query,
+rasqal_world_generate_bnodeid(rasqal_world* world,
                               unsigned char *user_bnodeid)
 {
-  if(rdf_query->generate_bnodeid_handler)
-    return rdf_query->generate_bnodeid_handler(rdf_query, 
-                                               rdf_query->generate_bnodeid_handler_user_data, user_bnodeid);
+  if(world->generate_bnodeid_handler)
+    return world->generate_bnodeid_handler(world, 
+                                           world->generate_bnodeid_handler_user_data, user_bnodeid);
   else
-    return rasqal_default_generate_bnodeid_handler(rdf_query, user_bnodeid);
+    return rasqal_world_default_generate_bnodeid_handler(world, user_bnodeid);
 }
 
 
-
-
 /**
  * rasqal_free_memory:
  * @ptr: memory pointer
@@ -1005,7 +1048,8 @@ rasqal_query_generate_bnodeid(rasqal_query* rdf_query,
 void
 rasqal_free_memory(void *ptr)
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(ptr, memory);
+  if(!ptr)
+    return;
   
   RASQAL_FREE(void, ptr);
 }
@@ -1120,24 +1164,3 @@ rasqal_sign_free(void *ptr)
   free(p);
 }
 #endif
-
-
-#if defined (RASQAL_DEBUG) && defined(HAVE_DMALLOC_H) && defined(RASQAL_MEMORY_DEBUG_DMALLOC)
-
-#undef malloc
-void*
-rasqal_system_malloc(size_t size)
-{
-  return malloc(size);
-}
-
-#undef free
-void
-rasqal_system_free(void *ptr)
-{
-  return free(ptr);
-  
-}
-
-#endif
-
