@@ -2,7 +2,7 @@
  *
  * rasqal_internal.h - Rasqal RDF Query library internals
  *
- * Copyright (C) 2003-2009, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2003-2010, David Beckett http://www.dajobe.org/
  * Copyright (C) 2003-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -35,15 +35,6 @@ extern "C" {
 
 #ifdef RASQAL_INTERNAL
 
-/* for the memory allocation functions */
-#if defined(HAVE_DMALLOC_H) && defined(RASQAL_MEMORY_DEBUG_DMALLOC)
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#undef HAVE_STDLIB_H
-#endif
-#include <dmalloc.h>
-#endif
-
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
 #define RASQAL_PRINTF_FORMAT(string_index, first_to_check_index) \
   __attribute__((__format__(__printf__, string_index, first_to_check_index)))
@@ -194,6 +185,71 @@ typedef struct rasqal_query_execution_factory_s rasqal_query_execution_factory;
 typedef struct rasqal_query_language_factory_s rasqal_query_language_factory;
 
 
+/**
+ * rasqal_projection:
+ * @query: rasqal query
+ * @wildcard: non-0 if @variables was '*'
+ * @distinct: 1 if distinct, 2 if reduced, otherwise neither
+ *
+ * Query projection (SELECT vars, SELECT *, SELECT DISTINCT/REDUCED ...)
+ *
+ */
+typedef struct {
+  rasqal_query* query;
+  
+  raptor_sequence* variables;
+
+  int wildcard:1;
+
+  int distinct;
+} rasqal_projection;  
+
+
+/**
+ * rasqal_solution_modifier:
+ * @query: rasqal query
+ * @order_conditions: sequence of order condition expressions (or NULL)
+ * @group_conditions: sequence of group by condition expressions (or NULL)
+ * @having_conditions: sequence of (group by ...) having condition expressions (or NULL)
+ * @limit: result limit LIMIT (>=0) or <0 if not given
+ * @offset: result offset OFFSET (>=0) or <0 if not given
+ *
+ * Query solution modifiers
+ *
+ */
+typedef struct {
+  rasqal_query* query;
+  
+  raptor_sequence* order_conditions;
+
+  raptor_sequence* group_conditions;
+
+  raptor_sequence* having_conditions;
+  
+  int limit;
+
+  int offset;
+} rasqal_solution_modifier;
+
+
+/**
+ * rasqal_bindings:
+ * @query: rasqal query
+ * @variables: sequence of variables
+ * @rows: sequence of #rasqal_row (or NULL)
+ *
+ * Result bindings from SPARQL 1.1 BINDINGS block
+ *
+ */
+typedef struct {
+  rasqal_query* query;
+  
+  raptor_sequence* variables;
+
+  raptor_sequence* rows;
+} rasqal_bindings;  
+
+
 /*
  * Graph Pattern
  */
@@ -215,17 +271,25 @@ struct rasqal_graph_pattern_s {
   /* index of the graph pattern in the query (0.. query->graph_pattern_count-1) */
   int gp_index;
 
-  /* Graph literal */
+  /* Graph literal / SERVICE literal */
   rasqal_literal *origin;
 
   /* Variable for LET graph pattern */
   rasqal_variable *var;
+
+  /* SELECT projection */
+  rasqal_projection* projection;
+
+  /* SELECT modifiers */
+  rasqal_solution_modifier* modifier;
 };
 
 rasqal_graph_pattern* rasqal_new_basic_graph_pattern(rasqal_query* query, raptor_sequence* triples, int start_column, int end_column);
 rasqal_graph_pattern* rasqal_new_graph_pattern_from_sequence(rasqal_query* query, raptor_sequence* graph_patterns, rasqal_graph_pattern_operator op);
 rasqal_graph_pattern* rasqal_new_filter_graph_pattern(rasqal_query* query, rasqal_expression* expr);
 rasqal_graph_pattern* rasqal_new_let_graph_pattern(rasqal_query *query, rasqal_variable *var, rasqal_expression *expr);  
+rasqal_graph_pattern* rasqal_new_select_graph_pattern(rasqal_query *query, rasqal_projection* projection, rasqal_graph_pattern* where, rasqal_solution_modifier* modifier);
+rasqal_graph_pattern* rasqal_new_single_graph_pattern(rasqal_query* query, rasqal_graph_pattern_operator op, rasqal_graph_pattern* single);
 void rasqal_free_graph_pattern(rasqal_graph_pattern* gp);
 void rasqal_graph_pattern_adjust(rasqal_graph_pattern* gp, int offset);
 void rasqal_graph_pattern_set_origin(rasqal_graph_pattern* graph_pattern, rasqal_literal* origin);
@@ -306,11 +370,11 @@ struct rasqal_query_s {
    */
   int distinct;
 
-  /* result limit LIMIT (>=0) or <0 if not given */
-  int limit;
+  /* WAS: result limit LIMIT (>=0) or <0 if not given */
+  int unused4;
 
-  /* result offset OFFSET (>=0) or <0 if not given */
-  int offset;
+  /* WAS: result offset OFFSET (>=0) or <0 if not given */
+  int unused5;
 
   /* non-0 if '*' was seen after a verb (the appropriate list such as selects or constructs will be NULL) */
   int wildcard;
@@ -344,9 +408,10 @@ struct rasqal_query_s {
   /* stuff for our user */
   void* user_data;
 
-  int default_generate_bnodeid_handler_base;
-  char *default_generate_bnodeid_handler_prefix;
-  size_t default_generate_bnodeid_handler_prefix_length;
+  /* former state for generating blank node IDs; now in world object */
+  int unused1;
+  char *unused2;
+  size_t unused3;
 
   void *generate_bnodeid_handler_user_data;
   rasqal_generate_bnodeid_handler generate_bnodeid_handler;
@@ -365,11 +430,11 @@ struct rasqal_query_s {
   /* incrementing counter for declaring prefixes in order of appearance */
   int prefix_depth;
 
-  /* sequence of order condition expressions */
-  raptor_sequence* order_conditions_sequence;
+  /* WAS: sequence of order condition expressions */
+  void* unused6;
 
-  /* sequence of group by condition expressions */
-  raptor_sequence* group_conditions_sequence;
+  /* WAS: sequence of group by condition expressions */
+  void* unused7;
 
   /* INTERNAL rasqal_literal_compare / rasqal_expression_evaluate flags */
   int compare_flags;
@@ -417,9 +482,45 @@ struct rasqal_query_s {
    * INSERT (deprecated), DELETE (deprecated) or UPDATE
    */
   raptor_sequence* updates;
+
+  /* WAS: sequence of (group by ...) having condition expressions */
+  raptor_sequence* unused8;
+  
+  /* INTERNAL solution modifier */
+  rasqal_solution_modifier* modifier;
+
+  /* INTERNAL SELECT bindings */
+  rasqal_bindings* bindings;
 };
 
 
+/* Definitions from raptor V2 raptor.h - REMOVE when Raptor V2 is required */
+#ifndef HAVE_RAPTOR2_API
+typedef void (*raptor_data_free_handler)(void* data);
+typedef void (*raptor_data_print_handler)(void *object, FILE *fh);
+
+typedef struct {
+  const char* mime_type;
+  size_t mime_type_len;
+  unsigned char q;
+} raptor_type_q;
+
+
+typedef struct {
+  const char* const* names;
+
+  const char* label;
+
+  const raptor_type_q* mime_types;
+  unsigned int mime_types_count;
+
+  const char* uri_string;
+
+  unsigned int flags;
+} raptor_syntax_description;
+#endif
+
+
 /*
  * A query language factory for a query language.
  *
@@ -428,22 +529,12 @@ struct rasqal_query_s {
  * query in any manner.
  */
 struct rasqal_query_language_factory_s {
+  rasqal_world* world;
+  
   struct rasqal_query_language_factory_s* next;
 
-  /* query language name */
-  const char* name;
-
-  /* query language readable label */
-  const char* label;
-
-  /* query language alternate name */
-  const char* alias;
-
-  /* query language MIME type (or NULL) */
-  const char* mime_type;
-
-  /* query language URI (or NULL) */
-  const unsigned char* uri_string;
+  /* static description that the query language registration initialises */
+  raptor_syntax_description desc;
   
   /* the rest of this structure is populated by the
      query-language-specific register function */
@@ -489,6 +580,9 @@ struct rasqal_row_s {
   /* number of expressions (can be 0) */
   int order_size;
   rasqal_literal** order_values;
+
+  /* Group ID */
+  int group_id;
 };
 
 
@@ -510,6 +604,8 @@ typedef enum {
 
 
 
+/* rasqal_rowsource_aggregation.c */
+rasqal_rowsource* rasqal_new_aggregation_rowsource(rasqal_world *world, rasqal_query* query, rasqal_rowsource* rowsource, raptor_sequence* exprs_seq, raptor_sequence* vars_seq);
 
 /* rasqal_rowsource_empty.c */
 rasqal_rowsource* rasqal_new_empty_rowsource(rasqal_world *world, rasqal_query* query);
@@ -529,6 +625,12 @@ rasqal_rowsource* rasqal_new_filter_rowsource(rasqal_world *world, rasqal_query
 /* rasqal_rowsource_graph.c */
 rasqal_rowsource* rasqal_new_graph_rowsource(rasqal_world *world, rasqal_query *query, rasqal_rowsource* rowsource, rasqal_variable *var);
 
+/* rasqal_rowsource_groupby.c */
+rasqal_rowsource* rasqal_new_groupby_rowsource(rasqal_world *world, rasqal_query* query, rasqal_rowsource* rowsource, raptor_sequence* exprs_seq);
+
+/* rasqal_rowsource_having.c */
+rasqal_rowsource* rasqal_new_having_rowsource(rasqal_world *world, rasqal_query *query, rasqal_rowsource* rowsource, raptor_sequence* exprs_seq);
+
 /* rasqal_rowsource_join.c */
 rasqal_rowsource* rasqal_new_join_rowsource(rasqal_world *world, rasqal_query* query, rasqal_rowsource* left, rasqal_rowsource* right, rasqal_join_type join_type, rasqal_expression *expr);
 
@@ -536,13 +638,13 @@ rasqal_rowsource* rasqal_new_join_rowsource(rasqal_world *world, rasqal_query* q
 rasqal_rowsource* rasqal_new_project_rowsource(rasqal_world *world, rasqal_query *query, rasqal_rowsource* rowsource, raptor_sequence* projection_variables);
 
 /* rasqal_rowsource_rowsequence.c */
-rasqal_rowsource* rasqal_new_rowsequence_rowsource(rasqal_world *world, rasqal_query* query, rasqal_variables_table* vt, raptor_sequence* row, raptor_sequence* vars_seq);
+rasqal_rowsource* rasqal_new_rowsequence_rowsource(rasqal_world *world, rasqal_query* query, rasqal_variables_table* vt, raptor_sequence* rows_seq, raptor_sequence* vars_seq);
 
 /* rasqal_rowsource_sort.c */
 rasqal_rowsource* rasqal_new_sort_rowsource(rasqal_world *world, rasqal_query *query, rasqal_rowsource *rowsource);
 
 /* rasqal_rowsource_triples.c */
-rasqal_rowsource* rasqal_new_triples_rowsource(rasqal_world *world, rasqal_query* query, rasqal_triples_source* triples_source, raptor_sequence* triples, int start_column, int end_column, int *bound_in, int bound_in_size);
+rasqal_rowsource* rasqal_new_triples_rowsource(rasqal_world *world, rasqal_query* query, rasqal_triples_source* triples_source, raptor_sequence* triples, int start_column, int end_column);
 
 /* rasqal_rowsource_union.c */
 rasqal_rowsource* rasqal_new_union_rowsource(rasqal_world *world, rasqal_query* query, rasqal_rowsource* left, rasqal_rowsource* right);
@@ -720,6 +822,7 @@ typedef struct {
  * @size: number of variables in @variables_sequence
  * @rows_sequence: stored sequence of rows for use by rasqal_rowsource_read_row() (or NULL)
  * @offset: size of @rows_sequence
+ * @generate_group: non-0 to generate a group (ID 0) around all the returned rows, if there is no grouping returned.
  *
  * Rasqal Row Source class providing a sequence of rows of values similar to a SQL table.
  *
@@ -764,7 +867,7 @@ struct rasqal_rowsource_s
 
   const rasqal_rowsource_handler* handler;
 
-  int finished;
+  unsigned int finished : 1;
 
   int count;
 
@@ -779,6 +882,8 @@ struct rasqal_rowsource_s
   raptor_sequence* rows_sequence;
 
   int offset;
+
+  unsigned int generate_group : 1;
 };
 
 
@@ -802,6 +907,7 @@ int rasqal_rowsource_write(rasqal_rowsource *rowsource,  raptor_iostream *iostr)
 void rasqal_rowsource_print(rasqal_rowsource* rs, FILE* fh);
 int rasqal_rowsource_ensure_variables(rasqal_rowsource *rowsource);
 int rasqal_rowsource_set_origin(rasqal_rowsource* rowsource, rasqal_literal *literal);
+int rasqal_rowsource_request_grouping(rasqal_rowsource* rowsource);
 
 typedef int (*rasqal_query_results_formatter_func)(raptor_iostream *iostr, rasqal_query_results* results, raptor_uri *base_uri);
 
@@ -855,21 +961,24 @@ typedef struct {
 
 /* rasqal_datetime.c */
 int rasqal_xsd_datetime_check(const unsigned char* string);
+int rasqal_xsd_date_check(const unsigned char* string);
 
 
 /* rasqal_general.c */
 char* rasqal_vsnprintf(const char* message, va_list arguments);
+unsigned char* rasqal_world_generate_bnodeid(rasqal_world* world, unsigned char *user_bnodeid);
 
-int rasqal_query_language_register_factory(rasqal_world*, const char* name, const char* label, const char* alias, const unsigned char* uri_string, void (*factory) (rasqal_query_language_factory*));
+rasqal_query_language_factory* rasqal_query_language_register_factory(rasqal_world *world, int (*factory) (rasqal_query_language_factory*));
 rasqal_query_language_factory* rasqal_get_query_language_factory (rasqal_world*, const char* name, const unsigned char* uri);
 void rasqal_log_error_simple(rasqal_world* world, raptor_log_level level, raptor_locator* locator, const char* message, ...) RASQAL_PRINTF_FORMAT(4, 5);
 void rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level, raptor_locator* locator, const char* message, va_list arguments) RASQAL_PRINTF_FORMAT(4, 0);
-void rasqal_log_error(raptor_log_level level, raptor_message_handler handler, void* handler_data, raptor_locator* locator, const char* message);
-void rasqal_query_simple_error(void* query, const char *message, ...) RASQAL_PRINTF_FORMAT(2, 3);
+void rasqal_query_simple_error(void* user_data /* query */, const char *message, ...) RASQAL_PRINTF_FORMAT(2, 3);
+void rasqal_world_simple_error(void* user_data /* world */, const char *message, ...) RASQAL_PRINTF_FORMAT(2, 3);
 
 const char* rasqal_basename(const char* name);
+unsigned char* rasqal_world_default_generate_bnodeid_handler(void *user_data, unsigned char *user_bnodeid);
 
-unsigned char* rasqal_escaped_name_to_utf8_string(const unsigned char* src, size_t len, size_t* dest_lenp, int (*error_handler)(rasqal_query *error_data, const char *message, ...), rasqal_query* error_data);
+unsigned char* rasqal_escaped_name_to_utf8_string(const unsigned char* src, size_t len, size_t* dest_lenp, int (*error_handler)(rasqal_query *error_data, const char *message, ...) RASQAL_PRINTF_FORMAT(2, 3), rasqal_query* error_data);
 
 /* rasqal_graph_pattern.c */
 unsigned char* rasqal_query_generate_bnodeid(rasqal_query* rdf_query, unsigned char *user_bnodeid);
@@ -884,6 +993,7 @@ int rasqal_init_query_language_rdql(rasqal_world*);
 
 /* sparql_parser.y */
 int rasqal_init_query_language_sparql(rasqal_world*);
+int rasqal_init_query_language_sparql11(rasqal_world*);
 int rasqal_init_query_language_laqrs(rasqal_world*);
 
 /* rasqal_query_transform.c */
@@ -895,6 +1005,7 @@ int rasqal_query_expand_query_constraints_qnames(rasqal_query* rq);
 int rasqal_query_build_anonymous_variables(rasqal_query* rq);
 int rasqal_query_expand_wildcards(rasqal_query* rq);
 int rasqal_query_remove_duplicate_select_vars(rasqal_query* rq);
+int rasqal_query_build_variables_use(rasqal_query* query);
 int rasqal_query_prepare_common(rasqal_query *query);
 int rasqal_query_merge_graph_patterns(rasqal_query* query, rasqal_graph_pattern* gp, void* data);
 int rasqal_graph_patterns_join(rasqal_graph_pattern *dest_gp, rasqal_graph_pattern *src_gp);
@@ -920,7 +1031,14 @@ void rasqal_expression_convert_to_literal(rasqal_expression* e, rasqal_literal*
 int rasqal_expression_mentions_variable(rasqal_expression* e, rasqal_variable* v);
 void rasqal_triple_write(rasqal_triple* t, raptor_iostream* iostr);
 void rasqal_variable_write(rasqal_variable* v, raptor_iostream* iostr);
+int rasqal_expression_is_aggregate(rasqal_expression* e);
+int rasqal_expression_convert_aggregate_to_variable(rasqal_expression* e_in, rasqal_variable* v, rasqal_expression** e_out);
+int rasqal_expression_mentions_aggregate(rasqal_expression* e);
 
+raptor_sequence* rasqal_expression_copy_expression_sequence(raptor_sequence* exprs_seq);
+int rasqal_literal_sequence_compare(int compare_flags, raptor_sequence* values_a, raptor_sequence* values_b);
+raptor_sequence* rasqal_expression_sequence_evaluate(rasqal_query* query, raptor_sequence* exprs_seq, int ignore_errors, int* error_p);
+int rasqal_literal_sequence_equals(raptor_sequence* values_a, raptor_sequence* values_b);
 
 /* strcasecmp.c */
 #ifdef HAVE_STRCASECMP
@@ -959,7 +1077,6 @@ rasqal_formula* rasqal_formula_join(rasqal_formula* first_formula, rasqal_formul
 /* The following should be public eventually in rasqal.h or raptor.h or ...? */
 
 typedef int (rasqal_compare_fn)(void* user_data, const void *a, const void *b);
-typedef void (rasqal_compare_free_user_data_fn)(const void *data);
 typedef void (rasqal_kv_free_fn)(const void *key, const void *value);
 
 
@@ -980,19 +1097,25 @@ void rasqal_literal_write_type(rasqal_literal* l, raptor_iostream* iostr);
 void rasqal_literal_write(rasqal_literal* l, raptor_iostream* iostr);
 void rasqal_expression_write_op(rasqal_expression* e, raptor_iostream* iostr);
 void rasqal_expression_write(rasqal_expression* e, raptor_iostream* iostr);
+int rasqal_literal_write_turtle(rasqal_literal* l, raptor_iostream* iostr);
+int rasqal_literal_array_equals(rasqal_literal** values_a, rasqal_literal** values_b, int size);
+int rasqal_literal_array_compare(rasqal_literal** values_a, rasqal_literal** values_b, raptor_sequence* exprs_seq, int size, int compare_flags);
+rasqal_map* rasqal_new_literal_sequence_sort_map(int is_distinct, int compare_flags);
+int rasqal_literal_sequence_sort_map_add_literal_sequence(rasqal_map* map, raptor_sequence* literals_sequence);
+raptor_sequence* rasqal_new_literal_sequence_of_sequence_from_data(rasqal_world* world, const char* const row_data[], int width);
+
 
 /* rasqal_map.c */
 typedef void (*rasqal_map_visit_fn)(void *key, void *value, void *user_data);
 
-#ifdef RAPTOR_V2_AVAILABLE
-rasqal_map* rasqal_new_map(rasqal_compare_fn* compare_fn, void* compare_user_data, rasqal_compare_free_user_data_fn* free_compare_user_data, rasqal_kv_free_fn* free_fn, raptor_data_print_handler* print_key_fn, raptor_data_print_handler* print_value_fn, int flags);
-#else
-rasqal_map* rasqal_new_map(rasqal_compare_fn* compare_fn, void* compare_user_data, rasqal_compare_free_user_data_fn* free_compare_user_data, rasqal_kv_free_fn* free_fn, raptor_sequence_print_handler* print_key_fn, raptor_sequence_print_handler* print_value_fn, int flags);
-#endif
+rasqal_map* rasqal_new_map(rasqal_compare_fn* compare_fn, void* compare_user_data, raptor_data_free_handler free_compare_user_data, raptor_data_free_handler free_key_fn, raptor_data_free_handler free_value_fn, raptor_data_print_handler print_key_fn, raptor_data_print_handler print_value_fn, int flags);
+
 void rasqal_free_map(rasqal_map *map);
 int rasqal_map_add_kv(rasqal_map* map, void* key, void *value);
 void rasqal_map_visit(rasqal_map* map, rasqal_map_visit_fn fn, void *user_data);
 int rasqal_map_print(rasqal_map* map, FILE* fh);
+void* rasqal_map_search(rasqal_map* map, const void* key);
+
 
 /* rasqal_query.c */
 rasqal_query_results* rasqal_query_execute_with_engine(rasqal_query* query, const rasqal_query_execution_factory* engine);
@@ -1005,6 +1128,7 @@ void rasqal_query_set_store_results(rasqal_query* query, int store_results);
 rasqal_variable* rasqal_query_get_variable_by_offset(rasqal_query* query, int idx);
 const rasqal_query_execution_factory* rasqal_query_get_engine_by_name(const char* name);
 int rasqal_query_variable_bound_in_triple(rasqal_query *query, rasqal_variable *v, int column);
+int rasqal_query_get_bound_in_column_for_variable(rasqal_query* query, rasqal_variable* v);
 
 /* rasqal_query_results.c */
 int rasqal_init_query_results(void);
@@ -1037,6 +1161,12 @@ int rasqal_init_result_format_sparql_xml(rasqal_world*);
 /* rasqal_format_table.c */
 int rasqal_init_result_format_table(rasqal_world*);
 
+/* rasqal_format_html.c */
+int rasqal_init_result_format_html(rasqal_world*);
+
+/* rasqal_format_turtle.c */
+int rasqal_init_result_format_turtle(rasqal_world*);
+
 /* rasqal_row.c */
 rasqal_row* rasqal_new_row(rasqal_rowsource* rowsource);
 rasqal_row* rasqal_new_row_from_row(rasqal_row* row);
@@ -1046,6 +1176,8 @@ int rasqal_row_to_nodes(rasqal_row* row);
 void rasqal_row_set_values_from_variables_table(rasqal_row* row, rasqal_variables_table* vars_table);
 int rasqal_row_set_order_size(rasqal_row *row, int order_size);
 int rasqal_row_expand_size(rasqal_row *row, int size);
+int rasqal_row_bind_variables(rasqal_row* row, rasqal_variables_table* vars_table);
+
 
 /* rasqal_row_compatible.c */
 rasqal_row_compatible* rasqal_new_row_compatible(rasqal_variables_table* vt, rasqal_rowsource *first_rowsource, rasqal_rowsource *second_rowsource);
@@ -1058,11 +1190,14 @@ rasqal_triples_source* rasqal_new_triples_source(rasqal_query* query);
 int rasqal_reset_triple_meta(rasqal_triple_meta* m);
 void rasqal_free_triples_source(rasqal_triples_source *rts);
 int rasqal_triples_source_triple_present(rasqal_triples_source *rts, rasqal_triple *t);
+int rasqal_triples_source_support_feature(rasqal_triples_source *rts, rasqal_triples_source_feature feature);
+
 rasqal_triples_match* rasqal_new_triples_match(rasqal_query* query, rasqal_triples_source* triples_source, rasqal_triple_meta *m, rasqal_triple *t);
 rasqal_triple_parts rasqal_triples_match_bind_match(struct rasqal_triples_match_s* rtm, rasqal_variable *bindings[4],rasqal_triple_parts parts);
 void rasqal_triples_match_next_match(struct rasqal_triples_match_s* rtm);
 int rasqal_triples_match_is_end(struct rasqal_triples_match_s* rtm);
 
+
 /* rasqal_xsd_datatypes.c */
 int rasqal_xsd_init(rasqal_world*);
 void rasqal_xsd_finish(rasqal_world*);
@@ -1072,6 +1207,8 @@ int rasqal_xsd_datatype_check(rasqal_literal_type native_type, const unsigned ch
 const char* rasqal_xsd_datatype_label(rasqal_literal_type native_type);
 int rasqal_xsd_is_datatype_uri(rasqal_world*, raptor_uri* uri);
 const unsigned char* rasqal_xsd_datetime_string_to_canonical(const unsigned char* datetime_string);
+const unsigned char* rasqal_xsd_date_string_to_canonical(const unsigned char* date_string);
+
 int rasqal_xsd_datatype_is_numeric(rasqal_literal_type type);
 unsigned char* rasqal_xsd_format_integer(int i, size_t *len_p);
 unsigned char* rasqal_xsd_format_float(float f, size_t *len_p);
@@ -1091,11 +1228,17 @@ struct rasqal_world_s {
   /* should rasqal free the raptor_world */
   int raptor_world_allocated_here;
 
+#ifdef HAVE_RAPTOR2_API
+  /* log handler */
+  raptor_log_handler log_handler;
+  void *log_handler_user_data;
+#else
   /* error handlers structure */
   raptor_error_handlers error_handlers;
+#endif
 
-  /* linked list of query languages */
-  rasqal_query_language_factory *query_languages;
+  /* sequence of query language factories */
+  raptor_sequence *query_languages;
 
   /* registered query results formats */
   raptor_sequence *query_results_formats;
@@ -1116,6 +1259,13 @@ struct rasqal_world_s {
   /* graph factory */
   rasqal_graph_factory *graph_factory;
   void *graph_factory_user_data;
+
+  int default_generate_bnodeid_handler_base;
+  char *default_generate_bnodeid_handler_prefix;
+  size_t default_generate_bnodeid_handler_prefix_length;
+
+  void *generate_bnodeid_handler_user_data;
+  rasqal_generate_bnodeid_handler2 generate_bnodeid_handler;
 };
 
 
@@ -1141,8 +1291,11 @@ typedef enum {
   RASQAL_ALGEBRA_OPERATOR_SLICE    = 12,
   RASQAL_ALGEBRA_OPERATOR_GRAPH    = 13,
   RASQAL_ALGEBRA_OPERATOR_ASSIGN   = 14,
+  RASQAL_ALGEBRA_OPERATOR_GROUP    = 15,
+  RASQAL_ALGEBRA_OPERATOR_AGGREGATION = 16,
+  RASQAL_ALGEBRA_OPERATOR_HAVING   = 17,
 
-  RASQAL_ALGEBRA_OPERATOR_LAST = RASQAL_ALGEBRA_OPERATOR_ASSIGN
+  RASQAL_ALGEBRA_OPERATOR_LAST = RASQAL_ALGEBRA_OPERATOR_HAVING
 } rasqal_algebra_node_operator;
 
 
@@ -1164,7 +1317,7 @@ struct rasqal_algebra_node_s {
   
   /* types JOIN, DIFF, LEFTJOIN, UNION, ORDERBY: node1 and node2 ALWAYS present
    * types FILTER, TOLIST: node1 ALWAYS present, node2 ALWAYS NULL
-   * type PROJECT, GRAPH: node1 always present
+   * type PROJECT, GRAPH, GROUPBY, AGGREGATION, HAVING: node1 always present
    * (otherwise NULL)
    */
   struct rasqal_algebra_node_s *node1;
@@ -1175,7 +1328,8 @@ struct rasqal_algebra_node_s {
    */
   rasqal_expression* expr;
 
-  /* types ORDERBY always present
+  /* types ORDERBY, GROUPBY, AGGREGATION, HAVING always present: sequence of
+   * #rasqal_expression
    * (otherwise NULL)
    */
   raptor_sequence* seq;
@@ -1183,7 +1337,7 @@ struct rasqal_algebra_node_s {
   /* types PROJECT, DISTINCT, REDUCED
    * FIXME: sequence of solution mappings */
 
-  /* types PROJECT, SLICE */
+  /* types PROJECT, SLICE, AGGREGATION: sequence of #rasqal_variable */
   raptor_sequence* vars_seq;
 
   /* type SLICE: start and length */
@@ -1198,6 +1352,7 @@ struct rasqal_algebra_node_s {
 };
 typedef struct rasqal_algebra_node_s rasqal_algebra_node;
 
+
 /**
  * rasqal_algebra_node_visit_fn:
  * @query: #rasqal_query containing the graph pattern
@@ -1211,6 +1366,41 @@ typedef struct rasqal_algebra_node_s rasqal_algebra_node;
  */
 typedef int (*rasqal_algebra_node_visit_fn)(rasqal_query* query, rasqal_algebra_node* node, void *user_data);
 
+typedef struct
+{
+  rasqal_query* query;
+  
+  /* aggregate expression variables map
+   * key: rasqal_expression* tree with an aggregate function at top
+   * value: rasqal_variable*
+   */
+  rasqal_map* agg_vars;
+
+  /* sequence of aggregate #rasqal_expression in same order as @agg_vars_seq */
+  raptor_sequence* agg_exprs;
+
+  /* sequence of aggregate #rasqal_expression in same order as @agg_exprs */
+  raptor_sequence* agg_vars_seq;
+
+  /* number of aggregate expressions seen
+   * ( = number of internal variables created )
+   */
+  int counter;
+
+  /* compare flags */
+  int flags;
+
+  /* error indicator */
+  int error;
+
+  /* if set, finding a new expression is an error */
+  unsigned int adding_new_vars_is_error : 1;
+
+  /* query part for error messages */
+  const char* error_part;
+} rasqal_algebra_aggregate;
+
+
 /* rasqal_algebra.c */
 
 rasqal_algebra_node* rasqal_new_assignment_algebra_node(rasqal_query* query, rasqal_variable *var, rasqal_expression *expr);
@@ -1225,6 +1415,10 @@ rasqal_algebra_node* rasqal_new_orderby_algebra_node(rasqal_query* query, rasqal
 rasqal_algebra_node* rasqal_new_project_algebra_node(rasqal_query* query, rasqal_algebra_node* node1, raptor_sequence* vars_seq);
 rasqal_algebra_node* rasqal_new_graph_algebra_node(rasqal_query* query, rasqal_algebra_node* node1, rasqal_literal *graph);
 rasqal_algebra_node* rasqal_new_let_algebra_node(rasqal_query* query, rasqal_variable *var, rasqal_expression *expr);
+rasqal_algebra_node* rasqal_new_groupby_algebra_node(rasqal_query* query, rasqal_algebra_node* node1, raptor_sequence* seq);
+rasqal_algebra_node* rasqal_new_aggregation_algebra_node(rasqal_query* query, rasqal_algebra_node* node1, raptor_sequence* exprs_seq, raptor_sequence* vars_seq);
+rasqal_algebra_node* rasqal_new_having_algebra_node(rasqal_query* query,rasqal_algebra_node* node1, raptor_sequence* exprs_seq);
+
 void rasqal_free_algebra_node(rasqal_algebra_node* node);
 rasqal_algebra_node_operator rasqal_algebra_node_get_operator(rasqal_algebra_node* node);
 const char* rasqal_algebra_node_operator_as_string(rasqal_algebra_node_operator op);
@@ -1232,8 +1426,17 @@ int rasqal_algebra_algebra_node_write(rasqal_algebra_node *node, raptor_iostream
 int rasqal_algebra_node_print(rasqal_algebra_node* node, FILE* fh);
 int rasqal_algebra_node_visit(rasqal_query *query, rasqal_algebra_node* node, rasqal_algebra_node_visit_fn fn, void *user_data);
 rasqal_algebra_node* rasqal_algebra_query_to_algebra(rasqal_query* query);
+rasqal_algebra_node* rasqal_algebra_query_add_group_by(rasqal_query* query, rasqal_algebra_node* node);
+rasqal_algebra_node* rasqal_algebra_query_add_modifiers(rasqal_query* query, rasqal_algebra_node* node);
+rasqal_algebra_node* rasqal_algebra_query_add_aggregation(rasqal_query* query, rasqal_algebra_aggregate* ae, rasqal_algebra_node* node);
+rasqal_algebra_node* rasqal_algebra_query_add_projection(rasqal_query* query, rasqal_algebra_node* node);
+rasqal_algebra_node* rasqal_algebra_query_add_distinct(rasqal_query* query, rasqal_algebra_node* node);
+rasqal_algebra_node* rasqal_algebra_query_add_having(rasqal_query* query, rasqal_algebra_node* node);
 int rasqal_algebra_node_is_empty(rasqal_algebra_node* node);
 
+rasqal_algebra_aggregate* rasqal_algebra_query_prepare_aggregates(rasqal_query* query, rasqal_algebra_node* node);
+void rasqal_free_algebra_aggregate(rasqal_algebra_aggregate* ae);
+
 /* rasqal_variable.c */
 rasqal_variables_table* rasqal_new_variables_table_from_variables_table(rasqal_variables_table* vt);
 rasqal_variable* rasqal_variables_table_get(rasqal_variables_table* vt, int idx);
@@ -1247,7 +1450,7 @@ int rasqal_variables_table_get_total_variables_count(rasqal_variables_table* vt)
 raptor_sequence* rasqal_variables_table_get_named_variables_sequence(rasqal_variables_table* vt);
 raptor_sequence* rasqal_variables_table_get_anonymous_variables_sequence(rasqal_variables_table* vt);
 const unsigned char** rasqal_variables_table_get_names(rasqal_variables_table* vt);
-
+raptor_sequence* rasqal_variable_copy_variable_sequence(raptor_sequence* vars_seq);
 
 /**
  * rasqal_engine_error:
@@ -1345,6 +1548,8 @@ const char* rasqal_engine_get_parts_string(rasqal_triple_parts parts);
 /* New query engine based on executing over query algebra */
 extern const rasqal_query_execution_factory rasqal_query_engine_algebra;
 
+/* rasqal_triples_source.c */
+void rasqal_triples_source_error_handler(rasqal_query* rdf_query, raptor_locator* locator, const char* message);
 
 /* rasqal_update.c */
 const char* rasqal_update_type_label(rasqal_update_type type);
@@ -1353,6 +1558,59 @@ void rasqal_free_update_operation(rasqal_update_operation *update);
 int rasqal_update_operation_print(rasqal_update_operation *update, FILE* stream);
 int rasqal_query_add_update_operation(rasqal_query* query, rasqal_update_operation *update);
 
+
+/* rasqal_bindings.c */
+rasqal_bindings* rasqal_new_bindings(rasqal_query* query, raptor_sequence* variables, raptor_sequence* rows);
+void rasqal_free_bindings(rasqal_bindings* bindings);
+int rasqal_bindings_print(rasqal_bindings* bindings, FILE* fh);
+
+
+/* rasqal_projection.c */
+rasqal_projection* rasqal_new_projection(rasqal_query* query, raptor_sequence* variables, int wildcard, int distinct);
+void rasqal_free_projection(rasqal_projection* projection);
+raptor_sequence* rasqal_projection_get_variables_sequence(rasqal_projection* projection);
+
+
+/* rasqal_solution_modifier.c */
+rasqal_solution_modifier* rasqal_new_solution_modifier(rasqal_query* query, raptor_sequence* order_conditions, raptor_sequence* group_conditions, raptor_sequence* having_conditions, int limit, int offset);
+void rasqal_free_solution_modifier(rasqal_solution_modifier* sm);
+
+
+/* raptor1 -> raptor2 migration */
+/* use preprocessor macros to map raptor2 calls to raptor1 if raptor2 is not available */
+#ifndef HAVE_RAPTOR2_API
+
+#define raptor_new_iostream_to_file_handle(world, fh) raptor_new_iostream_to_file_handle(fh)
+#define raptor_new_iostream_from_file_handle(world, fh) raptor_new_iostream_from_file_handle(fh)
+#define raptor_iostream_write_bytes(ptr, size, nmemb, iostr) raptor_iostream_write_bytes(iostr, ptr, size, nmemb)
+#define raptor_iostream_write_byte(byte, iostr) raptor_iostream_write_byte(iostr, byte)
+#define raptor_iostream_counted_string_write(string, len, iostr) raptor_iostream_write_counted_string(iostr, string, len)
+#define raptor_iostream_string_write(string, iostr) raptor_iostream_write_string(iostr, string)
+#define raptor_iostream_decimal_write(decimal, iostr) raptor_iostream_write_decimal(iostr, decimal)
+#define raptor_iostream_read_bytes(ptr, size, nmemb, iostr) raptor_iostream_read_bytes(iostr, ptr, size, nmemb)
+#define raptor_xml_escape_string_write(string, len, quote, iostr) raptor_iostream_write_xml_escaped_string(iostr, string, len, quote, NULL, NULL)
+
+#define raptor_string_ntriples_write(str, len, delim, iostr) raptor_iostream_write_string_ntriples(iostr, str, len, delim)
+
+#define raptor_new_qname_from_namespace_uri(nstack, uri, xml_version) raptor_namespaces_qname_from_uri(nstack, uri, xml_version)
+
+#define raptor_new_serializer(world, name) raptor_new_serializer(name)
+#define raptor_world_is_serializer_name(world, name) raptor_serializer_syntax_name_check(name)
+#define raptor_world_is_parser_name(world, name) raptor_syntax_name_check(name)
+
+#define raptor_unicode_utf8_string_get_char(input, length, output) raptor_utf8_to_unicode_char(output, input, length)
+#define raptor_unicode_utf8_string_put_char(char, output, length) raptor_unicode_char_to_utf8(char, output)
+
+#define raptor_new_uri(world, uri_string) raptor_new_uri(uri_string)
+#define raptor_new_uri_from_uri_local_name(world, base_uri, name) raptor_new_uri_from_uri_local_name(base_uri, name)
+#define raptor_uri_write(uri, iostr) raptor_iostream_write_uri(iostr, uri)
+
+#define raptor_new_www(world) raptor_www_new()
+
+#define RAPTOR_LOG_LEVEL_WARN RAPTOR_LOG_LEVEL_WARNING
+
+#endif /* !HAVE_RAPTOR2_API */
+
   
 /* end of RASQAL_INTERNAL */
 #endif
