@@ -98,7 +98,7 @@ rasqal_new_query(rasqal_world *world, const char *name,
 {
   rasqal_query_language_factory* factory;
   rasqal_query* query;
-#ifndef RAPTOR_V2_AVAILABLE
+#ifndef HAVE_RAPTOR2_API
   const raptor_uri_handler *uri_handler;
   void *uri_context;
 #endif
@@ -123,9 +123,6 @@ rasqal_new_query(rasqal_world *world, const char *name,
   
   query->factory = factory;
 
-  query->limit = -1;
-  query->offset = -1;
-
   query->genid_counter = 1;
 
   query->context = (char*)RASQAL_CALLOC(rasqal_query_context, 1,
@@ -133,7 +130,7 @@ rasqal_new_query(rasqal_world *world, const char *name,
   if(!query->context)
     goto tidy;
   
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   query->namespaces = raptor_new_namespaces(world->raptor_world_ptr, 0);
 #else
   raptor_uri_get_handler(&uri_handler, &uri_context);
@@ -149,32 +146,32 @@ rasqal_new_query(rasqal_world *world, const char *name,
   if(!query->vars_table)
     goto tidy;
 
-#ifdef RAPTOR_V2_AVAILABLE
-  query->triples = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_triple, (raptor_data_print_handler*)rasqal_triple_print);
+#ifdef HAVE_RAPTOR2_API
+  query->triples = raptor_new_sequence((raptor_data_free_handler)rasqal_free_triple, (raptor_data_print_handler)rasqal_triple_print);
 #else
   query->triples = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_triple, (raptor_sequence_print_handler*)rasqal_triple_print);
 #endif
   if(!query->triples)
     goto tidy;
 
-#ifdef RAPTOR_V2_AVAILABLE
-  query->prefixes = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_prefix, (raptor_data_print_handler*)rasqal_prefix_print);
+#ifdef HAVE_RAPTOR2_API
+  query->prefixes = raptor_new_sequence((raptor_data_free_handler)rasqal_free_prefix, (raptor_data_print_handler)rasqal_prefix_print);
 #else
   query->prefixes = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_prefix, (raptor_sequence_print_handler*)rasqal_prefix_print);
 #endif
   if(!query->prefixes)
     goto tidy;
 
-#ifdef RAPTOR_V2_AVAILABLE
-  query->data_graphs = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_data_graph, (raptor_data_print_handler*)rasqal_data_graph_print);
+#ifdef HAVE_RAPTOR2_API
+  query->data_graphs = raptor_new_sequence((raptor_data_free_handler)rasqal_free_data_graph, (raptor_data_print_handler)rasqal_data_graph_print);
 #else
   query->data_graphs = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_data_graph, (raptor_sequence_print_handler*)rasqal_data_graph_print);
 #endif
   if(!query->data_graphs)
     goto tidy;
 
-#ifdef RAPTOR_V2_AVAILABLE
-  query->results = raptor_new_sequence((raptor_data_free_handler*)rasqal_query_results_remove_query_reference, NULL);
+#ifdef HAVE_RAPTOR2_API
+  query->results = raptor_new_sequence((raptor_data_free_handler)rasqal_query_results_remove_query_reference, NULL);
 #else
   query->results = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_query_results_remove_query_reference, NULL);
 #endif
@@ -202,7 +199,8 @@ rasqal_new_query(rasqal_world *world, const char *name,
 void
 rasqal_free_query(rasqal_query* query) 
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+  if(!query)
+    return;
   
   if(--query->usage)
     return;
@@ -249,12 +247,6 @@ rasqal_free_query(rasqal_query* query)
   if(query->query_graph_pattern)
     rasqal_free_graph_pattern(query->query_graph_pattern);
 
-  if(query->order_conditions_sequence)
-    raptor_free_sequence(query->order_conditions_sequence);
-
-  if(query->group_conditions_sequence)
-    raptor_free_sequence(query->group_conditions_sequence);
-
   if(query->graph_patterns_sequence)
     raptor_free_sequence(query->graph_patterns_sequence);
 
@@ -267,6 +259,12 @@ rasqal_free_query(rasqal_query* query)
 
   if(query->updates)
     raptor_free_sequence(query->updates);
+
+  if(query->modifier)
+    rasqal_free_solution_modifier(query->modifier);
+  
+  if(query->bindings)
+    rasqal_free_bindings(query->bindings);
   
   RASQAL_FREE(rasqal_query, query);
 }
@@ -287,7 +285,7 @@ rasqal_query_get_name(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  return query->factory->name;
+  return query->factory->desc.names[0];
 }
 
 
@@ -304,7 +302,7 @@ rasqal_query_get_label(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  return query->factory->label;
+  return query->factory->desc.label;
 }
 
 
@@ -323,6 +321,8 @@ void
 rasqal_query_set_fatal_error_handler(rasqal_query* query, void *user_data,
                                      raptor_message_handler handler)
 {
+#ifdef HAVE_RAPTOR2_API
+#else
   raptor_error_handlers* error_handlers;
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
@@ -333,7 +333,8 @@ rasqal_query_set_fatal_error_handler(rasqal_query* query, void *user_data,
   error_handlers = &query->world->error_handlers;
   
   error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL].user_data = user_data;
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL].handler = handler;
+  error_handlers->handlers[RAPTOR_LOG_LEVEL_FATAL].handler   = handler;
+#endif
 }
 
 
@@ -352,6 +353,8 @@ void
 rasqal_query_set_error_handler(rasqal_query* query, void *user_data,
                                raptor_message_handler handler)
 {
+#ifdef HAVE_RAPTOR2_API
+#else
   raptor_error_handlers* error_handlers;
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
@@ -362,7 +365,8 @@ rasqal_query_set_error_handler(rasqal_query* query, void *user_data,
   error_handlers = &query->world->error_handlers;
   
   error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].user_data = user_data;
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].handler = handler;
+  error_handlers->handlers[RAPTOR_LOG_LEVEL_ERROR].handler   = handler;
+#endif
 }
 
 
@@ -381,6 +385,8 @@ void
 rasqal_query_set_warning_handler(rasqal_query* query, void *user_data,
                                  raptor_message_handler handler)
 {
+#ifdef HAVE_RAPTOR2_API
+#else
   raptor_error_handlers* error_handlers;
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
@@ -390,12 +396,8 @@ rasqal_query_set_warning_handler(rasqal_query* query, void *user_data,
 
   error_handlers = &query->world->error_handlers;
   
-#ifdef RAPTOR_V2_AVAILABLE
   error_handlers->handlers[RAPTOR_LOG_LEVEL_WARN].user_data = user_data;
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_WARN].handler  =handler;
-#else
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_WARNING].user_data = user_data;
-  error_handlers->handlers[RAPTOR_LOG_LEVEL_WARNING].handler  =handler;
+  error_handlers->handlers[RAPTOR_LOG_LEVEL_WARN].handler   = handler;
 #endif
 }
 
@@ -612,7 +614,10 @@ rasqal_query_get_limit(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
 
-  return query->limit;
+  if(query->modifier)
+    return query->modifier->limit;
+  else
+    return -1;
 }
 
 
@@ -623,14 +628,17 @@ rasqal_query_get_limit(rasqal_query* query)
  *
  * Set the query-specified limit on results.
  *
- * This is the limit given in the query on the number of results allowed.
+ * This is the limit given in the query on the number of results
+ * allowed.  It is only guaranteed to work after the query is
+ * prepared and before it is executed.
  **/
 void
 rasqal_query_set_limit(rasqal_query* query, int limit)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
 
-  query->limit = limit;
+  if(query->modifier)
+    query->modifier->limit = limit;
 }
 
 
@@ -640,7 +648,9 @@ rasqal_query_set_limit(rasqal_query* query, int limit)
  *
  * Get the query-specified offset on results.
  *
- * This is the offset given in the query on the number of results allowed.
+ * This is the offset given in the query on the number of results
+ * allowed.  It is only guaranteed to work after the query is
+ * prepared and before it is executed.
  *
  * Return value: integer >=0 if a offset is given, otherwise <0
  **/
@@ -649,7 +659,10 @@ rasqal_query_get_offset(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
 
-  return query->offset;
+  if(query->modifier)
+    return query->modifier->offset;
+  else
+    return -1;
 }
 
 
@@ -667,7 +680,29 @@ rasqal_query_set_offset(rasqal_query* query, int offset)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
 
-  query->offset = offset;
+  if(query->modifier)
+    query->modifier->offset = offset;
+}
+
+
+/**
+ * rasqal_query_add_data_graph2:
+ * @query: #rasqal_query query object
+ * @data_graph: data graph
+ *
+ * Add a data graph to the query.
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_query_add_data_graph2(rasqal_query* query, rasqal_data_graph* data_graph)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(data_graph, rasqal_data_graph, 1);
+
+  if(raptor_sequence_push(query->data_graphs, (void*)data_graph))
+    return 1;
+  return 0;
 }
 
 
@@ -684,6 +719,9 @@ rasqal_query_set_offset(rasqal_query* query, int offset)
  * It is the name of the graph and also used as the base URI
  * when resolving any relative URIs for the graph in uri.
  *
+ * @Deprecated: Replaced by rasqal_new_data_graph_from_uri() with
+ * extra format arguments and rasqal_query_add_data_graph2()
+ *
  * Return value: non-0 on failure
  **/
 int
@@ -691,20 +729,42 @@ rasqal_query_add_data_graph(rasqal_query* query,
                             raptor_uri* uri, raptor_uri* name_uri,
                             int flags)
 {
-  rasqal_data_graph *dg;
-
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(uri, raptor_uri, 1);
-
-  if((flags & RASQAL_DATA_GRAPH_NAMED) && !name_uri)
-    return 1;
+  rasqal_data_graph* dg;
   
-  dg = rasqal_new_data_graph(query->world, uri, name_uri, flags);
+  dg = rasqal_new_data_graph_from_uri(query->world, uri, name_uri, flags,
+                                      NULL, NULL, NULL);
   if(!dg)
     return 1;
-  if(raptor_sequence_push(query->data_graphs, (void*)dg))
-    return 1;
-  return 0;
+
+  return rasqal_query_add_data_graph2(query, dg);
+}
+
+
+/**
+ * rasqal_query_add_data_graphs:
+ * @query: #rasqal_query query object
+ * @data_graphs: sequence of #rasqal_data_graph
+ *
+ * Add a set of data graphs to the query.
+ *
+ * The objects in the passed-in @data_graphs sequence becomes owne by the query.
+ * The @data_graphs sequence itself is freed and must not be used after this call.
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_query_add_data_graphs(rasqal_query* query,
+                             raptor_sequence* data_graphs)
+{
+  int rc;
+  
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(data_graphs, raptor_sequence, 1);
+
+  rc = raptor_sequence_join(query->data_graphs, data_graphs);
+  raptor_free_sequence(data_graphs);
+
+  return rc;
 }
 
 
@@ -784,7 +844,7 @@ rasqal_query_dataset_contains_named_graph(rasqal_query* query,
  *
  * Add a binding variable to the query.
  *
- * See also rasqal_query_set_variable which assigns or removes a value to
+ * See also rasqal_query_set_variable() which assigns or removes a value to
  * a previously added variable in the query.
  *
  * Return value: non-0 on failure
@@ -796,15 +856,18 @@ rasqal_query_add_variable(rasqal_query* query, rasqal_variable* var)
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(var, rasqal_variable, 1);
 
   if(!query->selects) {
-#ifdef RAPTOR_V2_AVAILABLE
-    query->selects = raptor_new_sequence(NULL, (raptor_data_print_handler*)rasqal_variable_print);
+#ifdef HAVE_RAPTOR2_API
+    query->selects = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                         (raptor_data_print_handler)rasqal_variable_print);
 #else
-    query->selects = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+    query->selects = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
+                                         (raptor_sequence_print_handler*)rasqal_variable_print);
 #endif
     if(!query->selects)
       return 1;
   }
 
+  var = rasqal_new_variable_from_variable(var);
   return raptor_sequence_push(query->selects, (void*)var);
 }
 
@@ -1075,8 +1138,8 @@ rasqal_query_add_prefix(rasqal_query* query, rasqal_prefix* prefix)
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(prefix, rasqal_prefix, 1);
 
   if(!query->prefixes) {
-#ifdef RAPTOR_V2_AVAILABLE
-    query->prefixes = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_prefix, (raptor_data_print_handler*)rasqal_prefix_print);
+#ifdef HAVE_RAPTOR2_API
+    query->prefixes = raptor_new_sequence((raptor_data_free_handler)rasqal_free_prefix, (raptor_data_print_handler)rasqal_prefix_print);
 #else
     query->prefixes = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_prefix, (raptor_sequence_print_handler*)rasqal_prefix_print);
 #endif
@@ -1295,11 +1358,7 @@ rasqal_query_prepare(rasqal_query* query,
     base_uri = raptor_uri_copy(base_uri);
   else {
     unsigned char *uri_string = raptor_uri_filename_to_uri_string("");
-#ifdef RAPTOR_V2_AVAILABLE
     base_uri = raptor_new_uri(query->world->raptor_world_ptr, uri_string);
-#else
-    base_uri = raptor_new_uri(uri_string);
-#endif
     if(uri_string)
       raptor_free_memory(uri_string);
   }
@@ -1465,11 +1524,14 @@ rasqal_query_print(rasqal_query* query, FILE *fh)
             (query->distinct == 1 ? "distinct" : "reduced"));
   if(query->explain)
     fputs("query results explain: yes\n", fh);
-  if(query->limit >= 0)
-    fprintf(fh, "query results limit: %d\n", query->limit);
-  if(query->offset >= 0)
-    fprintf(fh, "query results offset: %d\n", query->offset);
 
+  if(query->modifier) {
+    if(query->modifier->limit >= 0)
+      fprintf(fh, "query results limit: %d\n", query->modifier->limit);
+    if(query->modifier->offset >= 0)
+      fprintf(fh, "query results offset: %d\n", query->modifier->offset);
+  }
+  
   fputs("data graphs: ", fh);
   if(query->data_graphs)
     raptor_sequence_print(query->data_graphs, fh);
@@ -1511,18 +1573,30 @@ rasqal_query_print(rasqal_query* query, FILE *fh)
     fputs("\nquery graph pattern: ", fh);
     rasqal_graph_pattern_print(query->query_graph_pattern, fh);
   }
-  if(query->order_conditions_sequence) {
-    fputs("\nquery order conditions: ", fh);
-    raptor_sequence_print(query->order_conditions_sequence, fh);
-  }
-  if(query->group_conditions_sequence) {
-    fputs("\nquery group conditions: ", fh);
-    raptor_sequence_print(query->group_conditions_sequence, fh);
+
+  if(query->modifier) {
+    if(query->modifier->order_conditions) {
+      fputs("\nquery order conditions: ", fh);
+      raptor_sequence_print(query->modifier->order_conditions, fh);
+    }
+    if(query->modifier->group_conditions) {
+      fputs("\nquery group conditions: ", fh);
+      raptor_sequence_print(query->modifier->group_conditions, fh);
+    }
+    if(query->modifier->having_conditions) {
+      fputs("\nquery having conditions: ", fh);
+      raptor_sequence_print(query->modifier->having_conditions, fh);
+    }
   }
+  
   if(query->updates) {
     fputs("\nupdate operations: ", fh);
     raptor_sequence_print(query->updates, fh);
   }
+  if(query->bindings) {
+    fputs("\nbindings: ", fh);
+    rasqal_bindings_print(query->bindings, fh);
+  }
   fputc('\n', fh);
 
   return 0;
@@ -1656,7 +1730,10 @@ rasqal_query_get_order_conditions_sequence(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  return query->order_conditions_sequence;
+  if(query->modifier)
+    return query->modifier->order_conditions;
+  else
+    return NULL;
 }
 
 
@@ -1674,10 +1751,10 @@ rasqal_query_get_order_condition(rasqal_query* query, int idx)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  if(!query->order_conditions_sequence)
+  if(!query->modifier || !query->modifier->order_conditions)
     return NULL;
   
-  return (rasqal_expression*)raptor_sequence_get_at(query->order_conditions_sequence, idx);
+  return (rasqal_expression*)raptor_sequence_get_at(query->modifier->order_conditions, idx);
 }
 
 
@@ -1694,7 +1771,10 @@ rasqal_query_get_group_conditions_sequence(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  return query->group_conditions_sequence;
+  if(query->modifier)
+    return query->modifier->group_conditions;
+  else
+    return NULL;
 }
 
 
@@ -1712,10 +1792,51 @@ rasqal_query_get_group_condition(rasqal_query* query, int idx)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  if(!query->group_conditions_sequence)
+   if(!query->modifier || !query->modifier->group_conditions)
     return NULL;
   
-  return (rasqal_expression*)raptor_sequence_get_at(query->group_conditions_sequence, idx);
+  return (rasqal_expression*)raptor_sequence_get_at(query->modifier->group_conditions, idx);
+}
+
+
+/**
+ * rasqal_query_get_having_conditions_sequence:
+ * @query: #rasqal_query query object
+ *
+ * Get the sequence of query having conditions.
+ *
+ * Return value: a #raptor_sequence of #rasqal_expression pointers.
+ **/
+raptor_sequence*
+rasqal_query_get_having_conditions_sequence(rasqal_query* query)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(query->modifier)
+    return query->modifier->having_conditions;
+  else
+    return NULL;
+}
+
+
+/**
+ * rasqal_query_get_having_condition:
+ * @query: #rasqal_query query object
+ * @idx: index into the sequence (0 or larger)
+ *
+ * Get a query having expression in the sequence of query havinging conditions.
+ *
+ * Return value: a #rasqal_expression pointer or NULL if out of the sequence range
+ **/
+rasqal_expression*
+rasqal_query_get_having_condition(rasqal_query* query, int idx)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+   if(!query->modifier || !query->modifier->having_conditions)
+    return NULL;
+  
+  return (rasqal_expression*)raptor_sequence_get_at(query->modifier->having_conditions, idx);
 }
 
 
@@ -1843,7 +1964,7 @@ rasqal_query_escape_counted_string(rasqal_query* query,
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(string, char*, NULL);
 
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   iostr = raptor_new_iostream_to_string(query->world->raptor_world_ptr,
                                         &output_string, output_len_p,
                                         rasqal_alloc_memory);
@@ -1963,8 +2084,8 @@ rasqal_query_add_update_operation(rasqal_query* query,
     return 1;
 
   if(!query->updates) {
-#ifdef RAPTOR_V2_AVAILABLE
-    query->updates = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_update_operation, (raptor_data_print_handler*)rasqal_update_operation_print);
+#ifdef HAVE_RAPTOR2_API
+    query->updates = raptor_new_sequence((raptor_data_free_handler)rasqal_free_update_operation, (raptor_data_print_handler)rasqal_update_operation_print);
 #else
     query->updates = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_update_operation, (raptor_sequence_print_handler*)rasqal_update_operation_print);
 #endif
@@ -2016,3 +2137,194 @@ rasqal_query_get_update_operation(rasqal_query* query, int idx)
   
   return (rasqal_update_operation*)raptor_sequence_get_at(query->updates, idx);
 }
+
+
+/**
+ * rasqal_query_set_default_generate_bnodeid_parameters:
+ * @rdf_query: #rasqal_query object
+ * @prefix: prefix string
+ * @base: integer base identifier
+ *
+ * Set default bnodeid generation parameters
+ *
+ * Sets the parameters for the default algorithm used to generate
+ * blank node IDs.  The default algorithm uses both @prefix and @base
+ * to generate a new identifier.  The exact identifier generated is
+ * not guaranteed to be a strict concatenation of @prefix and @base
+ * but will use both parts.
+ *
+ * For finer control of the generated identifiers, use
+ * rasqal_world_set_generate_bnodeid_handler() on the world class (or
+ * the deprecated rasqal_query_set_generate_bnodeid_handler() on the
+ * query class).
+ *
+ * If prefix is NULL, the default prefix is used (currently "bnodeid")
+ * If base is less than 1, it is initialised to 1.
+ * 
+ * This calls rasqal_world_set_default_generate_bnodeid_parameters()
+ * method on the world class; there is no separate configuration per
+ * query object.
+ *  
+ * @Deprecated: Use
+ * rasqal_world_set_default_generate_bnodeid_parameters() on the
+ * world class.
+ *
+ **/
+void
+rasqal_query_set_default_generate_bnodeid_parameters(rasqal_query* rdf_query, 
+                                                     char *prefix, int base)
+{
+
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(rdf_query, rasqal_query);
+
+  rasqal_world_set_default_generate_bnodeid_parameters(rdf_query->world, 
+                                                       prefix, base);
+}
+
+
+/**
+ * rasqal_query_set_generate_bnodeid_handler:
+ * @query: #rasqal_query query object
+ * @user_data: user data pointer for callback
+ * @handler: generate blank ID callback function
+ *
+ * Set the generate blank node ID handler function for the query.
+ *
+ * Sets the function to generate blank node IDs for the query.
+ * The handler is called with a pointer to the rasqal_query, the
+ * @user_data pointer and a user_bnodeid which is the value of
+ * a user-provided blank node identifier (may be NULL).
+ * It can either be returned directly as the generated value when present or
+ * modified.  The passed in value must be free()d if it is not used.
+ *
+ * If handler is NULL, the default method is used
+ *
+ * Any hander set by rasqal_world_set_generate_bnodeid_handler()
+ * overrides this.
+ *  
+ * @Deprecated: Use
+ * rasqal_world_set_generate_bnodeid_handler() on the world class.
+ * 
+ **/
+void
+rasqal_query_set_generate_bnodeid_handler(rasqal_query* query,
+                                          void *user_data,
+                                          rasqal_generate_bnodeid_handler handler)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
+  query->generate_bnodeid_handler_user_data = user_data;
+  query->generate_bnodeid_handler = handler;
+}
+
+
+/*
+ * rasqal_query_generate_bnodeid - Default generate id - internal
+ */
+unsigned char*
+rasqal_query_generate_bnodeid(rasqal_query* rdf_query,
+                              unsigned char *user_bnodeid)
+{
+  /* prefer world generate handler/data */
+  if(rdf_query->world->generate_bnodeid_handler)
+    return rasqal_world_generate_bnodeid(rdf_query->world, user_bnodeid);
+
+  if(rdf_query->generate_bnodeid_handler)
+    return rdf_query->generate_bnodeid_handler(rdf_query, 
+                                               rdf_query->generate_bnodeid_handler_user_data, user_bnodeid);
+  else
+    return rasqal_world_default_generate_bnodeid_handler(rdf_query->world,
+                                                         user_bnodeid);
+}
+
+
+/**
+ * rasqal_query_get_bindings_variables_sequence:
+ * @query: #rasqal_query query object
+ *
+ * Get the sequence of BINDINGS block variables
+ *
+ * Return value: a #raptor_sequence of #raptor_variable pointers
+ **/
+raptor_sequence*
+rasqal_query_get_bindings_variables_sequence(rasqal_query* query)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(query->bindings)
+    return query->bindings->variables;
+  else
+    return NULL;
+  
+}
+
+
+/**
+ * rasqal_query_get_bindings_variable:
+ * @query: #rasqal_query query object
+ * @idx: index into the sequence (0 or larger)
+ *
+ * Get a rasqal_variable* in the sequence of BINDINGS block variables
+ *
+ * Return value: a #raptor_sequence of #raptor_variable pointers
+ **/
+rasqal_variable*
+rasqal_query_get_bindings_variable(rasqal_query* query, int idx)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(!query->bindings || !query->bindings->variables)
+    return NULL;
+  
+  return (rasqal_variable*)raptor_sequence_get_at(query->bindings->variables, idx);
+}
+
+
+/**
+ * rasqal_query_get_bindings_rows_sequence:
+ * @query: #rasqal_query query object
+ *
+ * Get the sequence of BINDINGS block result rows
+ *
+ * Return value: a #raptor_sequence of #raptor_row pointers
+ **/
+raptor_sequence*
+rasqal_query_get_bindings_rows_sequence(rasqal_query* query)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(query->bindings)
+    return query->bindings->rows;
+  else
+    return NULL;
+  
+}
+
+
+/**
+ * rasqal_query_get_bindings_row:
+ * @query: #rasqal_query query object
+ * @idx: index into the sequence (0 or larger)
+ *
+ * Get a rasqal_row* in the sequence of BINDINGS block result rows
+ *
+ * Return value: a #raptor_sequence of #raptor_row pointers
+ **/
+rasqal_row*
+rasqal_query_get_bindings_row(rasqal_query* query, int idx)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  if(!query->bindings || !query->bindings->rows)
+    return NULL;
+  
+  return (rasqal_row*)raptor_sequence_get_at(query->bindings->rows, idx);
+}
+
+
+int
+rasqal_query_get_bound_in_column_for_variable(rasqal_query* query,
+                                              rasqal_variable* v)
+{
+  return query->variables_bound_in[v->offset];
+}
