@@ -57,14 +57,17 @@ rasqal_query_write_sparql_variable(sparql_writer_context *wc,
                                    raptor_iostream* iostr, rasqal_variable* v)
 {
   if(v->expression) {
+    raptor_iostream_counted_string_write("( ", 2, iostr);
     rasqal_query_write_sparql_expression(wc, iostr, v->expression);
-    raptor_iostream_write_counted_string(iostr, " AS ", 4);
+    raptor_iostream_counted_string_write(" AS ", 4, iostr);
   }
   if(v->type == RASQAL_VARIABLE_TYPE_ANONYMOUS)
-    raptor_iostream_write_counted_string(iostr, "_:", 2);
+    raptor_iostream_counted_string_write("_:", 2, iostr);
   else if(!v->expression)
-    raptor_iostream_write_byte(iostr, '?');
-  raptor_iostream_write_string(iostr, v->name);
+    raptor_iostream_write_byte('?', iostr);
+  raptor_iostream_string_write(v->name, iostr);
+  if(v->expression)
+    raptor_iostream_counted_string_write(" )", 2, iostr);
 }
 
 
@@ -76,12 +79,16 @@ rasqal_query_write_sparql_uri(sparql_writer_context *wc,
   unsigned char* string;
   raptor_qname* qname;
 
-  qname = raptor_namespaces_qname_from_uri(wc->nstack, uri, 10);
+  qname = raptor_new_qname_from_namespace_uri(wc->nstack, uri, 10);
   if(qname) {
     const raptor_namespace* nspace = raptor_qname_get_namespace(qname);
     if(!raptor_namespace_get_prefix(nspace))
-      raptor_iostream_write_byte(iostr, ':');
+      raptor_iostream_write_byte(':', iostr);
+#ifdef HAVE_RAPTOR2_API
+    raptor_qname_write(qname, iostr);
+#else
     raptor_iostream_write_qname(iostr, qname);
+#endif
     raptor_free_qname(qname);
     return;
   }
@@ -91,9 +98,9 @@ rasqal_query_write_sparql_uri(sparql_writer_context *wc,
   else
     string = raptor_uri_as_counted_string(uri, &len);
 
-  raptor_iostream_write_byte(iostr, '<');
-  raptor_iostream_write_string_ntriples(iostr, string, len, '>');
-  raptor_iostream_write_byte(iostr, '>');
+  raptor_iostream_write_byte('<', iostr);
+  raptor_string_ntriples_write(string, len, '>', iostr);
+  raptor_iostream_write_byte('>', iostr);
 
   if(wc->base_uri)
     raptor_free_memory(string);
@@ -105,7 +112,7 @@ rasqal_query_write_sparql_literal(sparql_writer_context *wc,
                                   raptor_iostream* iostr, rasqal_literal* l)
 {
   if(!l) {
-    raptor_iostream_write_counted_string(iostr, "null", 4);
+    raptor_iostream_counted_string_write("null", 4, iostr);
     return;
   }
 
@@ -114,35 +121,35 @@ rasqal_query_write_sparql_literal(sparql_writer_context *wc,
       rasqal_query_write_sparql_uri(wc, iostr, l->value.uri);
       break;
     case RASQAL_LITERAL_BLANK:
-      raptor_iostream_write_counted_string(iostr, "_:", 2);
-      raptor_iostream_write_string(iostr, l->string);
+      raptor_iostream_counted_string_write("_:", 2, iostr);
+      raptor_iostream_string_write(l->string, iostr);
       break;
     case RASQAL_LITERAL_STRING:
-      raptor_iostream_write_byte(iostr, '"');
-      raptor_iostream_write_string_ntriples(iostr, l->string, l->string_len, '"');
-      raptor_iostream_write_byte(iostr, '"');
+      raptor_iostream_write_byte('"', iostr);
+      raptor_string_ntriples_write(l->string, l->string_len, '"', iostr);
+      raptor_iostream_write_byte('"', iostr);
       if(l->language) {
-        raptor_iostream_write_byte(iostr, '@');
-        raptor_iostream_write_string(iostr, l->language);
+        raptor_iostream_write_byte('@', iostr);
+        raptor_iostream_string_write(l->language, iostr);
       }
       if(l->datatype) {
-        raptor_iostream_write_counted_string(iostr, "^^", 2);
+        raptor_iostream_counted_string_write("^^", 2, iostr);
         rasqal_query_write_sparql_uri(wc, iostr, l->datatype);
       }
       break;
     case RASQAL_LITERAL_QNAME:
-      raptor_iostream_write_counted_string(iostr, "QNAME(", 6);
-      raptor_iostream_write_counted_string(iostr, l->string, l->string_len);
-      raptor_iostream_write_byte(iostr, ')');
+      raptor_iostream_counted_string_write("QNAME(", 6, iostr);
+      raptor_iostream_counted_string_write(l->string, l->string_len, iostr);
+      raptor_iostream_write_byte(')', iostr);
       break;
     case RASQAL_LITERAL_INTEGER:
-      raptor_iostream_write_decimal(iostr, l->value.integer);
+      raptor_iostream_decimal_write(l->value.integer, iostr);
       break;
     case RASQAL_LITERAL_BOOLEAN:
     case RASQAL_LITERAL_DOUBLE:
     case RASQAL_LITERAL_FLOAT:
     case RASQAL_LITERAL_DECIMAL:
-      raptor_iostream_write_counted_string(iostr, l->string, l->string_len);
+      raptor_iostream_counted_string_write(l->string, l->string_len, iostr);
       break;
     case RASQAL_LITERAL_VARIABLE:
       rasqal_query_write_sparql_variable(wc, iostr, l->value.variable);
@@ -150,12 +157,13 @@ rasqal_query_write_sparql_literal(sparql_writer_context *wc,
     case RASQAL_LITERAL_DATETIME:
     case RASQAL_LITERAL_XSD_STRING:
     case RASQAL_LITERAL_UDT:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       if(1) {
         raptor_uri* dt_uri;
         
-        raptor_iostream_write_byte(iostr, '"');
-        raptor_iostream_write_string_ntriples(iostr, l->string, l->string_len, '"');
-        raptor_iostream_write_counted_string(iostr, "\"^^", 3);
+        raptor_iostream_write_byte('"', iostr);
+        raptor_string_ntriples_write(l->string, l->string_len, '"', iostr);
+        raptor_iostream_counted_string_write("\"^^", 3, iostr);
         if(l->type == RASQAL_LITERAL_UDT) 
           dt_uri = l->datatype;
         else
@@ -177,15 +185,15 @@ rasqal_query_write_sparql_triple(sparql_writer_context *wc,
                                  raptor_iostream* iostr, rasqal_triple* triple)
 {
   rasqal_query_write_sparql_literal(wc, iostr, triple->subject);
-  raptor_iostream_write_byte(iostr, ' ');
+  raptor_iostream_write_byte(' ', iostr);
   if(triple->predicate->type == RASQAL_LITERAL_URI &&
      raptor_uri_equals(triple->predicate->value.uri, wc->type_uri))
-    raptor_iostream_write_byte(iostr, 'a');
+    raptor_iostream_write_byte('a', iostr);
   else
     rasqal_query_write_sparql_literal(wc, iostr, triple->predicate);
-  raptor_iostream_write_byte(iostr, ' ');
+  raptor_iostream_write_byte(' ', iostr);
   rasqal_query_write_sparql_literal(wc, iostr, triple->object);
-  raptor_iostream_write_counted_string(iostr, " .", 2);
+  raptor_iostream_counted_string_write(" .", 2, iostr);
 }
 
 
@@ -197,7 +205,7 @@ rasqal_query_write_indent(raptor_iostream* iostr, int indent)
 {
   while(indent > 0) {
     int sp = (indent > SPACES_LENGTH) ? SPACES_LENGTH : indent;
-    raptor_iostream_write_bytes(iostr, spaces, sizeof(char), sp);
+    raptor_iostream_write_bytes(spaces, sizeof(char), sp, iostr);
     indent -= sp;
   }
 }
@@ -249,7 +257,18 @@ static const char* const rasqal_sparql_op_labels[RASQAL_EXPR_LAST+1] = {
   "AVG",
   "MIN",
   "MAX",
-  "COALESCE"
+  "COALESCE",
+  "IF",
+  "URI",
+  "IRI",
+  "STRLANG",
+  "STRDT",
+  "BNODE",
+  "GROUP_CONCAT",
+  "SAMPLE",
+  "IN",
+  "NOT IN",
+  "isNUMERIC"
 };
 
 
@@ -266,9 +285,9 @@ rasqal_query_write_sparql_expression_op(sparql_writer_context *wc,
   string = rasqal_sparql_op_labels[(int)op];
   
   if(string)
-    raptor_iostream_write_string(iostr, string);
+    raptor_iostream_string_write(string, iostr);
   else
-    raptor_iostream_write_string(iostr, "NONE");
+    raptor_iostream_string_write("NONE", iostr);
 }
 
 
@@ -296,13 +315,15 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
     case RASQAL_EXPR_REM:
     case RASQAL_EXPR_STR_EQ:
     case RASQAL_EXPR_STR_NEQ:
-      raptor_iostream_write_counted_string(iostr, "( ", 2);
+    case RASQAL_EXPR_STRLANG:
+    case RASQAL_EXPR_STRDT:
+      raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
-      raptor_iostream_write_byte(iostr, ' ');
+      raptor_iostream_write_byte(' ', iostr);
       rasqal_query_write_sparql_expression_op(wc, iostr, e);
-      raptor_iostream_write_byte(iostr, ' ');
+      raptor_iostream_write_byte(' ', iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg2);
-      raptor_iostream_write_counted_string(iostr, " )", 2);
+      raptor_iostream_counted_string_write(" )", 2, iostr);
       break;
 
     case RASQAL_EXPR_BOUND:
@@ -322,33 +343,39 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
     case RASQAL_EXPR_AVG:
     case RASQAL_EXPR_MIN:
     case RASQAL_EXPR_MAX:
+    case RASQAL_EXPR_URI:
+    case RASQAL_EXPR_IRI:
+    case RASQAL_EXPR_BNODE:
+    case RASQAL_EXPR_SAMPLE:
+    case RASQAL_EXPR_ISNUMERIC:
       rasqal_query_write_sparql_expression_op(wc, iostr, e);
-      raptor_iostream_write_counted_string(iostr, "( ", 2);
+      raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
-      raptor_iostream_write_counted_string(iostr, " )", 2);
+      raptor_iostream_counted_string_write(" )", 2, iostr);
       break;
       
     case RASQAL_EXPR_LANGMATCHES:
     case RASQAL_EXPR_REGEX:
+    case RASQAL_EXPR_IF:
       rasqal_query_write_sparql_expression_op(wc, iostr, e);
-      raptor_iostream_write_counted_string(iostr, "( ", 2);
+      raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
-      raptor_iostream_write_counted_string(iostr, ", ", 2);
+      raptor_iostream_counted_string_write(", ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg2);
-      if(e->op == RASQAL_EXPR_REGEX && e->arg3) {
-        raptor_iostream_write_counted_string(iostr, ", ", 2);
+      if((e->op == RASQAL_EXPR_REGEX || e->op == RASQAL_EXPR_IF) && e->arg3) {
+        raptor_iostream_counted_string_write(", ", 2, iostr);
         rasqal_query_write_sparql_expression(wc, iostr, e->arg3);
       }
-      raptor_iostream_write_counted_string(iostr, " )", 2);
+      raptor_iostream_counted_string_write(" )", 2, iostr);
       break;
 
     case RASQAL_EXPR_TILDE:
     case RASQAL_EXPR_BANG:
     case RASQAL_EXPR_UMINUS:
       rasqal_query_write_sparql_expression_op(wc, iostr, e);
-      raptor_iostream_write_counted_string(iostr, "( ", 2);
+      raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
-      raptor_iostream_write_counted_string(iostr, " )", 2);
+      raptor_iostream_counted_string_write(" )", 2, iostr);
       break;
 
     case RASQAL_EXPR_LITERAL:
@@ -356,41 +383,82 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
       break;
 
     case RASQAL_EXPR_FUNCTION:
-      raptor_iostream_write_uri(iostr, e->name);
-      raptor_iostream_write_counted_string(iostr, "( ", 2);
+      raptor_uri_write(e->name, iostr);
+      raptor_iostream_counted_string_write("( ", 2, iostr);
+      if(e->flags & RASQAL_EXPR_FLAG_DISTINCT)
+        raptor_iostream_counted_string_write(" DISTINCT ", 10, iostr);
       count = raptor_sequence_size(e->args);
       for(i = 0; i < count ; i++) {
         rasqal_expression* arg;
         arg = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
         if(i > 0)
-          raptor_iostream_write_counted_string(iostr, " ,", 2);
+          raptor_iostream_counted_string_write(", ", 2, iostr);
         rasqal_query_write_sparql_expression(wc, iostr, arg);
       }
-      raptor_iostream_write_counted_string(iostr, " )", 2);
+      raptor_iostream_counted_string_write(" )", 2, iostr);
       break;
 
     case RASQAL_EXPR_CAST:
-      raptor_iostream_write_uri(iostr, e->name);
-      raptor_iostream_write_counted_string(iostr, "( ", 2);
+      raptor_uri_write(e->name, iostr);
+      raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
-      raptor_iostream_write_counted_string(iostr, " )", 2);
+      raptor_iostream_counted_string_write(" )", 2, iostr);
       break;
 
     case RASQAL_EXPR_VARSTAR:
-      raptor_iostream_write_byte(iostr, '*');
+      raptor_iostream_write_byte('*', iostr);
       break;
       
     case RASQAL_EXPR_COALESCE:
-      raptor_iostream_write_counted_string(iostr, "COALESCE( ", 10);
+      raptor_iostream_counted_string_write("COALESCE( ", 10, iostr);
       count = raptor_sequence_size(e->args);
       for(i = 0; i < count ; i++) {
         rasqal_expression* e2;
         e2 = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
         if(i > 0)
-          raptor_iostream_write_counted_string(iostr, " ,", 2);
+          raptor_iostream_counted_string_write(", ", 2, iostr);
         rasqal_query_write_sparql_expression(wc, iostr, e2);
       }
-      raptor_iostream_write_counted_string(iostr, " )", 2);
+      raptor_iostream_counted_string_write(" )", 2, iostr);
+      break;
+
+    case RASQAL_EXPR_GROUP_CONCAT:
+      raptor_iostream_counted_string_write("GROUP_CONCAT( ", 14, iostr);
+      if(e->flags & RASQAL_EXPR_FLAG_DISTINCT)
+        raptor_iostream_counted_string_write("DISTINCT ", 9, iostr);
+
+      count = raptor_sequence_size(e->args);
+      for(i = 0; i < count ; i++) {
+        rasqal_expression* arg;
+        arg = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
+        if(i > 0)
+          raptor_iostream_counted_string_write(", ", 2, iostr);
+        rasqal_query_write_sparql_expression(wc, iostr, arg);
+      }
+
+      if(e->literal) {
+        raptor_iostream_counted_string_write(" ; SEPARATOR = ", 15, iostr);
+        rasqal_query_write_sparql_literal(wc, iostr, e->literal);
+      }
+      
+      raptor_iostream_counted_string_write(" )", 2, iostr);
+      break;
+
+    case RASQAL_EXPR_IN:
+    case RASQAL_EXPR_NOT_IN:
+      rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
+      raptor_iostream_write_byte(' ', iostr);
+      rasqal_query_write_sparql_expression_op(wc, iostr, e);
+      raptor_iostream_counted_string_write(" (", 2, iostr);
+      count = raptor_sequence_size(e->args);
+      for(i = 0; i < count ; i++) {
+        rasqal_expression* e2;
+        e2 = (rasqal_expression*)raptor_sequence_get_at(e->args, i);
+        if(i > 0)
+          raptor_iostream_counted_string_write(", ", 2, iostr);
+        rasqal_query_write_sparql_expression(wc, iostr, e2);
+      }
+      raptor_iostream_counted_string_write(" )", 2, iostr);
       break;
 
     case RASQAL_EXPR_UNKNOWN:
@@ -419,9 +487,9 @@ rasqal_query_write_sparql_triple_data(sparql_writer_context *wc,
       break;
 
     if(gp > 0)
-      raptor_iostream_write_byte(iostr, '\n');
+      raptor_iostream_write_byte('\n', iostr);
 
-    raptor_iostream_write_counted_string(iostr, "{\n", 2);
+    raptor_iostream_counted_string_write("{\n", 2, iostr);
     indent += 2;
     
     /* look for triples */
@@ -432,7 +500,7 @@ rasqal_query_write_sparql_triple_data(sparql_writer_context *wc,
       
       rasqal_query_write_indent(iostr, indent);
       rasqal_query_write_sparql_triple(wc, iostr, t);
-      raptor_iostream_write_byte(iostr, '\n');
+      raptor_iostream_write_byte('\n', iostr);
       
       triple_index++;
     }
@@ -440,13 +508,31 @@ rasqal_query_write_sparql_triple_data(sparql_writer_context *wc,
     indent -= 2;
     
     rasqal_query_write_indent(iostr, indent);
-    raptor_iostream_write_byte(iostr, '}');
+    raptor_iostream_write_byte('}', iostr);
 
   }
 
 }
 
 
+static int
+rasqal_query_write_sparql_select(sparql_writer_context *wc,
+                                 raptor_iostream *iostr, 
+                                 raptor_sequence* vars_seq)
+{
+  int count = raptor_sequence_size(vars_seq);
+  int i;
+  
+  for(i = 0; i < count; i++) {
+    rasqal_variable* v = (rasqal_variable*)raptor_sequence_get_at(vars_seq, i);
+    raptor_iostream_write_byte(' ', iostr);
+    rasqal_query_write_sparql_variable(wc, iostr, v);
+  }
+
+  return 0;
+}
+
+
 static void
 rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
                                         raptor_iostream* iostr,
@@ -460,14 +546,42 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
   int want_braces = 1;
   
   op = rasqal_graph_pattern_get_operator(gp);
-  
+
+  if(op == RASQAL_GRAPH_PATTERN_OPERATOR_SELECT) {
+    raptor_sequence* vars_seq;
+    rasqal_graph_pattern* where_gp;
+    
+    raptor_iostream_counted_string_write("SELECT", 6, iostr);
+    vars_seq = rasqal_projection_get_variables_sequence(gp->projection);
+    rasqal_query_write_sparql_select(wc, iostr, vars_seq);
+    raptor_iostream_write_byte('\n', iostr);
+    rasqal_query_write_indent(iostr, indent);
+    raptor_iostream_counted_string_write("WHERE ", 6, iostr);
+    where_gp = rasqal_graph_pattern_get_sub_graph_pattern(gp, 0);
+    rasqal_query_write_sparql_graph_pattern(wc, iostr, where_gp, 0, indent);
+    /* FIXME - not implemented: modifiers */
+    return;
+  }
+
   if(op == RASQAL_GRAPH_PATTERN_OPERATOR_LET) {
     /* LAQRS */
-    raptor_iostream_write_counted_string(iostr, "LET (", 5);
+    raptor_iostream_counted_string_write("LET (", 5, iostr);
     rasqal_query_write_sparql_variable(wc, iostr, gp->var);
-    raptor_iostream_write_counted_string(iostr, " := ", 4);
+    raptor_iostream_counted_string_write(" := ", 4, iostr);
     rasqal_query_write_sparql_expression(wc, iostr, gp->filter_expression);
-    raptor_iostream_write_counted_string(iostr, ") .", 3);
+    raptor_iostream_counted_string_write(") .", 3, iostr);
+    return;
+  }
+  
+  if(op == RASQAL_GRAPH_PATTERN_OPERATOR_SERVICE) {
+    rasqal_graph_pattern* service_gp;
+
+    /* LAQRS */
+    raptor_iostream_counted_string_write("SERVICE ", 8, iostr);
+    rasqal_query_write_sparql_literal(wc, iostr, gp->origin);
+    raptor_iostream_counted_string_write(" ", 1, iostr);
+    service_gp = rasqal_graph_pattern_get_sub_graph_pattern(gp, 0);
+    rasqal_query_write_sparql_graph_pattern(wc, iostr, service_gp, 0, indent);
     return;
   }
   
@@ -475,11 +589,11 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
      op == RASQAL_GRAPH_PATTERN_OPERATOR_GRAPH) {
     /* prefix verbs */
     if(op == RASQAL_GRAPH_PATTERN_OPERATOR_OPTIONAL) 
-      raptor_iostream_write_counted_string(iostr, "OPTIONAL ", 9);
+      raptor_iostream_counted_string_write("OPTIONAL ", 9, iostr);
     else {
-      raptor_iostream_write_counted_string(iostr, "GRAPH ", 6);
+      raptor_iostream_counted_string_write("GRAPH ", 6, iostr);
       rasqal_query_write_sparql_literal(wc, iostr, gp->origin);
-      raptor_iostream_write_byte(iostr, ' ');
+      raptor_iostream_write_byte(' ', iostr);
     }
   }
 
@@ -488,7 +602,7 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
 
 
   if(want_braces) {
-    raptor_iostream_write_counted_string(iostr, "{\n", 2);
+    raptor_iostream_counted_string_write("{\n", 2, iostr);
     indent += 2;
   }
 
@@ -500,7 +614,7 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
     
     rasqal_query_write_indent(iostr, indent);
     rasqal_query_write_sparql_triple(wc, iostr, t);
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_write_byte('\n', iostr);
 
     triple_index++;
   }
@@ -510,7 +624,9 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
   seq = rasqal_graph_pattern_get_sub_graph_pattern_sequence(gp);
   if(seq && raptor_sequence_size(seq) > 0) {
     for(gp_index = 0; 1; gp_index++) {
-      rasqal_graph_pattern* sgp = rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
+      rasqal_graph_pattern* sgp;
+
+      sgp = rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
       if(!sgp)
         break;
 
@@ -524,17 +640,17 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
       else {
         if(op == RASQAL_GRAPH_PATTERN_OPERATOR_UNION)
           /* infix verb */
-          raptor_iostream_write_counted_string(iostr, " UNION ", 7);
+          raptor_iostream_counted_string_write(" UNION ", 7, iostr);
         else {
           /* must be prefix verb */
-          raptor_iostream_write_byte(iostr, '\n');
+          raptor_iostream_write_byte('\n', iostr);
           rasqal_query_write_indent(iostr, indent);
         }
       }
       
       rasqal_query_write_sparql_graph_pattern(wc, iostr, sgp, gp_index, indent);
     }
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_write_byte('\n', iostr);
   }
   
 
@@ -554,9 +670,9 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
       expr = rasqal_graph_pattern_get_filter_expression(sgp);
 
       rasqal_query_write_indent(iostr, indent);
-      raptor_iostream_write_counted_string(iostr, "FILTER( ", 8);
+      raptor_iostream_counted_string_write("FILTER( ", 8, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, expr);
-      raptor_iostream_write_counted_string(iostr, " )\n", 3);
+      raptor_iostream_counted_string_write(" )\n", 3, iostr);
     }
   }
   
@@ -565,28 +681,104 @@ rasqal_query_write_sparql_graph_pattern(sparql_writer_context *wc,
     indent -= 2;
 
     rasqal_query_write_indent(iostr, indent);
-    raptor_iostream_write_byte(iostr, '}');
+    raptor_iostream_write_byte('}', iostr);
   }
 
 }
 
 
     
+static void
+rasqal_query_write_data_format_comment(sparql_writer_context* wc,
+                                       raptor_iostream *iostr,
+                                       rasqal_data_graph* dg) 
+{
+  if(dg->format_type || dg->format_name || dg->format_uri) {
+    raptor_iostream_counted_string_write("# format ", 9, iostr);
+    if(dg->format_type) {
+      raptor_iostream_counted_string_write("type ", 5, iostr);
+      raptor_iostream_string_write(dg->format_type, iostr);
+    }
+    if(dg->format_type) {
+      raptor_iostream_counted_string_write("name ", 5, iostr);
+      raptor_iostream_string_write(dg->format_name, iostr);
+    }
+    if(dg->format_type) {
+      raptor_iostream_counted_string_write("uri ", 4, iostr);
+      rasqal_query_write_sparql_uri(wc, iostr, dg->format_uri);
+    }
+  }
+}
+  
+
+
+static int
+rasqal_write_sparql_row(sparql_writer_context* wc,
+                        raptor_iostream* iostr,
+                        rasqal_row* row)
+{
+  int i;
+  
+  raptor_iostream_counted_string_write("( ", 2, iostr);
+  for(i = 0; i < row->size; i++) {
+    rasqal_literal* value = row->values[i];
+    if(i > 0)
+      raptor_iostream_counted_string_write(" ", 1, iostr);
+
+    if(value)
+      rasqal_query_write_sparql_literal(wc, iostr, value);
+    else
+      raptor_iostream_counted_string_write("UNDEF", 5, iostr);
+  }
+  raptor_iostream_counted_string_write(" )", 2, iostr);
+
+  return 0;
+}
+
+
+static int
+rasqal_write_sparql_bindings(sparql_writer_context* wc,
+                             raptor_iostream* iostr,
+                             rasqal_bindings* bindings)
+{
+  raptor_iostream_counted_string_write("BINDINGS", 8, iostr);
+  rasqal_query_write_sparql_select(wc, iostr, bindings->variables);
+  raptor_iostream_counted_string_write(" {\n", 3, iostr);
+
+  if(bindings->rows) {
+    int i;
+  
+    for(i = 0; i < raptor_sequence_size(bindings->rows); i++) {
+      rasqal_row* row;
+      row = raptor_sequence_get_at(bindings->rows, i);
+      raptor_iostream_counted_string_write("  ", 2, iostr);
+      rasqal_write_sparql_row(wc, iostr, row);
+      raptor_iostream_counted_string_write("\n", 1, iostr);
+    }
+  }
+
+  raptor_iostream_counted_string_write("}\n", 2, iostr);
+
+  return 0;
+}
+
+
 int
 rasqal_query_write_sparql_20060406(raptor_iostream *iostr, 
                                    rasqal_query* query, raptor_uri *base_uri)
 {
   int i;
   sparql_writer_context wc;
-#ifndef RAPTOR_V2_AVAILABLE
+#ifndef HAVE_RAPTOR2_API
   const raptor_uri_handler *uri_handler;
   void *uri_context;
 #endif
+  int limit, offset;
 
   wc.world = query->world;
   wc.base_uri = NULL;
 
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
   wc.type_uri = raptor_new_uri_for_rdf_concept(query->world->raptor_world_ptr,
                                                (const unsigned char*)"type");
   wc.nstack = raptor_new_namespaces(query->world->raptor_world_ptr, 1);
@@ -600,9 +792,9 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
 #endif
 
   if(base_uri) {
-    raptor_iostream_write_counted_string(iostr, "BASE ", 5);
+    raptor_iostream_counted_string_write("BASE ", 5, iostr);
     rasqal_query_write_sparql_uri(&wc, iostr, base_uri);
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_write_byte('\n', iostr);
 
     /* from now on all URIs are relative to this */
     wc.base_uri = raptor_uri_copy(base_uri);
@@ -615,12 +807,12 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
     if(!p)
       break;
     
-    raptor_iostream_write_counted_string(iostr, "PREFIX ", 7);
+    raptor_iostream_counted_string_write("PREFIX ", 7, iostr);
     if(p->prefix)
-      raptor_iostream_write_string(iostr, p->prefix);
-    raptor_iostream_write_counted_string(iostr,": ", 2);
+      raptor_iostream_string_write(p->prefix, iostr);
+    raptor_iostream_counted_string_write(": ", 2, iostr);
     rasqal_query_write_sparql_uri(&wc, iostr, p->uri);
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_write_byte('\n', iostr);
 
     /* Use this constructor so we copy a URI directly */
     nspace = raptor_new_namespace_from_uri(wc.nstack, p->prefix, p->uri, i);
@@ -628,15 +820,15 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
   }
 
   if(query->explain)
-    raptor_iostream_write_counted_string(iostr, "EXPLAIN ", 8);
+    raptor_iostream_counted_string_write("EXPLAIN ", 8, iostr);
 
   /* Experimental LAQRS INSERT and DELETE (RASQAL_QUERY_VERB_INSERT
    * and RASQAL_QUERY_VERB_DELETE) are handled in generic code
    * below */
   if(query->verb == RASQAL_QUERY_VERB_INSERT ||
      query->verb == RASQAL_QUERY_VERB_DELETE) {
-    raptor_iostream_write_string(iostr,
-                                 "\n# Writing deprecated LAQRS INSERT/DELETE\n");
+    raptor_iostream_string_write("\n# Writing deprecated LAQRS INSERT/DELETE\n",
+                                 iostr);
   }
 
   
@@ -653,34 +845,34 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
          * INSERT/DELETE DATA { triples } 
          */
         if(update->graph_uri) {
-          raptor_iostream_write_counted_string(iostr, "WITH ", 5);
+          raptor_iostream_counted_string_write("WITH ", 5, iostr);
           rasqal_query_write_sparql_uri(&wc, iostr, update->graph_uri);
-          raptor_iostream_write_byte(iostr, '\n');
+          raptor_iostream_write_byte('\n', iostr);
         }
         if(update->delete_templates) {
-          raptor_iostream_write_counted_string(iostr, "DELETE ", 7);
+          raptor_iostream_counted_string_write("DELETE ", 7, iostr);
           if(update->flags & RASQAL_UPDATE_FLAGS_DATA) 
-            raptor_iostream_write_counted_string(iostr, "DATA ", 5);
+            raptor_iostream_counted_string_write("DATA ", 5, iostr);
           rasqal_query_write_sparql_triple_data(&wc, iostr,
                                                 update->delete_templates,
                                                 0);
-          raptor_iostream_write_byte(iostr, '\n');
+          raptor_iostream_write_byte('\n', iostr);
         }
         if(update->insert_templates) {
-          raptor_iostream_write_counted_string(iostr, "INSERT ", 7);
+          raptor_iostream_counted_string_write("INSERT ", 7, iostr);
           if(update->flags & RASQAL_UPDATE_FLAGS_DATA) 
-            raptor_iostream_write_counted_string(iostr, "DATA ", 5);
+            raptor_iostream_counted_string_write("DATA ", 5, iostr);
           rasqal_query_write_sparql_triple_data(&wc, iostr,
                                                 update->insert_templates,
                                                 0);
-          raptor_iostream_write_byte(iostr, '\n');
+          raptor_iostream_write_byte('\n', iostr);
         }
         if(update->where) {
-          raptor_iostream_write_counted_string(iostr, "WHERE ", 6);
+          raptor_iostream_counted_string_write("WHERE ", 6, iostr);
           rasqal_query_write_sparql_graph_pattern(&wc, iostr,
                                                   update->where,
                                                   -1, 0);
-          raptor_iostream_write_byte(iostr, '\n');
+          raptor_iostream_write_byte('\n', iostr);
         }
       } else {
         /* admin operations:
@@ -689,22 +881,22 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
          * DROP (SILENT) GRAPH graph-uri
          * LOAD GRAPH graph-uri / LOAD doc-uri INTO graph-uri
          */
-        raptor_iostream_write_string(iostr, 
-                                     rasqal_update_type_label(update->type));
+        raptor_iostream_string_write(rasqal_update_type_label(update->type),
+                                     iostr);
         if(update->flags & RASQAL_UPDATE_FLAGS_SILENT)
-          raptor_iostream_write_counted_string(iostr, " SILENT", 7);
+          raptor_iostream_counted_string_write(" SILENT", 7, iostr);
         if(update->document_uri) {
-          raptor_iostream_write_byte(iostr, ' ');
+          raptor_iostream_write_byte(' ', iostr);
           rasqal_query_write_sparql_uri(&wc, iostr, update->document_uri);
         }
         if(update->graph_uri) {
           if(update->type == RASQAL_UPDATE_TYPE_LOAD &&
              update->document_uri)
-            raptor_iostream_write_counted_string(iostr, " INTO ", 6);
+            raptor_iostream_counted_string_write(" INTO ", 6, iostr);
           else
-            raptor_iostream_write_counted_string(iostr, " GRAPH ", 7);
+            raptor_iostream_counted_string_write(" GRAPH ", 7, iostr);
           rasqal_query_write_sparql_uri(&wc, iostr, update->graph_uri);
-          raptor_iostream_write_byte(iostr, '\n');
+          raptor_iostream_write_byte('\n', iostr);
         }
       }
     }
@@ -714,38 +906,31 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
 
 
   if(query->verb != RASQAL_QUERY_VERB_CONSTRUCT)
-    raptor_iostream_write_string(iostr,
-                                 rasqal_query_verb_as_string(query->verb));
+    raptor_iostream_string_write(rasqal_query_verb_as_string(query->verb),
+                                 iostr);
 
   if(query->distinct) {
     if(query->distinct == 1)
-      raptor_iostream_write_counted_string(iostr, " DISTINCT", 9);
+      raptor_iostream_counted_string_write(" DISTINCT", 9, iostr);
     else
-      raptor_iostream_write_counted_string(iostr, " REDUCED", 8);
+      raptor_iostream_counted_string_write(" REDUCED", 8, iostr);
   }
 
   if(query->wildcard)
-    raptor_iostream_write_counted_string(iostr, " *", 2);
+    raptor_iostream_counted_string_write(" *", 2, iostr);
   else if(query->verb == RASQAL_QUERY_VERB_DESCRIBE) {
     raptor_sequence *lit_seq = query->describes;
     int count = raptor_sequence_size(lit_seq);
 
     for(i = 0; i < count; i++) {
       rasqal_literal* l = (rasqal_literal*)raptor_sequence_get_at(lit_seq, i);
-      raptor_iostream_write_byte(iostr, ' ');
+      raptor_iostream_write_byte(' ', iostr);
       rasqal_query_write_sparql_literal(&wc, iostr, l);
     }
   } else if(query->verb == RASQAL_QUERY_VERB_SELECT) {
-    raptor_sequence *var_seq = query->selects;
-    int count = raptor_sequence_size(var_seq);
-
-    for(i = 0; i < count; i++) {
-      rasqal_variable* v = (rasqal_variable*)raptor_sequence_get_at(var_seq, i);
-      raptor_iostream_write_byte(iostr, ' ');
-      rasqal_query_write_sparql_variable(&wc, iostr, v);
-    }
+    rasqal_query_write_sparql_select(&wc, iostr, query->selects);
   }
-  raptor_iostream_write_byte(iostr, '\n');
+  raptor_iostream_write_byte('\n', iostr);
 
   if(query->data_graphs) {
     for(i = 0; 1; i++) {
@@ -756,9 +941,10 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
       if(dg->flags & RASQAL_DATA_GRAPH_NAMED)
         continue;
       
-      raptor_iostream_write_counted_string(iostr, "FROM ", 5);
+      rasqal_query_write_data_format_comment(&wc, iostr, dg);
+      raptor_iostream_counted_string_write("FROM ", 5, iostr);
       rasqal_query_write_sparql_uri(&wc, iostr, dg->uri);
-      raptor_iostream_write_counted_string(iostr, "\n", 1);
+      raptor_iostream_counted_string_write("\n", 1, iostr);
     }
     
     for(i = 0; 1; i++) {
@@ -769,76 +955,95 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
       if(!(dg->flags & RASQAL_DATA_GRAPH_NAMED))
         continue;
       
-      raptor_iostream_write_counted_string(iostr, "FROM NAMED ", 11);
+      rasqal_query_write_data_format_comment(&wc, iostr, dg);
+      raptor_iostream_counted_string_write("FROM NAMED ", 11, iostr);
       rasqal_query_write_sparql_uri(&wc, iostr, dg->name_uri);
-      raptor_iostream_write_byte(iostr, '\n');
+      raptor_iostream_write_byte('\n', iostr);
     }
     
   }
 
   if(query->constructs) {
-    raptor_iostream_write_string(iostr, "CONSTRUCT {\n");
+    raptor_iostream_string_write("CONSTRUCT {\n", iostr);
     for(i = 0; 1; i++) {
       rasqal_triple* t = rasqal_query_get_construct_triple(query, i);
       if(!t)
         break;
 
-      raptor_iostream_write_counted_string(iostr, "  ", 2);
+      raptor_iostream_counted_string_write("  ", 2, iostr);
       rasqal_query_write_sparql_triple(&wc, iostr, t);
-      raptor_iostream_write_byte(iostr, '\n');
+      raptor_iostream_write_byte('\n', iostr);
     }
-    raptor_iostream_write_counted_string(iostr, "}\n", 2);
+    raptor_iostream_counted_string_write("}\n", 2, iostr);
   }
   if(query->query_graph_pattern) {
-    raptor_iostream_write_counted_string(iostr, "WHERE ", 6);
+    raptor_iostream_counted_string_write("WHERE ", 6, iostr);
     rasqal_query_write_sparql_graph_pattern(&wc, iostr,
                                             query->query_graph_pattern, 
                                             -1, 0);
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_write_byte('\n', iostr);
   }
 
-  if(query->group_conditions_sequence) {
-    raptor_iostream_write_counted_string(iostr, "GROUP BY ", 9);
+  if(rasqal_query_get_group_conditions_sequence(query)) {
+    raptor_iostream_counted_string_write("GROUP BY ", 9, iostr);
     for(i = 0; 1; i++) {
       rasqal_expression* expr = rasqal_query_get_group_condition(query, i);
       if(!expr)
         break;
 
       if(i > 0)
-        raptor_iostream_write_byte(iostr, ' ');
+        raptor_iostream_write_byte(' ', iostr);
       rasqal_query_write_sparql_expression(&wc, iostr, expr);
     }
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_write_byte('\n', iostr);
   }
 
-  if(query->order_conditions_sequence) {
-    raptor_iostream_write_counted_string(iostr, "ORDER BY ", 9);
+  if(rasqal_query_get_having_conditions_sequence(query)) {
+    raptor_iostream_counted_string_write("HAVING ", 7, iostr);
+    for(i = 0; 1; i++) {
+      rasqal_expression* expr = rasqal_query_get_having_condition(query, i);
+      if(!expr)
+        break;
+
+      if(i > 0)
+        raptor_iostream_write_byte(' ', iostr);
+      rasqal_query_write_sparql_expression(&wc, iostr, expr);
+    }
+    raptor_iostream_write_byte('\n', iostr);
+  }
+
+  if(rasqal_query_get_order_conditions_sequence(query)) {
+    raptor_iostream_counted_string_write("ORDER BY ", 9, iostr);
     for(i = 0; 1; i++) {
       rasqal_expression* expr = rasqal_query_get_order_condition(query, i);
       if(!expr)
         break;
 
       if(i > 0)
-        raptor_iostream_write_byte(iostr, ' ');
+        raptor_iostream_write_byte(' ', iostr);
       rasqal_query_write_sparql_expression(&wc, iostr, expr);
     }
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_write_byte('\n', iostr);
   }
 
-  if(query->limit >= 0 || query->offset >= 0) {
-    if(query->limit >= 0) {
-      raptor_iostream_write_counted_string(iostr, "LIMIT ", 7);
-      raptor_iostream_write_decimal(iostr, query->limit);
+  limit = rasqal_query_get_limit(query);
+  offset = rasqal_query_get_offset(query);
+  if(limit >= 0 || offset >= 0) {
+    if(limit >= 0) {
+      raptor_iostream_counted_string_write("LIMIT ", 7, iostr);
+      raptor_iostream_decimal_write(limit, iostr);
     }
-    if(query->offset >= 0) {
-      if(query->limit)
-        raptor_iostream_write_byte(iostr, ' ');
-      raptor_iostream_write_counted_string(iostr, "OFFSET ", 8);
-      raptor_iostream_write_decimal(iostr, query->offset);
+    if(offset >= 0) {
+      if(limit)
+        raptor_iostream_write_byte(' ', iostr);
+      raptor_iostream_counted_string_write("OFFSET ", 8, iostr);
+      raptor_iostream_decimal_write(offset, iostr);
     }
-    raptor_iostream_write_byte(iostr, '\n');
+    raptor_iostream_write_byte('\n', iostr);
   }
 
+  if(query->bindings)
+    rasqal_write_sparql_bindings(&wc, iostr, query->bindings);
 
   tidy:
   raptor_free_uri(wc.type_uri);
