@@ -81,6 +81,7 @@ static int rasqal_raptor_triple_present(rasqal_triples_source *rts, void *user_d
 static void rasqal_raptor_free_triples_source(void *user_data);
 
 
+#ifndef HAVE_RAPTOR2_API
 static raptor_uri* 
 ordinal_as_uri(rasqal_world* world, int ordinal) 
 {
@@ -97,18 +98,15 @@ ordinal_as_uri(rasqal_world* world, int ordinal)
     return NULL;
   
   sprintf((char*)buffer, "%s_%d", raptor_rdf_namespace_uri, ordinal);
-#ifdef RAPTOR_V2_AVAILABLE
   uri = raptor_new_uri(world->raptor_world_ptr, buffer);
-#else
-  uri = raptor_new_uri(buffer);
-#endif
   RASQAL_FREE(cstring, buffer);
 
   return uri;
 }
+#endif
 
 
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
 static rasqal_triple*
 raptor_statement_as_rasqal_triple(rasqal_world* world,
                                   const raptor_statement *statement)
@@ -120,9 +118,9 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
   /* subject */
   if(statement->subject->type == RAPTOR_TERM_TYPE_BLANK) {
-    len = strlen((char*)statement->subject->value.blank);
+    len = strlen((char*)statement->subject->value.blank.string);
     new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
-    strncpy((char*)new_str, (const char*)statement->subject->value.blank, len + 1);
+    memcpy(new_str, statement->subject->value.blank.string, len + 1);
     s = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_str);
   } else {
     uri = raptor_uri_copy((raptor_uri*)statement->subject->value.uri);
@@ -139,13 +137,12 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
     len = strlen((char*)statement->object->value.literal.string);
     new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
-    strncpy((char*)new_str, (const char*)statement->object->value.literal.string, len + 1);
+    memcpy(new_str, statement->object->value.literal.string, len + 1);
 
     if(statement->object->value.literal.language) {
       len = strlen((const char*)statement->object->value.literal.language);
       language = (char*)RASQAL_MALLOC(cstring, len + 1);
-      strncpy(language, (const char*)statement->object->value.literal.language,
-              len + 1);
+      memcpy(language, statement->object->value.literal.language, len + 1);
     }
 
     if(statement->object->value.literal.datatype)
@@ -155,9 +152,9 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
     o = rasqal_new_string_literal(world, new_str, language, uri, NULL);
   } else if(statement->object->type == RAPTOR_TERM_TYPE_BLANK) {
-    len = strlen((char*)statement->object->value.blank);
+    len = strlen((char*)statement->object->value.blank.string);
     new_str = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
-    strncpy((char*)new_str, (const char*)statement->object->value.blank, len + 1);
+    memcpy(new_str, statement->object->value.blank.string, len + 1);
     o = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_str);
   } else {
     uri = raptor_uri_copy((raptor_uri*)statement->object->value.uri);
@@ -166,7 +163,7 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
   return rasqal_new_triple(s, p, o);
 }
-#else /* ifdef RAPTOR_V2_AVAILABLE */
+#else /* ifdef HAVE_RAPTOR_API */
 static rasqal_triple*
 raptor_statement_as_rasqal_triple(rasqal_world* world,
                                   const raptor_statement *statement)
@@ -179,7 +176,7 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
     blank_len = strlen((char*)statement->subject);
     new_blank = (unsigned char*)RASQAL_MALLOC(cstring, blank_len + 1);
-    strncpy((char*)new_blank, (const char*)statement->subject, blank_len + 1);
+    memcpy(new_blank, statement->subject, blank_len + 1);
     s = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_blank);
   } else if(statement->subject_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
     raptor_uri* uri;
@@ -217,21 +214,20 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
     literal_len = strlen((char*)statement->object);
     string = (unsigned char*)RASQAL_MALLOC(cstring, literal_len + 1);
-    strncpy((char*)string, (const char*)statement->object, literal_len + 1);
+    memcpy(string, statement->object, literal_len + 1);
 
     if(statement->object_literal_language) {
       size_t language_len;
       language_len = strlen((const char*)statement->object_literal_language);
       language = (char*)RASQAL_MALLOC(cstring, language_len + 1);
-      strncpy(language, (const char*)statement->object_literal_language,
-              language_len + 1);
+      memcpy(language, statement->object_literal_language, language_len + 1);
     }
 
     if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL) {
       const unsigned char* dt_uri_string;
 
       dt_uri_string = (const unsigned char*)raptor_xml_literal_datatype_uri_string;
-      uri = raptor_new_uri(dt_uri_string);
+      uri = raptor_new_uri(world->raptor_world_ptr, dt_uri_string);
     } else if(statement->object_literal_datatype) {
       raptor_uri *dt_uri = (raptor_uri*)statement->object_literal_datatype;
 
@@ -245,7 +241,7 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
     blank_len = strlen(blank);
     new_blank = (unsigned char*)RASQAL_MALLOC(cstring, blank_len + 1);
-    strncpy((char*)new_blank, (const char*)blank, blank_len + 1);
+    memcpy(new_blank, blank, blank_len + 1);
     o = rasqal_new_simple_literal(world, RASQAL_LITERAL_BLANK, new_blank);
   } else if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
     raptor_uri* uri = ordinal_as_uri(world, *((int*)statement->object));
@@ -261,12 +257,12 @@ raptor_statement_as_rasqal_triple(rasqal_world* world,
 
   return rasqal_new_triple(s, p, o);
 }
-#endif /* ifdef RAPTOR_V2_AVAILABLE ... else */
+#endif /* ifdef HAVE_RAPTOR2_API ... else */
 
 
 static void
 rasqal_raptor_statement_handler(void *user_data,
-#ifndef RAPTOR_V2_AVAILABLE
+#ifndef HAVE_RAPTOR2_API
                                 const
 #endif
                                 raptor_statement *statement)
@@ -297,7 +293,7 @@ rasqal_raptor_statement_handler(void *user_data,
 }
 
 
-#ifndef RAPTOR_V2_AVAILABLE
+#ifndef HAVE_RAPTOR2_API
 static void
 rasqal_raptor_error_handler(void *user_data, 
                             raptor_locator* locator, const char *message)
@@ -308,14 +304,14 @@ rasqal_raptor_error_handler(void *user_data,
   query->failed = 1;
 
   if(locator &&
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
      (locator_len = raptor_locator_format(NULL, 0, locator)) > 0
 #else
      (locator_len = raptor_format_locator(NULL, 0, locator)) > 0
 #endif
     ) {
     char *buffer = (char*)RASQAL_MALLOC(cstring, locator_len + 1);
-#ifdef RAPTOR_V2_AVAILABLE
+#ifdef HAVE_RAPTOR2_API
     raptor_locator_format(buffer, locator_len, locator);
 #else
     raptor_format_locator(buffer, locator_len, locator);
@@ -335,7 +331,10 @@ rasqal_raptor_error_handler(void *user_data,
 
 static unsigned char*
 rasqal_raptor_generate_id_handler(void *user_data,
+#ifdef HAVE_RAPTOR2_API
+#else
                                   raptor_genid_type type,
+#endif
                                   unsigned char *user_bnodeid) 
 {
   rasqal_raptor_triples_source_user_data* rtsc;
@@ -349,11 +348,10 @@ rasqal_raptor_generate_id_handler(void *user_data,
     mapped_id = (unsigned char*)RASQAL_MALLOC(cstring, 
                                               rtsc->mapped_id_base_len + 1 + 
                                               user_bnodeid_len + 1);
-    strncpy((char*)mapped_id, (const char*)rtsc->mapped_id_base, 
-            rtsc->mapped_id_base_len);
+    memcpy(mapped_id, rtsc->mapped_id_base, rtsc->mapped_id_base_len);
     mapped_id[rtsc->mapped_id_base_len] = '_';
-    strncpy((char*)(mapped_id+rtsc->mapped_id_base_len + 1),
-            (const char*)user_bnodeid, user_bnodeid_len + 1);
+    memcpy(mapped_id + rtsc->mapped_id_base_len + 1,
+           user_bnodeid, user_bnodeid_len + 1);
 
     raptor_free_memory(user_bnodeid);
     return mapped_id;
@@ -364,10 +362,63 @@ rasqal_raptor_generate_id_handler(void *user_data,
 
 
 static int
-rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
-                                 void *factory_user_data,
-                                 void *user_data,
-                                 rasqal_triples_source *rts)
+rasqal_raptor_support_feature(void *user_data,
+                              rasqal_triples_source_feature feature)
+{
+  switch(feature) {
+    case RASQAL_TRIPLES_SOURCE_FEATURE_IOSTREAM_DATA_GRAPH:
+      return 1;
+      break;
+      
+    default:
+    case RASQAL_TRIPLES_SOURCE_FEATURE_NONE:
+      return 0;
+  }
+}
+
+
+#ifdef HAVE_RAPTOR2_API
+#else
+
+#define RASQAL_RAPTOR_READ_BUFFER_SIZE 4096
+
+/* Based on raptor_parser_parse_iostream() from Raptor 1.9.0
+ * backported to the Raptor V1 API
+ */
+static int
+rasqal_raptor_parse_iostream(raptor_parser* rdf_parser, raptor_iostream *iostr,
+                             raptor_uri *base_uri)
+{
+  /* Read buffer */
+  unsigned char buffer[RASQAL_RAPTOR_READ_BUFFER_SIZE + 1];
+  int rc = 0;
+
+  rc = raptor_start_parse(rdf_parser, base_uri);
+  if(rc)
+    return rc;
+  
+  while(!raptor_iostream_read_eof(iostr)) {
+    int len = raptor_iostream_read_bytes(buffer, 1,
+                                         RASQAL_RAPTOR_READ_BUFFER_SIZE,
+                                         iostr);
+    int is_end = (len < RASQAL_RAPTOR_READ_BUFFER_SIZE);
+
+    rc = raptor_parse_chunk(rdf_parser, buffer, len, is_end);
+    if(rc || is_end)
+      break;
+  }
+  
+  return rc;
+}
+#endif
+
+
+static int
+rasqal_raptor_init_triples_source(rasqal_query* rdf_query,
+                                  void *factory_user_data,
+                                  void *user_data,
+                                  rasqal_triples_source *rts,
+                                  rasqal_triples_error_handler handler)
 {
   rasqal_raptor_triples_source_user_data* rtsc;
   raptor_parser *parser;
@@ -375,20 +426,27 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
 
   rtsc = (rasqal_raptor_triples_source_user_data*)user_data;
 
-  if(!rdf_query->data_graphs)
+  if(!rdf_query->data_graphs) {
+    if(handler)
+      handler(rdf_query, /* locator */ NULL, "Query has no data graphs.");
     return -1;  /* no data */
+  }
 
   /* Max API version this triples source generates */
-  rts->version = 1;
+  rts->version = 2;
   
   rts->init_triples_match = rasqal_raptor_init_triples_match;
   rts->triple_present = rasqal_raptor_triple_present;
   rts->free_triples_source = rasqal_raptor_free_triples_source;
+  rts->support_feature = rasqal_raptor_support_feature;
 
   rtsc->sources_count = raptor_sequence_size(rdf_query->data_graphs);
   /* no default triple source possible */
-  if(!rtsc->sources_count)
+  if(!rtsc->sources_count) {
+    if(handler)
+      handler(rdf_query, /* locator */ NULL, "Query has no data graphs.");
     return -1;  /* no data */
+  }
 
   rtsc->source_literals = (rasqal_literal**)RASQAL_CALLOC(rasqal_literal_ptr,
                                                           rtsc->sources_count,
@@ -398,22 +456,26 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
 
   for(i = 0; i < rtsc->sources_count; i++) {
     rasqal_data_graph *dg;
-    raptor_uri* uri;
+    raptor_uri* uri = NULL;
     raptor_uri* name_uri;
     int free_name_uri = 0;
-
+    const char* parser_name;
+    raptor_iostream* iostr = NULL;
+    
     dg = (rasqal_data_graph*)raptor_sequence_get_at(rdf_query->data_graphs, i);
     uri = dg->uri;
     name_uri = dg->name_uri;
+    iostr = dg->iostr;
 
     rtsc->source_index = i;
-    rtsc->source_uri = raptor_uri_copy(uri);
+    if(uri)
+      rtsc->source_uri = raptor_uri_copy(uri);
 
     if(name_uri)
       rtsc->source_literals[i] = rasqal_new_uri_literal(rdf_query->world, 
                                                         raptor_uri_copy(name_uri)
                                                         );
-    else {
+    else if(uri) {
       name_uri = raptor_uri_copy(uri);
       free_name_uri = 1;
     }
@@ -423,13 +485,26 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
                                                   i);
     rtsc->mapped_id_base_len = strlen((const char*)rtsc->mapped_id_base);
 
-#ifdef RAPTOR_V2_AVAILABLE
-    parser = raptor_new_parser(rdf_query->world->raptor_world_ptr, "guess");
+    parser_name = dg->format_name;
+    if(parser_name) {
+      if(!raptor_world_is_parser_name(rdf_query->world->raptor_world_ptr,
+                                      parser_name)) {
+        handler(rdf_query, /* locator */ NULL,
+                "Invalid data graph parser name ignored");
+        parser_name = NULL;
+      }
+    }
+    if(!parser_name)
+      parser_name = "guess";
+    
+#ifdef HAVE_RAPTOR2_API
+    parser = raptor_new_parser(rdf_query->world->raptor_world_ptr, parser_name);
     raptor_parser_set_statement_handler(parser, rtsc, rasqal_raptor_statement_handler);
-    raptor_parser_set_generate_id_handler(parser, rtsc,
-                                          rasqal_raptor_generate_id_handler);
+    raptor_world_set_generate_bnodeid_handler(rdf_query->world->raptor_world_ptr,
+                                              rtsc,
+                                              rasqal_raptor_generate_id_handler);
 #else
-    parser = raptor_new_parser("guess");
+    parser = raptor_new_parser(parser_name);
     raptor_set_statement_handler(parser, rtsc, rasqal_raptor_statement_handler);
     raptor_set_error_handler(parser, rdf_query, rasqal_raptor_error_handler);
     raptor_set_generate_id_handler(parser, rtsc,
@@ -442,7 +517,20 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
                          rdf_query->features[RASQAL_FEATURE_NO_NET]);
 #endif
 
-    raptor_parse_uri(parser, uri, name_uri);
+    if(iostr) {
+#ifdef HAVE_RAPTOR2_API
+      raptor_parser_parse_iostream(parser, iostr, dg->base_uri);
+#else
+      rasqal_raptor_parse_iostream(parser, iostr, dg->base_uri);
+#endif
+    } else {
+#ifdef HAVE_RAPTOR2_API
+      raptor_parser_parse_uri(parser, uri, name_uri);
+#else
+      raptor_parse_uri(parser, uri, name_uri);
+#endif
+    }
+    
     raptor_free_parser(parser);
 
     raptor_free_uri(rtsc->source_uri);
@@ -464,6 +552,18 @@ rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
 }
 
 
+static int
+rasqal_raptor_new_triples_source(rasqal_query* rdf_query,
+                                 void *factory_user_data,
+                                 void *user_data,
+                                 rasqal_triples_source *rts)
+{
+  return rasqal_raptor_init_triples_source(rdf_query, factory_user_data,
+                                           user_data, rts,
+                                           rasqal_triples_source_error_handler);
+}
+
+
 /**
  * rasqal_raptor_triple_match:
  * @world: rasqal_world object
@@ -599,9 +699,12 @@ rasqal_raptor_free_triples_source(void *user_data)
 static int
 rasqal_raptor_register_triples_source_factory(rasqal_triples_source_factory *factory) 
 {
-  factory->version = 1;
+  factory->version = 2;
   factory->user_data_size = sizeof(rasqal_raptor_triples_source_user_data);
+  /* V1 */
   factory->new_triples_source = rasqal_raptor_new_triples_source;
+  /* V2 */
+  factory->init_triples_source = rasqal_raptor_init_triples_source;
 
   return 0;
 }
