@@ -88,18 +88,22 @@ rasqal_new_rowsource_from_handler(rasqal_world* world,
   rowsource->handler = handler;
   rowsource->flags = flags;
 
-  rowsource->size= -1;
+  rowsource->size = -1;
 
+  rowsource->generate_group = 0;
+  
   if(vars_table)
     rowsource->vars_table = rasqal_new_variables_table_from_variables_table(vars_table);
   else
     rowsource->vars_table = NULL;
 
   /* no free method here - the variables are owned by rowsource->vars_table */
-#ifdef RAPTOR_V2_AVAILABLE
-  rowsource->variables_sequence = raptor_new_sequence(NULL, (raptor_data_print_handler*)rasqal_variable_print);
+#ifdef HAVE_RAPTOR2_API
+  rowsource->variables_sequence = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                                      (raptor_data_print_handler)rasqal_variable_print);
 #else
-  rowsource->variables_sequence = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+  rowsource->variables_sequence = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
+                                                      (raptor_sequence_print_handler*)rasqal_variable_print);
 #endif
   if(!rowsource->variables_sequence) {
     rasqal_free_rowsource(rowsource);
@@ -120,7 +124,7 @@ rasqal_new_rowsource_from_handler(rasqal_world* world,
  * rasqal_free_rowsource:
  * @rowsource: rowsource object
  *
- * Destructor - destroy an rowsource.
+ * INTERNAL - Destructor - destroy an rowsource.
  **/
 void
 rasqal_free_rowsource(rasqal_rowsource *rowsource)
@@ -162,6 +166,7 @@ rasqal_rowsource_add_variable(rasqal_rowsource *rowsource, rasqal_variable* v)
   if(offset >= 0)
     return offset;
 
+  v = rasqal_new_variable_from_variable(v);
   if(raptor_sequence_push(rowsource->variables_sequence, v))
     return -1;
 
@@ -199,7 +204,8 @@ rasqal_rowsource_ensure_variables(rasqal_rowsource *rowsource)
 
 #ifdef RASQAL_DEBUG
   if(!rc) {
-    RASQAL_DEBUG2("rowsource %p ", rowsource);
+    RASQAL_DEBUG3("%s rowsource %p header: ", rowsource->handler->name,
+                  rowsource);
     rasqal_rowsource_print_header(rowsource, stderr);
   }
 #endif
@@ -245,12 +251,17 @@ rasqal_rowsource_read_row(rasqal_rowsource *rowsource)
   
   if(!row)
     rowsource->finished = 1;
-  else
+  else {
     rowsource->count++;
 
+    /* Generate a group around all rows if there are no groups returned */
+    if(rowsource->generate_group && row->group_id < 0)
+      row->group_id = 0;
+  }
+
 #ifdef RASQAL_DEBUG
-  RASQAL_DEBUG3("rowsource %p read %s row : ", rowsource, 
-                rowsource->handler->name);
+  RASQAL_DEBUG3("%s rowsource %p read row:  ", rowsource->handler->name, 
+                rowsource);
   if(row)
     rasqal_row_print(row, stderr);
   else
@@ -298,20 +309,37 @@ rasqal_rowsource_read_all_rows(rasqal_rowsource *rowsource)
 
   if(rowsource->handler->read_all_rows) {
     seq = rowsource->handler->read_all_rows(rowsource, rowsource->user_data);
-    if(!seq)
-#ifdef RAPTOR_V2_AVAILABLE
-      seq = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_row,
-                                (raptor_data_print_handler*)rasqal_row_print);
+    if(!seq) {
+#ifdef HAVE_RAPTOR2_API
+      seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row,
+                                (raptor_data_print_handler)rasqal_row_print);
 #else
       seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row,
                                 (raptor_sequence_print_handler*)rasqal_row_print);
 #endif
+    }
+
+    if(seq && rowsource->generate_group) {
+      int i;
+      rasqal_row* row;
+      
+      /* Set group for all rows if there are no groups returned */
+
+      for(i = 0; (row = raptor_sequence_get_at(seq, i)); i++) {
+        /* if first row has a group ID, end */
+        if(!i && row->group_id >= 0)
+          break;
+        
+        row->group_id = 0;
+      }
+    }
+
     return seq;
   }
 
-#ifdef RAPTOR_V2_AVAILABLE
-  seq = raptor_new_sequence((raptor_data_free_handler*)rasqal_free_row,
-                            (raptor_data_print_handler*)rasqal_row_print);
+#ifdef HAVE_RAPTOR2_API
+  seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row,
+                            (raptor_data_print_handler)rasqal_row_print);
 #else
   seq = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_row,
                             (raptor_sequence_print_handler*)rasqal_row_print);
@@ -323,6 +351,11 @@ rasqal_rowsource_read_all_rows(rasqal_rowsource *rowsource)
     rasqal_row* row = rasqal_rowsource_read_row(rowsource);
     if(!row)
       break;
+
+    /* Generate a group around all rows if there are no groups returned */
+    if(rowsource->generate_group && row->group_id < 0)
+      row->group_id = 0;
+
     raptor_sequence_push(seq, row);
   }
   
@@ -585,6 +618,14 @@ rasqal_rowsource_set_origin(rasqal_rowsource* rowsource,
 }
 
 
+int
+rasqal_rowsource_request_grouping(rasqal_rowsource* rowsource)
+{
+  rowsource->generate_group = 1;
+
+  return 0;
+}
+
 
 #define SPACES_LENGTH 80
 static const char spaces[SPACES_LENGTH+1] = "                                                                                ";
@@ -594,7 +635,7 @@ rasqal_rowsource_write_indent(raptor_iostream *iostr, int indent)
 {
   while(indent > 0) {
     int sp = (indent > SPACES_LENGTH) ? SPACES_LENGTH : indent;
-    raptor_iostream_write_bytes(iostr, spaces, sizeof(char), sp);
+    raptor_iostream_write_bytes(spaces, sizeof(char), sp, iostr);
     indent -= sp;
   }
 }
@@ -612,8 +653,8 @@ rasqal_rowsource_write_internal(rasqal_rowsource *rowsource,
   
   indent_delta = strlen(rs_name);
 
-  raptor_iostream_write_counted_string(iostr, rs_name, indent_delta);
-  raptor_iostream_write_counted_string(iostr, "(\n", 2);
+  raptor_iostream_counted_string_write(rs_name, indent_delta, iostr);
+  raptor_iostream_counted_string_write("(\n", 2, iostr);
   indent_delta++;
   
   indent += indent_delta;
@@ -624,18 +665,18 @@ rasqal_rowsource_write_internal(rasqal_rowsource *rowsource,
       (inner_rowsource = rasqal_rowsource_get_inner_rowsource(rowsource, offset));
       offset++) {
       if(arg_count) {
-        raptor_iostream_write_counted_string(iostr, " ,\n", 3);
+        raptor_iostream_counted_string_write(" ,\n", 3, iostr);
         rasqal_rowsource_write_indent(iostr, indent);
       }
       rasqal_rowsource_write_internal(inner_rowsource, iostr, indent);
       arg_count++;
   }
 
-  raptor_iostream_write_byte(iostr, '\n');
+  raptor_iostream_write_byte('\n', iostr);
   indent-= indent_delta;
 
   rasqal_rowsource_write_indent(iostr, indent);
-  raptor_iostream_write_byte(iostr, ')');
+  raptor_iostream_write_byte(')', iostr);
 
   return 0;
 }
@@ -663,11 +704,7 @@ rasqal_rowsource_print(rasqal_rowsource *rowsource, FILE* fh)
 {
   raptor_iostream *iostr;
 
-#ifdef RAPTOR_V2_AVAILABLE
   iostr = raptor_new_iostream_to_file_handle(rowsource->world->raptor_world_ptr, fh);
-#else
-  iostr = raptor_new_iostream_to_file_handle(fh);
-#endif
   rasqal_rowsource_write(rowsource, iostr);
   raptor_free_iostream(iostr);
 }
