@@ -73,12 +73,6 @@ typedef struct
   /* number of variables used in variables table  */
   int size;
   
-  /* bound-in array index[index into vars table] = column */
-  int *bound_in;
-
-  /* number of variables in bound_in[] array - may be more than 'size' */
-  int bound_in_size;
-  
   /* preserve bindings when all rows are finished - for optional mostly */
   int preserve_on_all_finished;
 
@@ -90,25 +84,27 @@ typedef struct
 static int
 rasqal_triples_rowsource_init(rasqal_rowsource* rowsource, void *user_data)
 {
+  rasqal_query *query = rowsource->query;
   rasqal_triples_rowsource_context *con;
   int column;
-  int *bound_in = NULL;
   int rc = 0;
   int size;
   int i;
   
   con = (rasqal_triples_rowsource_context*)user_data;
 
-  bound_in = con->bound_in;
-  size = con->bound_in_size;
+  size = rasqal_variables_table_get_total_variables_count(query->vars_table);
   
   /* Construct the ordered projection of the variables set by these triples */
   con->size = 0;
   for(i = 0; i < size; i++) {
-    column = bound_in[i];
+    rasqal_variable *v;
+    v = rasqal_variables_table_get(rowsource->vars_table, i);
+    
+    column = rasqal_query_get_bound_in_column_for_variable(query, v);
+
     if(column >= con->start_column && column <= con->end_column) {
-      rasqal_variable *v;
-      v = rasqal_variables_table_get(rowsource->vars_table, i);
+      v = rasqal_new_variable_from_variable(v);
       if(raptor_sequence_push(rowsource->variables_sequence, v))
         return -1;
       con->size++;
@@ -129,15 +125,15 @@ rasqal_triples_rowsource_init(rasqal_rowsource* rowsource, void *user_data)
     t = (rasqal_triple*)raptor_sequence_get_at(con->triples, column);
     
     if((v = rasqal_literal_as_variable(t->subject)) &&
-       bound_in[v->offset] == column)
+       rasqal_query_get_bound_in_column_for_variable(query, v) == column)
       m->parts = (rasqal_triple_parts)(m->parts | RASQAL_TRIPLE_SUBJECT);
     
     if((v = rasqal_literal_as_variable(t->predicate)) &&
-       bound_in[v->offset] == column)
+       rasqal_query_get_bound_in_column_for_variable(query, v) == column)
       m->parts = (rasqal_triple_parts)(m->parts | RASQAL_TRIPLE_PREDICATE);
     
     if((v = rasqal_literal_as_variable(t->object)) &&
-       bound_in[v->offset] == column)
+       rasqal_query_get_bound_in_column_for_variable(query, v) == column)
       m->parts = (rasqal_triple_parts)(m->parts | RASQAL_TRIPLE_OBJECT);
 
     RASQAL_DEBUG4("triple pattern column %d has parts %s (%d)\n", column,
@@ -180,9 +176,6 @@ rasqal_triples_rowsource_finish(rasqal_rowsource* rowsource, void *user_data)
     RASQAL_FREE(rasqal_triple_meta, con->triple_meta);
   }
 
-  if(con->bound_in)
-    RASQAL_FREE(rasqal_bound_in, con->bound_in);
-
   if(con->origin)
     rasqal_free_literal(con->origin);
 
@@ -435,7 +428,7 @@ static const rasqal_rowsource_handler rasqal_triples_rowsource_handler = {
 };
 
 
-/*
+/**
  * rasqal_new_triples_rowsource:
  * @world: world object
  * @query: query object
@@ -447,8 +440,6 @@ static const rasqal_rowsource_handler rasqal_triples_rowsource_handler = {
  *
  * INTERNAL - create a new triples rowsource
  *
- * NOTE @bound_in becomes owned by the new rowsource
- *
  * Return value: new triples rowsource or NULL on failure
  */
 rasqal_rowsource*
@@ -456,35 +447,26 @@ rasqal_new_triples_rowsource(rasqal_world *world,
                              rasqal_query *query,
                              rasqal_triples_source* triples_source,
                              raptor_sequence* triples,
-                             int start_column, int end_column,
-                             int *bound_in, int bound_in_size)
+                             int start_column, int end_column)
 {
   rasqal_triples_rowsource_context *con;
   int flags = 0;
 
-  if(!world || !query || !triples_source) {
-    RASQAL_FREE(rasqal_bound_in, bound_in);
+  if(!world || !query || !triples_source)
     return NULL;
-  }
 
-  if(!triples) {
-    RASQAL_FREE(rasqal_bound_in, bound_in);
+  if(!triples)
     return rasqal_new_empty_rowsource(world, query);
-  }
   
   con = (rasqal_triples_rowsource_context*)RASQAL_CALLOC(rasqal_triples_rowsource_context, 1, sizeof(rasqal_triples_rowsource_context));
-  if(!con) {
-    RASQAL_FREE(rasqal_bound_in, bound_in);
+  if(!con)
     return NULL;
-  }
 
   con->triples_source = triples_source;
   con->triples = triples;
   con->start_column = start_column;
   con->end_column = end_column;
   con->column = -1;
-  con->bound_in = bound_in;
-  con->bound_in_size = bound_in_size;
 
   con->triples_count = con->end_column - con->start_column + 1;
 
@@ -545,8 +527,6 @@ main(int argc, char *argv[])
 #define OBJECT_STRING "object"
   raptor_uri* s_uri = NULL;
   raptor_uri* p_uri = NULL;
-  int size;
-  int *bound_in;
   const char *data_file;
   
   if((data_file = getenv("NT_DATA_FILE"))) {
@@ -573,11 +553,7 @@ main(int argc, char *argv[])
   raptor_free_memory(data_string);
   
   uri_string = raptor_uri_filename_to_uri_string("");
-#ifdef RAPTOR_V2_AVAILABLE
   base_uri = raptor_new_uri(world->raptor_world_ptr, uri_string);
-#else
-  base_uri = raptor_new_uri(uri_string);
-#endif
   raptor_free_memory(uri_string);
 
   query = rasqal_new_query(world, query_language_name, NULL);
@@ -606,19 +582,8 @@ main(int argc, char *argv[])
 
   triples_source = rasqal_new_triples_source(query);
   
-  size = rasqal_variables_table_get_named_variables_count(query->vars_table);
-  bound_in = rasqal_query_triples_build_bound_in(query, size,
-                                                 start_column,
-                                                 end_column);
-  if(!bound_in) {
-    fprintf(stderr, "%s: failed to create bound_in\n", program);
-    failures++;
-    goto tidy;
-  }
-
   rowsource = rasqal_new_triples_rowsource(world, query, triples_source,
-                                           triples, start_column, end_column,
-                                           bound_in, size);
+                                           triples, start_column, end_column);
   if(!rowsource) {
     fprintf(stderr, "%s: failed to create triples rowsource\n", program);
     failures++;
@@ -641,13 +606,8 @@ main(int argc, char *argv[])
     fputc('\n', stderr);
   #endif
 
-#ifdef RAPTOR_V2_AVAILABLE
     s_uri = raptor_new_uri(world->raptor_world_ptr, SUBJECT_URI_STRING);
     p_uri = raptor_new_uri(world->raptor_world_ptr, PREDICATE_URI_STRING);
-#else
-    s_uri = raptor_new_uri(SUBJECT_URI_STRING);
-    p_uri = raptor_new_uri(PREDICATE_URI_STRING);
-#endif
     
     s = row->values[0];
     if(!s ||
