@@ -580,14 +580,14 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   198,   198,   202,   231,   236,   241,   254,   258,   265,
-     269,   274,   283,   288,   293,   306,   310,   314,   318,   329,
-     334,   339,   343,   347,   355,   360,   365,   370,   375,   383,
-     387,   393,   398,   404,   408,   412,   416,   420,   426,   430,
-     434,   440,   444,   448,   452,   456,   462,   469,   473,   477,
-     483,   487,   491,   495,   501,   505,   509,   515,   519,   523,
-     528,   532,   538,   542,   546,   552,   556,   562,   568,   574,
-     578,   582,   586,   590,   594,   597,   604,   609
+       0,   198,   198,   202,   237,   242,   247,   261,   265,   272,
+     276,   281,   290,   295,   300,   313,   317,   321,   325,   336,
+     341,   346,   350,   354,   362,   367,   372,   377,   382,   390,
+     394,   400,   405,   411,   415,   419,   423,   427,   433,   437,
+     441,   447,   451,   455,   459,   463,   469,   476,   480,   484,
+     490,   494,   498,   502,   508,   512,   516,   522,   526,   530,
+     535,   539,   545,   549,   553,   559,   563,   569,   576,   582,
+     586,   590,   594,   598,   602,   605,   612,   617
 };
 #endif
 
@@ -1265,11 +1265,10 @@ yysyntax_error (char *yyresult, int yystate, int yychar)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yydestruct (void *YYPARSE_PARAM, const char *yymsg, int yytype, YYSTYPE *yyvaluep)
+yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep)
 #else
 static void
-yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
-    void *YYPARSE_PARAM;
+yydestruct (yymsg, yytype, yyvaluep)
     const char *yymsg;
     int yytype;
     YYSTYPE *yyvaluep;
@@ -1293,7 +1292,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1297 "rdql_parser.c"
+#line 1296 "rdql_parser.c"
 	break;
       case 46: /* "\"string literal\"" */
 
@@ -1305,7 +1304,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1309 "rdql_parser.c"
+#line 1308 "rdql_parser.c"
 	break;
       case 47: /* "\"integer literal\"" */
 
@@ -1317,7 +1316,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1321 "rdql_parser.c"
+#line 1320 "rdql_parser.c"
 	break;
       case 48: /* "\"pattern literal\"" */
 
@@ -1329,7 +1328,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1333 "rdql_parser.c"
+#line 1332 "rdql_parser.c"
 	break;
       case 49: /* "\"boolean literal\"" */
 
@@ -1341,7 +1340,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1345 "rdql_parser.c"
+#line 1344 "rdql_parser.c"
 	break;
       case 50: /* "\"null\"" */
 
@@ -1353,7 +1352,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1357 "rdql_parser.c"
+#line 1356 "rdql_parser.c"
 	break;
       case 51: /* "\"URI literal\"" */
 
@@ -1365,7 +1364,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1369 "rdql_parser.c"
+#line 1368 "rdql_parser.c"
 	break;
       case 52: /* "\"QName literal\"" */
 
@@ -1377,7 +1376,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1381 "rdql_parser.c"
+#line 1380 "rdql_parser.c"
 	break;
       case 53: /* "\"identifier\"" */
 
@@ -1389,7 +1388,7 @@ yydestruct (YYPARSE_PARAM, yymsg, yytype, yyvaluep)
 };
 
 /* Line 1000 of yacc.c  */
-#line 1393 "rdql_parser.c"
+#line 1392 "rdql_parser.c"
 	break;
 
       default:
@@ -1702,8 +1701,14 @@ yyreduce:
     int i;
     
     for(i=0; i < raptor_sequence_size((yyvsp[(3) - (7)].seq)); i++) {
-      raptor_uri* uri=(raptor_uri*)raptor_sequence_get_at((yyvsp[(3) - (7)].seq), i);
-      rasqal_query_add_data_graph((rasqal_query*)rq, uri, NULL, RASQAL_DATA_GRAPH_BACKGROUND);
+      raptor_uri* uri = (raptor_uri*)raptor_sequence_get_at((yyvsp[(3) - (7)].seq), i);
+      rasqal_data_graph* dg;
+      
+      dg = rasqal_new_data_graph_from_uri(((rasqal_query*)rq)->world, uri, NULL,
+                                          RASQAL_DATA_GRAPH_BACKGROUND,
+                                          NULL, NULL, NULL);
+      if(dg)
+        rasqal_query_add_data_graph2((rasqal_query*)rq, dg);
     }
     raptor_free_sequence((yyvsp[(3) - (7)].seq));
   }
@@ -1725,7 +1730,7 @@ yyreduce:
   case 4:
 
 /* Line 1455 of yacc.c  */
-#line 232 "./rdql_parser.y"
+#line 238 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (3)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(3) - (3)].variable));
@@ -1735,7 +1740,7 @@ yyreduce:
   case 5:
 
 /* Line 1455 of yacc.c  */
-#line 237 "./rdql_parser.y"
+#line 243 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (2)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(2) - (2)].variable));
@@ -1745,13 +1750,14 @@ yyreduce:
   case 6:
 
 /* Line 1455 of yacc.c  */
-#line 242 "./rdql_parser.y"
+#line 248 "./rdql_parser.y"
     {
-  /* The variables are freed from the rasqal_query field variables */
-#ifdef RAPTOR_V2_AVAILABLE
-  (yyval.seq) = raptor_new_sequence(NULL, (raptor_data_print_handler*)rasqal_variable_print);
+#ifdef HAVE_RAPTOR2_API
+  (yyval.seq) = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                           (raptor_data_print_handler)rasqal_variable_print);
 #else
-  (yyval.seq) = raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+  (yyval.seq) = raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_variable,
+                           (raptor_sequence_print_handler*)rasqal_variable_print);
 #endif
   raptor_sequence_push((yyval.seq), (yyvsp[(1) - (1)].variable));
 }
@@ -1760,7 +1766,7 @@ yyreduce:
   case 7:
 
 /* Line 1455 of yacc.c  */
-#line 255 "./rdql_parser.y"
+#line 262 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (1)].seq);
 }
@@ -1769,7 +1775,7 @@ yyreduce:
   case 8:
 
 /* Line 1455 of yacc.c  */
-#line 259 "./rdql_parser.y"
+#line 266 "./rdql_parser.y"
     {
   (yyval.seq)=NULL;
   ((rasqal_query*)rq)->wildcard=1;
@@ -1779,7 +1785,7 @@ yyreduce:
   case 9:
 
 /* Line 1455 of yacc.c  */
-#line 266 "./rdql_parser.y"
+#line 273 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(2) - (2)].seq);
 }
@@ -1788,7 +1794,7 @@ yyreduce:
   case 10:
 
 /* Line 1455 of yacc.c  */
-#line 270 "./rdql_parser.y"
+#line 277 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(2) - (2)].seq);
 }
@@ -1797,7 +1803,7 @@ yyreduce:
   case 11:
 
 /* Line 1455 of yacc.c  */
-#line 274 "./rdql_parser.y"
+#line 281 "./rdql_parser.y"
     {
   (yyval.seq)=NULL;
 }
@@ -1806,7 +1812,7 @@ yyreduce:
   case 12:
 
 /* Line 1455 of yacc.c  */
-#line 284 "./rdql_parser.y"
+#line 291 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (3)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(3) - (3)].triple));
@@ -1816,7 +1822,7 @@ yyreduce:
   case 13:
 
 /* Line 1455 of yacc.c  */
-#line 289 "./rdql_parser.y"
+#line 296 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (2)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(2) - (2)].triple));
@@ -1826,7 +1832,7 @@ yyreduce:
   case 14:
 
 /* Line 1455 of yacc.c  */
-#line 294 "./rdql_parser.y"
+#line 301 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->triples;
   raptor_sequence_push((yyval.seq), (yyvsp[(1) - (1)].triple));
@@ -1836,7 +1842,7 @@ yyreduce:
   case 15:
 
 /* Line 1455 of yacc.c  */
-#line 307 "./rdql_parser.y"
+#line 314 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (7)].literal), (yyvsp[(4) - (7)].literal), (yyvsp[(6) - (7)].literal));
 }
@@ -1845,7 +1851,7 @@ yyreduce:
   case 16:
 
 /* Line 1455 of yacc.c  */
-#line 311 "./rdql_parser.y"
+#line 318 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (6)].literal), (yyvsp[(3) - (6)].literal), (yyvsp[(5) - (6)].literal));
 }
@@ -1854,7 +1860,7 @@ yyreduce:
   case 17:
 
 /* Line 1455 of yacc.c  */
-#line 315 "./rdql_parser.y"
+#line 322 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (6)].literal), (yyvsp[(4) - (6)].literal), (yyvsp[(5) - (6)].literal));
 }
@@ -1863,7 +1869,7 @@ yyreduce:
   case 18:
 
 /* Line 1455 of yacc.c  */
-#line 319 "./rdql_parser.y"
+#line 326 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (5)].literal), (yyvsp[(3) - (5)].literal), (yyvsp[(4) - (5)].literal));
 }
@@ -1872,7 +1878,7 @@ yyreduce:
   case 19:
 
 /* Line 1455 of yacc.c  */
-#line 330 "./rdql_parser.y"
+#line 337 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(2) - (2)].expr);
 }
@@ -1881,7 +1887,7 @@ yyreduce:
   case 20:
 
 /* Line 1455 of yacc.c  */
-#line 334 "./rdql_parser.y"
+#line 341 "./rdql_parser.y"
     {
   (yyval.expr)=NULL;
 }
@@ -1890,7 +1896,7 @@ yyreduce:
   case 21:
 
 /* Line 1455 of yacc.c  */
-#line 340 "./rdql_parser.y"
+#line 347 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_AND, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -1899,7 +1905,7 @@ yyreduce:
   case 22:
 
 /* Line 1455 of yacc.c  */
-#line 344 "./rdql_parser.y"
+#line 351 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_AND, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -1908,7 +1914,7 @@ yyreduce:
   case 23:
 
 /* Line 1455 of yacc.c  */
-#line 348 "./rdql_parser.y"
+#line 355 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -1917,7 +1923,7 @@ yyreduce:
   case 24:
 
 /* Line 1455 of yacc.c  */
-#line 356 "./rdql_parser.y"
+#line 363 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(2) - (2)].seq);
 }
@@ -1926,7 +1932,7 @@ yyreduce:
   case 25:
 
 /* Line 1455 of yacc.c  */
-#line 360 "./rdql_parser.y"
+#line 367 "./rdql_parser.y"
     {
   (yyval.seq)=NULL;
 }
@@ -1935,7 +1941,7 @@ yyreduce:
   case 26:
 
 /* Line 1455 of yacc.c  */
-#line 366 "./rdql_parser.y"
+#line 373 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->prefixes;
   raptor_sequence_shift((yyval.seq), rasqal_new_prefix(((rasqal_query*)rq)->world, (yyvsp[(1) - (5)].name), (yyvsp[(3) - (5)].uri)));
@@ -1945,7 +1951,7 @@ yyreduce:
   case 27:
 
 /* Line 1455 of yacc.c  */
-#line 371 "./rdql_parser.y"
+#line 378 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->prefixes;
   raptor_sequence_shift((yyval.seq), rasqal_new_prefix(((rasqal_query*)rq)->world, (yyvsp[(1) - (4)].name), (yyvsp[(3) - (4)].uri)));
@@ -1955,7 +1961,7 @@ yyreduce:
   case 28:
 
 /* Line 1455 of yacc.c  */
-#line 376 "./rdql_parser.y"
+#line 383 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->prefixes;
   raptor_sequence_push((yyval.seq), rasqal_new_prefix(((rasqal_query*)rq)->world, (yyvsp[(1) - (3)].name), (yyvsp[(3) - (3)].uri)));
@@ -1965,7 +1971,7 @@ yyreduce:
   case 29:
 
 /* Line 1455 of yacc.c  */
-#line 384 "./rdql_parser.y"
+#line 391 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_OR, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -1974,7 +1980,7 @@ yyreduce:
   case 30:
 
 /* Line 1455 of yacc.c  */
-#line 388 "./rdql_parser.y"
+#line 395 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -1983,7 +1989,7 @@ yyreduce:
   case 31:
 
 /* Line 1455 of yacc.c  */
-#line 394 "./rdql_parser.y"
+#line 401 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_AND, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 ;
@@ -1993,7 +1999,7 @@ yyreduce:
   case 32:
 
 /* Line 1455 of yacc.c  */
-#line 399 "./rdql_parser.y"
+#line 406 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2002,7 +2008,7 @@ yyreduce:
   case 33:
 
 /* Line 1455 of yacc.c  */
-#line 405 "./rdql_parser.y"
+#line 412 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_EQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2011,7 +2017,7 @@ yyreduce:
   case 34:
 
 /* Line 1455 of yacc.c  */
-#line 409 "./rdql_parser.y"
+#line 416 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_NEQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2020,7 +2026,7 @@ yyreduce:
   case 35:
 
 /* Line 1455 of yacc.c  */
-#line 413 "./rdql_parser.y"
+#line 420 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_string_op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_MATCH, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].literal));
 }
@@ -2029,7 +2035,7 @@ yyreduce:
   case 36:
 
 /* Line 1455 of yacc.c  */
-#line 417 "./rdql_parser.y"
+#line 424 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_string_op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_NMATCH, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].literal));
 }
@@ -2038,7 +2044,7 @@ yyreduce:
   case 37:
 
 /* Line 1455 of yacc.c  */
-#line 421 "./rdql_parser.y"
+#line 428 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2047,7 +2053,7 @@ yyreduce:
   case 38:
 
 /* Line 1455 of yacc.c  */
-#line 427 "./rdql_parser.y"
+#line 434 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_EQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2056,7 +2062,7 @@ yyreduce:
   case 39:
 
 /* Line 1455 of yacc.c  */
-#line 431 "./rdql_parser.y"
+#line 438 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_NEQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2065,7 +2071,7 @@ yyreduce:
   case 40:
 
 /* Line 1455 of yacc.c  */
-#line 435 "./rdql_parser.y"
+#line 442 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2074,7 +2080,7 @@ yyreduce:
   case 41:
 
 /* Line 1455 of yacc.c  */
-#line 441 "./rdql_parser.y"
+#line 448 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_LT, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2083,7 +2089,7 @@ yyreduce:
   case 42:
 
 /* Line 1455 of yacc.c  */
-#line 445 "./rdql_parser.y"
+#line 452 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_GT, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2092,7 +2098,7 @@ yyreduce:
   case 43:
 
 /* Line 1455 of yacc.c  */
-#line 449 "./rdql_parser.y"
+#line 456 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_LE, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2101,7 +2107,7 @@ yyreduce:
   case 44:
 
 /* Line 1455 of yacc.c  */
-#line 453 "./rdql_parser.y"
+#line 460 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_GE, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2110,7 +2116,7 @@ yyreduce:
   case 45:
 
 /* Line 1455 of yacc.c  */
-#line 457 "./rdql_parser.y"
+#line 464 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2119,7 +2125,7 @@ yyreduce:
   case 46:
 
 /* Line 1455 of yacc.c  */
-#line 463 "./rdql_parser.y"
+#line 470 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2128,7 +2134,7 @@ yyreduce:
   case 47:
 
 /* Line 1455 of yacc.c  */
-#line 470 "./rdql_parser.y"
+#line 477 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_PLUS, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2137,7 +2143,7 @@ yyreduce:
   case 48:
 
 /* Line 1455 of yacc.c  */
-#line 474 "./rdql_parser.y"
+#line 481 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_MINUS, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2146,7 +2152,7 @@ yyreduce:
   case 49:
 
 /* Line 1455 of yacc.c  */
-#line 478 "./rdql_parser.y"
+#line 485 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2155,7 +2161,7 @@ yyreduce:
   case 50:
 
 /* Line 1455 of yacc.c  */
-#line 484 "./rdql_parser.y"
+#line 491 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STAR, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2164,7 +2170,7 @@ yyreduce:
   case 51:
 
 /* Line 1455 of yacc.c  */
-#line 488 "./rdql_parser.y"
+#line 495 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_SLASH, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2173,7 +2179,7 @@ yyreduce:
   case 52:
 
 /* Line 1455 of yacc.c  */
-#line 492 "./rdql_parser.y"
+#line 499 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_REM, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2182,7 +2188,7 @@ yyreduce:
   case 53:
 
 /* Line 1455 of yacc.c  */
-#line 496 "./rdql_parser.y"
+#line 503 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2191,7 +2197,7 @@ yyreduce:
   case 54:
 
 /* Line 1455 of yacc.c  */
-#line 502 "./rdql_parser.y"
+#line 509 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(2) - (2)].expr);
 }
@@ -2200,7 +2206,7 @@ yyreduce:
   case 55:
 
 /* Line 1455 of yacc.c  */
-#line 506 "./rdql_parser.y"
+#line 513 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_1op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_UMINUS, (yyvsp[(2) - (2)].expr));
 }
@@ -2209,7 +2215,7 @@ yyreduce:
   case 56:
 
 /* Line 1455 of yacc.c  */
-#line 510 "./rdql_parser.y"
+#line 517 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2218,7 +2224,7 @@ yyreduce:
   case 57:
 
 /* Line 1455 of yacc.c  */
-#line 516 "./rdql_parser.y"
+#line 523 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_1op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_TILDE, (yyvsp[(2) - (2)].expr));
 }
@@ -2227,7 +2233,7 @@ yyreduce:
   case 58:
 
 /* Line 1455 of yacc.c  */
-#line 520 "./rdql_parser.y"
+#line 527 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_1op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_BANG, (yyvsp[(2) - (2)].expr));
 }
@@ -2236,7 +2242,7 @@ yyreduce:
   case 59:
 
 /* Line 1455 of yacc.c  */
-#line 524 "./rdql_parser.y"
+#line 531 "./rdql_parser.y"
     {
   rasqal_literal *l=rasqal_new_variable_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].variable));
   (yyval.expr)=rasqal_new_literal_expression(((rasqal_query*)rq)->world, l);
@@ -2246,7 +2252,7 @@ yyreduce:
   case 60:
 
 /* Line 1455 of yacc.c  */
-#line 529 "./rdql_parser.y"
+#line 536 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_literal_expression(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].literal));
 }
@@ -2255,7 +2261,7 @@ yyreduce:
   case 61:
 
 /* Line 1455 of yacc.c  */
-#line 533 "./rdql_parser.y"
+#line 540 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(2) - (3)].expr);
 }
@@ -2264,7 +2270,7 @@ yyreduce:
   case 62:
 
 /* Line 1455 of yacc.c  */
-#line 539 "./rdql_parser.y"
+#line 546 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_variable_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].variable));
 }
@@ -2273,7 +2279,7 @@ yyreduce:
   case 63:
 
 /* Line 1455 of yacc.c  */
-#line 543 "./rdql_parser.y"
+#line 550 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_uri_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].uri));
 }
@@ -2282,7 +2288,7 @@ yyreduce:
   case 64:
 
 /* Line 1455 of yacc.c  */
-#line 547 "./rdql_parser.y"
+#line 554 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_simple_literal(((rasqal_query*)rq)->world, RASQAL_LITERAL_QNAME, (yyvsp[(1) - (1)].name));
 }
@@ -2291,7 +2297,7 @@ yyreduce:
   case 65:
 
 /* Line 1455 of yacc.c  */
-#line 553 "./rdql_parser.y"
+#line 560 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_variable_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].variable));
 }
@@ -2300,7 +2306,7 @@ yyreduce:
   case 66:
 
 /* Line 1455 of yacc.c  */
-#line 557 "./rdql_parser.y"
+#line 564 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2309,16 +2315,17 @@ yyreduce:
   case 67:
 
 /* Line 1455 of yacc.c  */
-#line 563 "./rdql_parser.y"
+#line 570 "./rdql_parser.y"
     {
-  (yyval.variable)=rasqal_new_variable((rasqal_query*)rq, (yyvsp[(2) - (2)].name), NULL);
+  (yyval.variable) = rasqal_variables_table_add(((rasqal_query*)rq)->vars_table,
+                                  RASQAL_VARIABLE_TYPE_NORMAL, (yyvsp[(2) - (2)].name), NULL);
 }
     break;
 
   case 68:
 
 /* Line 1455 of yacc.c  */
-#line 569 "./rdql_parser.y"
+#line 577 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2327,7 +2334,7 @@ yyreduce:
   case 69:
 
 /* Line 1455 of yacc.c  */
-#line 575 "./rdql_parser.y"
+#line 583 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_uri_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].uri));
 }
@@ -2336,7 +2343,7 @@ yyreduce:
   case 70:
 
 /* Line 1455 of yacc.c  */
-#line 579 "./rdql_parser.y"
+#line 587 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2345,7 +2352,7 @@ yyreduce:
   case 71:
 
 /* Line 1455 of yacc.c  */
-#line 583 "./rdql_parser.y"
+#line 591 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2354,7 +2361,7 @@ yyreduce:
   case 72:
 
 /* Line 1455 of yacc.c  */
-#line 587 "./rdql_parser.y"
+#line 595 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2363,7 +2370,7 @@ yyreduce:
   case 73:
 
 /* Line 1455 of yacc.c  */
-#line 591 "./rdql_parser.y"
+#line 599 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2372,7 +2379,7 @@ yyreduce:
   case 74:
 
 /* Line 1455 of yacc.c  */
-#line 595 "./rdql_parser.y"
+#line 603 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2381,7 +2388,7 @@ yyreduce:
   case 75:
 
 /* Line 1455 of yacc.c  */
-#line 598 "./rdql_parser.y"
+#line 606 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_simple_literal(((rasqal_query*)rq)->world, RASQAL_LITERAL_QNAME, (yyvsp[(1) - (1)].name));
 }
@@ -2390,7 +2397,7 @@ yyreduce:
   case 76:
 
 /* Line 1455 of yacc.c  */
-#line 605 "./rdql_parser.y"
+#line 613 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(3) - (3)].seq);
   raptor_sequence_shift((yyval.seq), (yyvsp[(1) - (3)].uri));
@@ -2400,11 +2407,11 @@ yyreduce:
   case 77:
 
 /* Line 1455 of yacc.c  */
-#line 610 "./rdql_parser.y"
+#line 618 "./rdql_parser.y"
     {
-#ifdef RAPTOR_V2_AVAILABLE
-  (yyval.seq) = raptor_new_sequence((raptor_data_free_handler*)raptor_free_uri,
-                           (raptor_data_print_handler*)raptor_uri_print);
+#ifdef HAVE_RAPTOR2_API
+  (yyval.seq) = raptor_new_sequence((raptor_data_free_handler)raptor_free_uri,
+                           (raptor_data_print_handler)raptor_uri_print);
 #else
   (yyval.seq) = raptor_new_sequence((raptor_sequence_free_handler*)raptor_free_uri,
                            (raptor_sequence_print_handler*)raptor_uri_print);
@@ -2416,7 +2423,7 @@ yyreduce:
 
 
 /* Line 1455 of yacc.c  */
-#line 2420 "rdql_parser.c"
+#line 2427 "rdql_parser.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2502,7 +2509,7 @@ yyerrlab:
 	}
       else
 	{
-	  yydestruct (YYPARSE_PARAM, "Error: discarding",
+	  yydestruct ("Error: discarding",
 		      yytoken, &yylval);
 	  yychar = YYEMPTY;
 	}
@@ -2558,7 +2565,7 @@ yyerrlab1:
 	YYABORT;
 
 
-      yydestruct (YYPARSE_PARAM, "Error: popping",
+      yydestruct ("Error: popping",
 		  yystos[yystate], yyvsp);
       YYPOPSTACK (1);
       yystate = *yyssp;
@@ -2601,7 +2608,7 @@ yyexhaustedlab:
 
 yyreturn:
   if (yychar != YYEMPTY)
-     yydestruct (YYPARSE_PARAM, "Cleanup: discarding lookahead",
+     yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval);
   /* Do not reclaim the symbols of the rule which action triggered
      this YYABORT or YYACCEPT.  */
@@ -2609,7 +2616,7 @@ yyreturn:
   YY_STACK_PRINT (yyss, yyssp);
   while (yyssp != yyss)
     {
-      yydestruct (YYPARSE_PARAM, "Cleanup: popping",
+      yydestruct ("Cleanup: popping",
 		  yystos[*yyssp], yyvsp);
       YYPOPSTACK (1);
     }
@@ -2628,7 +2635,7 @@ yyreturn:
 
 
 /* Line 1675 of yacc.c  */
-#line 622 "./rdql_parser.y"
+#line 630 "./rdql_parser.y"
 
 
 
@@ -2833,11 +2840,7 @@ rdql_syntax_warning(rasqal_query *rq, const char *message, ...)
 
   va_start(arguments, message);
   rasqal_log_error_varargs(rq->world,
-#ifdef RAPTOR_V2_AVAILABLE
                            RAPTOR_LOG_LEVEL_WARN,
-#else
-                           RAPTOR_LOG_LEVEL_WARNING,
-#endif
                            &rq->locator,
                            message, arguments);
   va_end(arguments);
@@ -2846,27 +2849,44 @@ rdql_syntax_warning(rasqal_query *rq, const char *message, ...)
 }
 
 
-static void
+static const char* const rdql_names[2] = { "rdql", NULL};
+
+#define RDQL_TYPES_COUNT 1
+static const raptor_type_q rdql_types[RDQL_TYPES_COUNT + 1] = {
+  { "application/x-rdql", 18, 10}, 
+  { NULL, 0, 0}
+};
+
+static int
 rasqal_rdql_query_language_register_factory(rasqal_query_language_factory *factory)
 {
+  int rc = 0;
+
+  factory->desc.names = rdql_names;
+
+  factory->desc.mime_types = rdql_types;
+  factory->desc.mime_types_count = RDQL_TYPES_COUNT;
+
+  factory->desc.label = "RDF Data Query Language (RDQL)";
+
+  /* http://www.w3.org/Submission/2004/SUBM-RDQL-20040109/ */
+  factory->desc.uri_string = "http://jena.hpl.hp.com/2003/07/query/RDQL";
+
   factory->context_length = sizeof(rasqal_rdql_query_language);
 
   factory->init      = rasqal_rdql_query_language_init;
   factory->terminate = rasqal_rdql_query_language_terminate;
   factory->prepare   = rasqal_rdql_query_language_prepare;
+
+  return rc;
 }
 
 
 int
-rasqal_init_query_language_rdql(rasqal_world* world) {
-  /* http://www.w3.org/Submission/2004/SUBM-RDQL-20040109/ */
-
-  return rasqal_query_language_register_factory(world,
-                                                "rdql", 
-                                                "RDF Data Query Language (RDQL)",
-                                                NULL,
-                                                (const unsigned char*)"http://jena.hpl.hp.com/2003/07/query/RDQL",
-                                                &rasqal_rdql_query_language_register_factory);
+rasqal_init_query_language_rdql(rasqal_world* world)
+{
+  return !rasqal_query_language_register_factory(world,
+                                                 &rasqal_rdql_query_language_register_factory);
 }
 
 
@@ -2934,11 +2954,7 @@ main(int argc, char *argv[])
   query=rasqal_new_query(world, "rdql", NULL);
 
   uri_string=raptor_uri_filename_to_uri_string(filename);
-#ifdef RAPTOR_V2_AVAILABLE
   base_uri = raptor_new_uri(world->raptor_world_ptr, uri_string);
-#else
-  base_uri = raptor_new_uri(uri_string);
-#endif
   
   rc=rasqal_query_prepare(query, (const unsigned char*)query_string, base_uri);
 
